package data.scripts;

import com.fs.starfarer.api.BaseModPlugin;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.StarSystemAPI;
import com.fs.starfarer.api.impl.campaign.CoreScript;
import com.fs.starfarer.api.impl.campaign.shared.PersonBountyEventData;
import java.util.List;
import org.apache.log4j.Logger;
import org.json.JSONObject;
import org.lazywizard.lazylib.MathUtils;
import com.fs.starfarer.api.impl.campaign.shared.SharedData;
import java.util.ArrayList;

public class EZFactionModPlugin extends BaseModPlugin
{
    @Override
    public void onNewGameAfterProcGen()
    {
		List<FactionAPI> factions = Global.getSector().getAllFactions();
		for(FactionAPI faction : factions){
			FactionAPI foundFaction = Global.getSector().getFaction("derelict");
			int minPlanets = 0;
			int maxPlanets = 0;
			float minRangeFromCore = 0f;
			float maxRangeFromCore = 0f;
			int minMarketSize = 0;
			int maxMarketSize = 0;
			boolean onlyStations = false;
			boolean randomPirates = false;
			float initialAngle = 0f;

			try {
				JSONObject json = Global.getSettings().loadJSON("data/ezfaction/" + faction.getId() + ".ezfaction");
				if(json != null){

					boolean hatesAll = false;
					foundFaction = faction;
					Logger log = Global.getLogger(CoreScript.class);
					log.info("Found EZ Faction File for: " + foundFaction.getDisplayName());

					minPlanets = json.getInt("minPlanets");
					maxPlanets = json.getInt("maxPlanets");
					minRangeFromCore = (float) json.getDouble("minRangeFromCore");
					maxRangeFromCore = (float) json.getDouble("maxRangeFromCore");
					minMarketSize = json.getInt("minMarketSize");
					maxMarketSize = json.getInt("maxMarketSize");
					if(json.has("onlyStations")){
						onlyStations = json.getBoolean("onlyStations");
					}
					if(json.has("randomPirates")){
						randomPirates = json.getBoolean("randomPirates");
					}
					if(json.has("initialAngle")){
						initialAngle = (float) json.getDouble("initialAngle");
					}
					
					if(json.has("hatedFactions")){
						for(int i = 0; i < json.getJSONArray("hatedFactions").length(); i++){
							String factionName = json.getJSONArray("hatedFactions").getString(i);
							if(factionName.equalsIgnoreCase("everybody")){
								hatesAll = true;
								for(FactionAPI thisFaction : factions){
									if(!thisFaction.getId().equalsIgnoreCase("derelict") && thisFaction != foundFaction){
										foundFaction.adjustRelationship(thisFaction.getId(), -50f);
										thisFaction.adjustRelationship(foundFaction.getId(), -50f);
									}
								}
							} else {
								FactionAPI enemy = Global.getSector().getFaction(factionName);
								foundFaction.adjustRelationship(enemy.getId(), -50f);
								enemy.adjustRelationship(foundFaction.getId(), -50f);
							}
						}
					}
					if(json.has("lovedFactions")){
						for(int i = 0; i < json.getJSONArray("lovedFactions").length(); i++){
							String factionName = json.getJSONArray("lovedFactions").getString(i);
							if(factionName.equalsIgnoreCase("everybody")){
								for(FactionAPI thisFaction : factions){
									if(!thisFaction.getId().equalsIgnoreCase("derelict") && thisFaction != foundFaction){
										foundFaction.adjustRelationship(thisFaction.getId(), 50f);
										thisFaction.adjustRelationship(foundFaction.getId(), 50f);
									}
								}
							} else {
								FactionAPI friend = Global.getSector().getFaction(factionName);
								foundFaction.adjustRelationship(friend.getId(), 50f);
								friend.adjustRelationship(foundFaction.getId(), 50f);
							}
						}
					}
					if(!hatesAll){
						//Adds in Bounties to this Faction.
						PersonBountyEventData data = SharedData.getData().getPersonBountyEventData();
						data.addParticipatingFaction(foundFaction.getId());	
					}
				}
			}  catch (Throwable ex) {
				//Keep going because we don't actually care about IO access errors, etc.
				//Global.getLogger(EZFactionModPlugin.class).log(Level.ERROR,"ERROR: couldn't find the EZ Faction directory, etc.", ex);
				continue;
			}
			if(!foundFaction.getId().equalsIgnoreCase("derelict")){
				int colonies = MathUtils.getRandomNumberInRange(minPlanets, maxPlanets);
				float coreDist = MathUtils.getRandomNumberInRange(minRangeFromCore, maxRangeFromCore);
				EZFactionStartup.EZFactionStartup(foundFaction, colonies, coreDist, minMarketSize, maxMarketSize, onlyStations, randomPirates, initialAngle);
			}
		}
    }

    @Override
    public void onApplicationLoad()
    {
        boolean hasLazyLib = Global.getSettings().getModManager().isModEnabled("lw_lazylib");
        if (!hasLazyLib)
        {
            throw new RuntimeException("EZ Faction requires LazyLib.");
        }
    }
}
