package data.scripts;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.ModSpecAPI;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.PlanetAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.StarSystemAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI.SurveyLevel;
import com.fs.starfarer.api.impl.campaign.CoreScript;
import com.fs.starfarer.api.impl.campaign.ids.Commodities;
import com.fs.starfarer.api.impl.campaign.ids.Conditions;
import com.fs.starfarer.api.impl.campaign.ids.Submarkets;
import com.fs.starfarer.api.impl.campaign.ids.Tags;
import static com.fs.starfarer.api.impl.campaign.procgen.PlanetConditionGenerator.generateConditionsForPlanet;
import java.util.ArrayList;
import java.util.List;
import org.apache.log4j.Logger;
import org.json.JSONArray;
import org.json.JSONObject;
import org.lazywizard.lazylib.MathUtils;
import org.lwjgl.util.vector.Vector2f;

public class EZFactionStartup {
	private static final List<String> ILLUSTRATIONS = new ArrayList<>();
	private static final List<String> STATION_TYPE = new ArrayList<>();

	//private static final List<String> PICKED_SYSTEMS = new ArrayList();

	private static final List<String> PLANET_TYPES = new ArrayList();
	private static final List<String> SAFE_PLANET_TYPES = new ArrayList();

	static{
		ILLUSTRATIONS.add("orbital");
		ILLUSTRATIONS.add("industrial_megafacility");
		ILLUSTRATIONS.add("terran_orbit");
		ILLUSTRATIONS.add("space_bar");
		ILLUSTRATIONS.add("urban00");
		ILLUSTRATIONS.add("urban01");
		ILLUSTRATIONS.add("urban02");
		ILLUSTRATIONS.add("urban03");
		STATION_TYPE.add("station_side00");
		STATION_TYPE.add("station_side01");
		STATION_TYPE.add("station_side02");
		STATION_TYPE.add("station_side03");
		STATION_TYPE.add("station_side04");
		STATION_TYPE.add("station_side05");
		STATION_TYPE.add("station_side06");
		PLANET_TYPES.add("barren");
		
		SAFE_PLANET_TYPES.add("barren");
		SAFE_PLANET_TYPES.add("barren_castiron");
		SAFE_PLANET_TYPES.add("barren2");
		SAFE_PLANET_TYPES.add("barren3");
		SAFE_PLANET_TYPES.add("arid");
		SAFE_PLANET_TYPES.add("frozen");
		SAFE_PLANET_TYPES.add("frozen1");
		SAFE_PLANET_TYPES.add("frozen2");
		SAFE_PLANET_TYPES.add("frozen3");
		SAFE_PLANET_TYPES.add("barren_venuslike");
		SAFE_PLANET_TYPES.add("rocky_metallic");
		SAFE_PLANET_TYPES.add("rocky_unstable");
		SAFE_PLANET_TYPES.add("rocky_ice");

		List<ModSpecAPI> mods = Global.getSettings().getModManager().getEnabledModsCopy();
		for(ModSpecAPI mod : mods){
			try {
				JSONObject json = Global.getSettings().loadJSON("data/config/planets.json", mod.getId());
				for(int i = 0; i<json.names().length(); i++){
					if(!json.names().getString(i).contains("giant") && !json.names().getString(i).contains("nebula")
					   && !json.names().getString(i).contains("star") && !json.names().getString(i).contains("hole")
							)
					if(json.names().getString(i) != null){
						PLANET_TYPES.add(json.names().getString(i));
					}
				}
			}  catch (Throwable ex) {
				//Global.getLogger(EZFactionModPlugin.class).log(Level.ERROR,"ERROR: couldn't find the EZ Faction directory, etc.", ex);
			}
		}
	}

	private static StarSystemAPI closestSystem(Vector2f point, float dist, float initialAngle){
		List<StarSystemAPI> systems = Global.getSector().getStarSystems();
		Vector2f center;
		if(initialAngle == 0f){
			center = MathUtils.getRandomPointOnCircumference(point, dist);
		} else {
			center = MathUtils.getPointOnCircumference(point, dist, initialAngle);
		}
		float closestDist = 1000000000000f;
		StarSystemAPI pickedSystem = Global.getSector().getStarSystem("corvus");
		for(StarSystemAPI system : systems){
			Vector2f location = system.getCenter().getLocationInHyperspace();
			float distance = MathUtils.getDistance(location, center);
			if(system.getStar() == null) distance = 1000000000f;
			if(system.getStar().getName() == null) distance = 1000000000f;		
			if(system.isNebula()) distance = 1000000000f;
			if(system.isHyperspace()) distance = 1000000000f;
			if(!system.isInConstellation()) distance = 1000000000f;
			if(distance < closestDist){
				pickedSystem = system;
				closestDist = distance;
			}
		}
		return pickedSystem;
	}

	private static void assignPopulation(MarketAPI market, int marketSize){
		switch (marketSize){
			case 1:
				market.addCondition(Conditions.POPULATION_1);
				break;
			case 2:
				market.addCondition(Conditions.POPULATION_2);
				break;
			case 3:
				market.addCondition(Conditions.POPULATION_3);
				break;
			case 4:
				market.addCondition(Conditions.POPULATION_4);
				break;
			case 5:
				market.addCondition(Conditions.POPULATION_5);
				break;
			case 6:
				market.addCondition(Conditions.POPULATION_6);
				break;
			case 7:
				market.addCondition(Conditions.POPULATION_7);
				break;
			case 8:
				market.addCondition(Conditions.POPULATION_8);
				break;
			case 9:
				market.addCondition(Conditions.POPULATION_9);
				break;
			case 10:
				market.addCondition(Conditions.POPULATION_10);
				break;
			default:
				market.addCondition(Conditions.POPULATION_5);
		}
	}

	private static void setMarketConditions(MarketAPI market, SectorEntityToken planet, boolean isStation, boolean isUninhabited){
		String planetType = "barren";
		if(!isStation){
			PlanetAPI thisPlanet = (PlanetAPI) planet;
			planetType = thisPlanet.getSpec().getPlanetType();
		}

		int randomCondition = MathUtils.getRandomNumberInRange(0,7);

		//Randomized resources START
		//ORE RESOURCES
		switch(randomCondition){
			case 0:
				market.addCondition(Conditions.ORE_SPARSE);
				break;
			case 1:
				market.addCondition(Conditions.ORE_MODERATE);
				break;
			case 2:
				market.addCondition(Conditions.ORE_ABUNDANT);
				break;
			case 3:
				market.addCondition(Conditions.ORE_RICH);
				break;
			case 4:
				market.addCondition(Conditions.ORE_ULTRARICH);
				break;
			default:
				market.addCondition(Conditions.ORE_SPARSE);
				break;
		}

		randomCondition = MathUtils.getRandomNumberInRange(0,6);
		//VOLATILE RESOURCES
		switch(randomCondition){
			case 0:
				market.addCondition(Conditions.VOLATILES_TRACE);
				break;
			case 1:
				market.addCondition(Conditions.VOLATILES_DIFFUSE);
				break;
			case 2:
				market.addCondition(Conditions.VOLATILES_PLENTIFUL);
				break;
			case 3:
				market.addCondition(Conditions.VOLATILES_ABUNDANT);
				break;
			default:
				market.addCondition(Conditions.VOLATILES_TRACE);
				break;
		}

		randomCondition = MathUtils.getRandomNumberInRange(0,6);
		//RARE ORE
		switch(randomCondition){
			case 0:
				market.addCondition(Conditions.RARE_ORE_SPARSE);
				break;
			case 1:
				market.addCondition(Conditions.RARE_ORE_MODERATE);
				break;
			case 2:
				market.addCondition(Conditions.RARE_ORE_ABUNDANT);
				break;
			case 3:
				market.addCondition(Conditions.RARE_ORE_RICH);
				break;
			case 4:
				market.addCondition(Conditions.RARE_ORE_ULTRARICH);
				break;
			default:
				market.addCondition(Conditions.RARE_ORE_SPARSE);
				break;
		}

		randomCondition = MathUtils.getRandomNumberInRange(0,6);
		//ORGANICS
		switch(randomCondition){
			case 0:
				market.addCondition(Conditions.ORGANICS_TRACE);
				break;
			case 1:
				market.addCondition(Conditions.ORGANICS_COMMON);
				break;
			case 2:
				market.addCondition(Conditions.ORGANICS_PLENTIFUL);
				break;
			case 3:
				market.addCondition(Conditions.ORGANICS_ABUNDANT);
				break;
			default:
				market.addCondition(Conditions.ORGANICS_TRACE);
				break;
		}

		if(planet.getRadius() > 200f){
			market.addCondition(Conditions.HIGH_GRAVITY);
		}
		if(planet.getRadius() < 100f){
			market.addCondition(Conditions.LOW_GRAVITY);
		}


		switch (planetType){
			case "lava":
				market.addCondition(Conditions.VERY_HOT);
				market.addCondition(Conditions.EXTREME_TECTONIC_ACTIVITY);
				if(!isUninhabited){
					market.addCondition(Conditions.ORE_REFINING_COMPLEX);
					market.addCondition(Conditions.VICE_DEMAND);
				}
				break;
			case "lava_minor":
				market.addCondition(Conditions.HOT);
				market.addCondition(Conditions.TECTONIC_ACTIVITY);
				if(!isUninhabited){
					market.addCondition(Conditions.AUTOFAC_HEAVY_INDUSTRY);
				}
				market.addCondition(Conditions.POLLUTION);
				break;
			case "frozen":
				market.addCondition(Conditions.VERY_COLD);
				if(!isUninhabited){
					market.addCondition(Conditions.VOLATILES_DEPOT);
				}
				break;
			case "frozen1":
				market.addCondition(Conditions.COLD);
				if(!isUninhabited){
					market.addCondition(Conditions.HYDROPONICS_COMPLEX);
				}
				market.addCondition(Conditions.EXTREME_WEATHER);
				break;
			case "frozen2":
				market.addCondition(Conditions.COLD);
				if(!isUninhabited){
					market.addCondition(Conditions.ORGANICS_COMPLEX);
				}
				break;
			case "frozen3":
				market.addCondition(Conditions.DENSE_ATMOSPHERE);
				market.addCondition(Conditions.ICE);
				if(!isUninhabited){
					market.addCondition(Conditions.VOLATILES_COMPLEX);
				}
				break;
			case "barren":
				market.addCondition(Conditions.NO_ATMOSPHERE);
				if(!isUninhabited){
					market.addCondition(Conditions.SHIPBREAKING_CENTER);
					market.addCondition(Conditions.FREE_PORT);
				}
				break;
			case "barren_castiron":
				market.addCondition(Conditions.THIN_ATMOSPHERE);
				if(!isUninhabited){
					market.addCondition(Conditions.AUTOFAC_HEAVY_INDUSTRY);
					market.addCondition(Conditions.FREE_PORT);
				}
				market.addCondition(Conditions.DARK);
				break;
			case "barren2":
				market.addCondition(Conditions.TOXIC_ATMOSPHERE);
				if(!isUninhabited){
					market.addCondition(Conditions.ORE_REFINING_COMPLEX);
					market.addCondition(Conditions.FREE_PORT);
				}
				break;
			case "barren3":
				market.addCondition(Conditions.THIN_ATMOSPHERE);
				market.addCondition(Conditions.COLD);
				if(!isUninhabited){
					market.addCondition(Conditions.COTTAGE_INDUSTRY);
					market.addCondition(Conditions.HYDROPONICS_COMPLEX);
				}
				break;
			case "barren_venuslike":
				market.addCondition(Conditions.INIMICAL_BIOSPHERE);
				market.addCondition(Conditions.HOT);
				market.addCondition(Conditions.POOR_LIGHT);
				if(!isUninhabited){
					market.addCondition(Conditions.LIGHT_INDUSTRIAL_COMPLEX);
				}
				break;
			case "toxic":
				market.addCondition(Conditions.INIMICAL_BIOSPHERE);
				market.addCondition(Conditions.HOT);
				market.addCondition(Conditions.POOR_LIGHT);
				if(!isUninhabited){
					market.addCondition(Conditions.AUTOFAC_HEAVY_INDUSTRY);
					market.addCondition(Conditions.DISSIDENT);
				}
				break;
			case "toxic_cold":
				market.addCondition(Conditions.INIMICAL_BIOSPHERE);
				market.addCondition(Conditions.COLD);
				market.addCondition(Conditions.POOR_LIGHT);
				if(!isUninhabited){
					market.addCondition(Conditions.ANTIMATTER_FUEL_PRODUCTION);
				}
				break;
			case "jungle":
				market.addCondition(Conditions.HABITABLE);
				market.addCondition(Conditions.FARMLAND_BOUNTIFUL);
				if(!isUninhabited){
					market.addCondition(Conditions.LIGHT_INDUSTRIAL_COMPLEX);
					market.addCondition(Conditions.URBANIZED_POLITY);
					market.addCondition(Conditions.ORBITAL_BURNS);
					market.addCondition(Conditions.ORGANICS_COMPLEX);
					market.addCondition(Conditions.ORGANIZED_CRIME);
				}
				break;
			case "terran":
				market.addCondition(Conditions.HABITABLE);
				market.addCondition(Conditions.FARMLAND_RICH);
				if(!isUninhabited){
					market.addCondition(Conditions.LIGHT_INDUSTRIAL_COMPLEX);
					market.addCondition(Conditions.AUTOFAC_HEAVY_INDUSTRY);
					market.addCondition(Conditions.URBANIZED_POLITY);
					market.addCondition(Conditions.HYDROPONICS_COMPLEX);
				}
				break;
			case "desert":
				market.addCondition(Conditions.HABITABLE);
				market.addCondition(Conditions.FARMLAND_POOR);
				market.addCondition(Conditions.ARID);
				if(!isUninhabited){
					market.addCondition(Conditions.HYDROPONICS_COMPLEX);
					market.addCondition(Conditions.RURAL_POLITY);
				}
				break;
			case "desert1":
				market.addCondition(Conditions.HABITABLE);
				market.addCondition(Conditions.FARMLAND_POOR);
				market.addCondition(Conditions.ARID);
				if(!isUninhabited){
					market.addCondition(Conditions.ANTIMATTER_FUEL_PRODUCTION);
					market.addCondition(Conditions.RURAL_POLITY);
				}
				break;
			case "arid":
				market.addCondition(Conditions.HABITABLE);
				market.addCondition(Conditions.FARMLAND_POOR);
				market.addCondition(Conditions.HOT);
				market.addCondition(Conditions.ARID);
				if(!isUninhabited){
					market.addCondition(Conditions.LIGHT_INDUSTRIAL_COMPLEX);
					market.addCondition(Conditions.FREE_PORT);
					market.addCondition(Conditions.RURAL_POLITY);
				}
				break;
			case "cryovolcanic":
				market.addCondition(Conditions.HABITABLE);
				market.addCondition(Conditions.FARMLAND_POOR);
				market.addCondition(Conditions.DENSE_ATMOSPHERE);
				market.addCondition(Conditions.POOR_LIGHT);
				if(!isUninhabited){
					market.addCondition(Conditions.RURAL_POLITY);
				}
				break;
			case "rocky_metallic":
				market.addCondition(Conditions.THIN_ATMOSPHERE);
				if(!isUninhabited){
					market.addCondition(Conditions.ORE_REFINING_COMPLEX);
				}
				break;
			case "rocky_unstable":
				market.addCondition(Conditions.TECTONIC_ACTIVITY);
				market.addCondition(Conditions.HOT);
				market.addCondition(Conditions.METEOR_IMPACTS);
				if(!isUninhabited){
					market.addCondition(Conditions.ORE_REFINING_COMPLEX);
				}
				break;
			case "rocky_ice":
				market.addCondition(Conditions.HABITABLE);
				market.addCondition(Conditions.COLD);
				market.addCondition(Conditions.FARMLAND_POOR);
				market.addCondition(Conditions.POOR_LIGHT);
				if(!isUninhabited){
					market.addCondition(Conditions.COTTAGE_INDUSTRY);
					market.addCondition(Conditions.RURAL_POLITY);
				}
				break;
			case "irradiated":
				market.addCondition(Conditions.IRRADIATED);
				if(!isUninhabited){
					market.addCondition(Conditions.AUTOFAC_HEAVY_INDUSTRY);
				}
				break;
			case "barren-bombarded":
				market.addCondition(Conditions.RUINS_WIDESPREAD);
				market.addCondition(Conditions.POLLUTION);
				if(!isUninhabited){
					market.addCondition(Conditions.DECIVILIZED);
					market.addCondition(Conditions.VICE_DEMAND);
				}
				break;
			case "tundra":
				market.addCondition(Conditions.COLD);
				market.addCondition(Conditions.HABITABLE);
				market.addCondition(Conditions.FARMLAND_POOR);
				if(!isUninhabited){
					market.addCondition(Conditions.COTTAGE_INDUSTRY);
					market.addCondition(Conditions.RURAL_POLITY);
				}
				break;
			case "barren-desert":
				market.addCondition(Conditions.ARID);
				market.addCondition(Conditions.HOT);
				market.addCondition(Conditions.HABITABLE);
				market.addCondition(Conditions.FARMLAND_POOR);
				if(!isUninhabited){
					market.addCondition(Conditions.RURAL_POLITY);
				}
				break;
			case "terran-eccentric":
				market.addCondition(Conditions.HABITABLE);
				market.addCondition(Conditions.RUINS_SCATTERED);
				market.addCondition(Conditions.FARMLAND_RICH);
				market.addCondition(Conditions.MILD_CLIMATE);
				if(!isUninhabited){
					market.addCondition(Conditions.LIGHT_INDUSTRIAL_COMPLEX);
					market.addCondition(Conditions.URBANIZED_POLITY);
				}
				break;
			default:
				if(!isUninhabited){
					market.addCondition(Conditions.ANTIMATTER_FUEL_PRODUCTION);
					market.addCondition(Conditions.HYDROPONICS_COMPLEX);
					market.addCondition(Conditions.LIGHT_INDUSTRIAL_COMPLEX);
				}
				break;
		}
	}

	private static String pickDenomination(StarSystemAPI pickedSystem, boolean ignorePlanets, boolean ignoreStations){
		int count = 0;
		String denomination;
		for(SectorEntityToken object : pickedSystem.getAllEntities()){
			if(!ignorePlanets && object.hasTag("customPlanet")){
				count += 1;
			} else if (!ignoreStations && object.hasTag("customStation")){
				count += 1;
			}
		}

		switch(count){
			case 0:
				denomination = "I";
				break;
			case 1:
				denomination = "II";
				break;
			case 2:
				denomination = "III";
				break;
			case 3:
				denomination = "IV";
				break;
			case 4:
				denomination = "V";
				break;
			case 5:
				denomination = "VI";
				break;
			case 6:
				denomination = "VII";
				break;
			case 7:
				denomination = "VIII";
				break;
			case 8:
				denomination = "IX";
				break;
			case 9:
				denomination = "X";
				break;
			default:
				denomination = "XXX";
				break;
		}

		return denomination;
	}
	
	private static float pickOrbitalRadius(StarSystemAPI pickedSystem){
		int count = 0;
		float orbit;
		for(SectorEntityToken object : pickedSystem.getAllEntities()){
			if(object instanceof PlanetAPI && object.hasTag("customPlanet")){
				count += 1;
			}
		}

		switch(count){
			case 0:
				orbit = MathUtils.getRandomNumberInRange(3000f, 3500f);
				break;
			case 1:
				orbit = MathUtils.getRandomNumberInRange(4000f, 4500f);
				break;
			case 2:
				orbit = MathUtils.getRandomNumberInRange(5000f, 5500f);
				break;
			case 3:
				orbit = MathUtils.getRandomNumberInRange(6000f, 6500f);
				break;
			case 4:
				orbit = MathUtils.getRandomNumberInRange(7000f, 7500f);
				break;
			case 5:
				orbit = MathUtils.getRandomNumberInRange(8000f, 8500f);
				break;
			case 6:
				orbit = MathUtils.getRandomNumberInRange(9000f, 9500f);
				break;
			case 7:
				orbit = MathUtils.getRandomNumberInRange(10000f, 10500f);
				break;
			case 8:
				orbit = MathUtils.getRandomNumberInRange(11000f, 11500f);
				break;
			case 9:
				orbit = MathUtils.getRandomNumberInRange(12000f, 12500f);
				break;
			default:
				orbit = MathUtils.getRandomNumberInRange(13000f, 13500f);
				break;
		}

		return orbit;
	}	

	private static String planetType(FactionAPI faction){
		boolean defined = false;
		String planetType = "barren";
		List<ModSpecAPI> mods = Global.getSettings().getModManager().getEnabledModsCopy();
		for(ModSpecAPI mod : mods){
			if(defined) continue;
			try {
				JSONObject json = Global.getSettings().loadJSON("data/ezfaction/" + faction.getId() + ".ezfaction", mod.getId());
				if(json.has("preferred_planets")){
					JSONArray subArray = json.getJSONArray("preferred_planets");
					planetType = (String) subArray.getString(MathUtils.getRandomNumberInRange(0, subArray.length()-1));
					defined = true;
				}
			} catch (Throwable ex) {
			}
		}
		if(!defined) planetType = (String) PLANET_TYPES.get(MathUtils.getRandomNumberInRange(0, PLANET_TYPES.size()-1));
		if(planetType == null) planetType = "barren";
		return planetType;
	}
	
	private static String safePlanetType(){
		String planetType = (String) SAFE_PLANET_TYPES.get(MathUtils.getRandomNumberInRange(0, SAFE_PLANET_TYPES.size()-1));
		if(planetType == null) planetType = "barren";
		return planetType;
	}	
	
	private static void buildCommRelayIfNeeded(StarSystemAPI pickedSystem, FactionAPI faction){
		boolean foundCommRelay = false;
		for(SectorEntityToken object : pickedSystem.getEntitiesWithTag("customCommRelay")){
			if(object.getFaction().equals(faction)) foundCommRelay = true;
		}
		if(!foundCommRelay){
					// Guayota Relay - L5 (behind); well, okay, not quite the L5. But whatever.
		SectorEntityToken custom_relay = pickedSystem.addCustomEntity(pickedSystem.getId()+faction.getId()+"_relay", // unique id
				 faction.getDisplayName()+" Relay", // name - if null, defaultName from custom_entities.json will be used
				 "comm_relay", // type of object, defined in custom_entities.json
				 faction.getId()); // faction
		
		custom_relay.addTag("customCommRelay");
		custom_relay.setCircularOrbitPointingDown(pickedSystem.getStar(), 
				MathUtils.getRandomNumberInRange(10, 50), 
				MathUtils.getRandomNumberInRange(3500, 8000), 
				MathUtils.getRandomNumberInRange(120, 300)
				);
		}
	}

	private static SectorEntityToken buildEntity(StarSystemAPI pickedSystem, FactionAPI faction, int minMarketSize, int maxMarketSize, boolean isCapital){
		int marketSize = MathUtils.getRandomNumberInRange(minMarketSize, maxMarketSize);
		if(isCapital) marketSize = maxMarketSize;

		float orbit = pickOrbitalRadius(pickedSystem);
		float orbitalPeriod = ((orbit / 4500f) * 200f) * MathUtils.getRandomNumberInRange(0.5f, 1.5f);
		SectorEntityToken ezColony;

		String starName = pickedSystem.getStar().getName();
		String factionID = faction.getId();
		String denomination = pickDenomination(pickedSystem,false,true);
		try{
			ezColony = pickedSystem.addPlanet(starName + factionID + "planet" + denomination + MathUtils.getRandom(),//Attempts to build a random numbered location id; needs improving
			pickedSystem.getStar(), //Orbital focus
			starName + " Colony " + denomination,//Probably needs improving
			planetType(faction),//Pulls a string at random
			MathUtils.getRandomNumberInRange(30f,60f),//Planet angle
			MathUtils.getRandomNumberInRange(50f,300f),//Planet radius
			orbit,//Orbital radius
			orbitalPeriod);//Orbital period
		}catch (Throwable ex) {
			return null;
		}
		ezColony.setFaction(faction.getId());
		ezColony.setInteractionImage("illustrations", ILLUSTRATIONS.get((int)MathUtils.getRandomNumberInRange(0, ILLUSTRATIONS.size()-1)));
		//ezColony.addTag(Tags.STATION);
		ezColony.addTag("customPlanet");
		
		MarketAPI market = Global.getFactory().createMarket(faction.getId() + ezColony.getId() +"_market"+MathUtils.getRandom(), ezColony.getName(), marketSize);
		market.setPrimaryEntity(ezColony);
		//market.setPlanetConditionMarketOnly(false);
				
		market.addCondition(Conditions.SPACEPORT);
		//market.addCondition(Conditions.OUTPOST);
		market.setFactionId(faction.getId());
		market.setSurveyLevel(SurveyLevel.FULL);
		
		//Set up all of the Market Conditions here
		if(isCapital == true || MathUtils.getRandomNumberInRange(0f, 100f) > 66f) market.addCondition(Conditions.MILITARY_BASE);
		setMarketConditions(market, ezColony, false, false);

		//Set the market's size and population here
		market.setSize(marketSize);
		assignPopulation(market,marketSize);

		market.addSubmarket(Submarkets.SUBMARKET_OPEN);
		if(market.hasCondition(Conditions.MILITARY_BASE)){
			market.addSubmarket(Submarkets.GENERIC_MILITARY);
		}
		market.addSubmarket(Submarkets.SUBMARKET_BLACK);
		market.addSubmarket(Submarkets.SUBMARKET_STORAGE);

		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCommodity(Commodities.FOOD, 1000);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCommodity(Commodities.LUXURY_GOODS, 100);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCommodity(Commodities.ORGANICS, 3000);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCommodity(Commodities.VOLATILES, 3000);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCommodity(Commodities.HEAVY_MACHINERY, 1000);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCommodity(Commodities.METALS, 1000);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCommodity(Commodities.RARE_METALS, 1000);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCrew(300);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addFuel(5000);

		buildCommRelayIfNeeded(pickedSystem,faction);

		//Logger log = Global.getLogger(CoreScript.class);
		//log.info("Created EZ Faction Colony " + ezColony.getName() + " for Faction " + faction.getDisplayName() + " at System " +pickedSystem.getBaseName());
		ezColony.setMarket(market);
		Global.getSector().getEconomy().addMarket(market);
		//PICKED_SYSTEMS.add(pickedSystem.getName());
		return ezColony;
	}
	
	private static SectorEntityToken buildStation(StarSystemAPI pickedSystem, FactionAPI faction, int minMarketSize, int maxMarketSize, boolean isCapital){
		int marketSize = MathUtils.getRandomNumberInRange(minMarketSize, maxMarketSize);
		if(isCapital) marketSize = maxMarketSize;
		if(pickedSystem == null) return null;

		if(faction == null) return null;
		if(pickedSystem.getStar() == null) return null;
		if(pickedSystem.getStar().getName() == null) return null;
		if(faction.getId() == null) return null;
		if(pickedSystem.isHyperspace() || pickedSystem.isNebula()) return null;
		float orbit = pickOrbitalRadius(pickedSystem);
		float orbitalPeriod = ((orbit / 4500f) * 200f) * MathUtils.getRandomNumberInRange(0.5f, 1.5f);		
		PlanetAPI ezColonyPlanet;
		try{
			String starName = pickedSystem.getStar().getName();
			String factionID = faction.getId();
			String denomination = pickDenomination(pickedSystem,false,true);
			ezColonyPlanet = pickedSystem.addPlanet(starName + factionID + "customPlanet" + denomination+MathUtils.getRandom(),//Attempts to build a random numbered location id; needs improving
			pickedSystem.getStar(), starName+" "+ denomination,//Probably needs improving
			safePlanetType(),//Pulls a string at random
			MathUtils.getRandomNumberInRange(30f,60f),//Angle
			MathUtils.getRandomNumberInRange(50f,300f),//Radius
			orbit,//Orbital radius
			orbitalPeriod);//Orbital period
		}catch (Throwable ex) {
			return null;
		}
		//ezColonyPlanet.setFaction(faction.getId());
		SectorEntityToken ezColony;
		try{
			String starName = pickedSystem.getStar().getName();
			String factionID = faction.getId();
			String denomination = pickDenomination(pickedSystem,true,false);
			String displayName = faction.getDisplayName();
			if(displayName.equalsIgnoreCase("pirates")){
				displayName = "Pirates";
			}
			ezColony = pickedSystem.addCustomEntity(starName + factionID + denomination + "_station"+MathUtils.getRandom(),//Attempts to build a random numbered location id; needs improving
			displayName + " Station " + denomination,//Probably needs improving
			STATION_TYPE.get(MathUtils.getRandomNumberInRange(0,STATION_TYPE.size()-1)),
			factionID
			);
			ezColony.setCircularOrbitPointingDown(ezColonyPlanet, 
			MathUtils.getRandomNumberInRange(10f, 30f), 
			MathUtils.getRandomNumberInRange(300f, 500f),
			MathUtils.getRandomNumberInRange(30f, 60f)
			);
		}catch (Throwable ex) {
			return null;
		}
		ezColonyPlanet.addTag("customPlanet");
		ezColonyPlanet.addTag("customPlanetNoKill");
		generateConditionsForPlanet(ezColonyPlanet,pickedSystem.getAge());

		ezColony.addTag("customStation");
		ezColony.setFaction(faction.getId());
		ezColony.setInteractionImage("illustrations", ILLUSTRATIONS.get((int)MathUtils.getRandomNumberInRange(0, ILLUSTRATIONS.size()-1)));
		ezColony.addTag(Tags.STATION);
		MarketAPI market = Global.getFactory().createMarket(faction.getId() + ezColony.getId() +"_stationmarket"+MathUtils.getRandom(), ezColony.getName(), marketSize);
		ezColony.setMarket(market);
		//market.addCondition(Conditions.SPACEPORT);
		market.addCondition(Conditions.OUTPOST);

		market.setFactionId(faction.getId());

		market.setSurveyLevel(SurveyLevel.FULL);
		market.setPrimaryEntity(ezColony);

		//Set up all of the Market Conditions here
		if(isCapital == true || MathUtils.getRandomNumberInRange(0f, 100f) > 66f) market.addCondition(Conditions.MILITARY_BASE);
		setMarketConditions(market, ezColony, true, false);

		//Set the market's size and population here
		market.setSize(marketSize);
		assignPopulation(market,marketSize);

		market.addSubmarket(Submarkets.SUBMARKET_OPEN);
		if(market.hasCondition(Conditions.MILITARY_BASE)){
			market.addSubmarket(Submarkets.GENERIC_MILITARY);
		}
		market.addSubmarket(Submarkets.SUBMARKET_BLACK);
		market.addSubmarket(Submarkets.SUBMARKET_STORAGE);

		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCommodity(Commodities.FOOD, 1000);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCommodity(Commodities.LUXURY_GOODS, 100);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCommodity(Commodities.ORGANICS, 3000);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCommodity(Commodities.VOLATILES, 3000);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCommodity(Commodities.HEAVY_MACHINERY, 1000);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCommodity(Commodities.METALS, 1000);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCommodity(Commodities.RARE_METALS, 1000);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addCrew(300);
		market.getSubmarket(Submarkets.SUBMARKET_OPEN).getCargo().addFuel(5000);

		Global.getSector().getEconomy().addMarket(market);
		
		buildCommRelayIfNeeded(pickedSystem,faction);

		Logger log = Global.getLogger(CoreScript.class);
		log.info("Created EZ Faction Station " + ezColony.getName() + " for Faction " + faction.getDisplayName() + " at System " +pickedSystem.getBaseName());
		//PICKED_SYSTEMS.add(pickedSystem.getName());
				
		return ezColony;
	}


	public static void EZFactionStartup(FactionAPI faction, int numColonies, float coreDist, int minMarketSize, int maxMarketSize, boolean onlyStations, boolean randomPirates, float initialAngle){
		Vector2f center = Global.getSector().getStarSystem("corvus").getCenter().getLocationInHyperspace();
		Vector2f capLoc = center;
		for(int i = 0; i < numColonies; i++){
			StarSystemAPI pickedSystem;		
			boolean isCapital;
			//Sets the initial colony
			if(i == 0){
				pickedSystem = closestSystem(capLoc,coreDist,initialAngle);
				capLoc = pickedSystem.getCenter().getLocationInHyperspace();
				isCapital = true;
			} else {
				pickedSystem = closestSystem(capLoc,MathUtils.getRandomNumberInRange(500f, 10000f),MathUtils.getRandomNumberInRange(0f, 360f));
				isCapital = false;
			}
			
			SectorEntityToken newEntity = null;

			if(!onlyStations){
				if(MathUtils.getRandomNumberInRange(0, 100) < 70){
					while(newEntity == null){
						pickedSystem = closestSystem(capLoc,MathUtils.getRandomNumberInRange(500f, 10000f),MathUtils.getRandomNumberInRange(0f, 360f));
						newEntity = buildEntity(pickedSystem, faction, minMarketSize, maxMarketSize, isCapital);
					}
				} else {
					while(newEntity == null) {
						pickedSystem = closestSystem(capLoc,MathUtils.getRandomNumberInRange(500f, 10000f),MathUtils.getRandomNumberInRange(0f, 360f));
						newEntity = buildStation(pickedSystem, faction, minMarketSize, maxMarketSize, isCapital);
					}
				}
			} else {
				while(newEntity == null) {
					pickedSystem = closestSystem(capLoc,MathUtils.getRandomNumberInRange(500f, 10000f),MathUtils.getRandomNumberInRange(0f, 360f));
					newEntity = buildStation(pickedSystem, faction, minMarketSize, maxMarketSize, isCapital);
				}
			}
			//Randomized Pirates
			if(randomPirates && (MathUtils.getRandomNumberInRange(0, 100) < 40)){
				SectorEntityToken pirateEnt = null;
				while(pirateEnt == null){
					pirateEnt = buildStation(pickedSystem, Global.getSector().getFaction("pirates"),2,6,false);
					pickedSystem = closestSystem(capLoc,MathUtils.getRandomNumberInRange(500f, 10000f),MathUtils.getRandomNumberInRange(0f, 360f));
				}
			}
		}
	}
}






