package data.scripts;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.BeamAPI;
import com.fs.starfarer.api.combat.DamagingProjectileAPI;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.WeaponAPI;
import com.fs.starfarer.api.graphics.SpriteAPI;
import java.awt.Color;
import java.awt.geom.Line2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import org.json.JSONObject;
import org.lazywizard.lazylib.MathUtils;
import org.lwjgl.util.vector.Vector2f;

public class fx_SharedLib {
	//private static int lastTex = 0;

    public static float frameTime = 0f;
    public static final float LONG_DIST = 2500f * 2500f;
	public static int MAX_PARTICLES = 500;
	public static int MAX_ADDITIVE_PARTICLES = 500;
	
	public static int ORIGINAL_MAX_PARTICLES;
	public static int ORIGINAL_MAX_ADD_PARTICLES;
	
	private static final Color FADE_ADDITIVE = new Color(0,0,0,255);
	private static final Color FADE_NORMAL = new Color(0,0,0,0);

    public static Vector2f centerPoint;

    public static Map<Integer,ArrayList<fx_Particle>> particleList = new HashMap<>();
	public static Map<Integer,ArrayList<fx_Particle>> additiveParticleList = new HashMap<>();

	public static Map projSizeMap = new HashMap();
    public static Map projCoreColorMap = new HashMap();
    public static Map projFringeColorMap = new HashMap();

    public static HashSet<DamagingProjectileAPI> deadShotList = new HashSet<>(100);

    public static HashSet<MissileAPI> seenMissiles = new HashSet<>(100);

    public static void clearMe() {
		additiveParticleList.clear();
		particleList.clear();
		deadShotList.clear();
		seenMissiles.clear();
		try {
				JSONObject json;
				try{
					json = Global.getSettings().loadJSON("fx_mod.ini");
					MAX_PARTICLES = json.getInt("maxParticles");
					ORIGINAL_MAX_PARTICLES = MAX_PARTICLES;
					MAX_ADDITIVE_PARTICLES = MAX_PARTICLES;
					ORIGINAL_MAX_ADD_PARTICLES = MAX_PARTICLES;
				} catch(Throwable ex) {
					MAX_PARTICLES = 500;
					ORIGINAL_MAX_PARTICLES = MAX_PARTICLES;
					MAX_ADDITIVE_PARTICLES = MAX_PARTICLES;
					ORIGINAL_MAX_ADD_PARTICLES = MAX_PARTICLES;
				}
			} catch(Throwable ex) {
				MAX_PARTICLES = 500;
				ORIGINAL_MAX_PARTICLES = MAX_PARTICLES;
				MAX_ADDITIVE_PARTICLES = MAX_PARTICLES;
				ORIGINAL_MAX_ADD_PARTICLES = MAX_PARTICLES;
		}
    }

    //Returns a randomized velocity vector; needs a velocity component
    public static Vector2f randomCircularVelocity(float velocity) {
		Vector2f newVel = MathUtils.getRandomPointOnCircumference(new Vector2f(), velocity);
		return newVel;
    }

    //Returns a specific velocity vector; requires a velocity and an angular component
    public static Vector2f circularVelocity(float angle, float velocity) {
		Vector2f newVel = MathUtils.getPointOnCircumference(new Vector2f(), velocity, angle);
		return newVel;
    }

    public static void handleParticle(fx_Particle thisParticle) {
		//Don't bother moving or doing any math on dead particles
		if (thisParticle.isDead()) return;
		//Age the living particles
		thisParticle.timerChange(thisParticle.timeLeft() - fx_SharedLib.frameTime);

		float timeLeft = thisParticle.timeLeft();
		boolean isFading = thisParticle.isFading();

		//Kill particles that are dead and exit the loop here.
		if (timeLeft <= 0f && isFading) {
			thisParticle.kill();
			return;
		}

		//Sets this particle to fade out, if we're not already there
		if (timeLeft <= 0f && !isFading) {
			thisParticle.fade();
			thisParticle.timerChange(thisParticle.fadeTime);
			thisParticle.totalTime = thisParticle.fadeTime;
		}

		//We're now ready to move the particle around, etc.
		Vector2f movement = thisParticle.position;
		if (frameTime > 0f) {
			if (thisParticle.isFading) {
				if(thisParticle.isAdditive){
					thisParticle.curColor = blend(FADE_ADDITIVE, thisParticle.endRGB, thisParticle.timeLeft() / thisParticle.totalTime);
				} else {
					thisParticle.curColor = blend(FADE_NORMAL, thisParticle.endRGB, thisParticle.timeLeft() / thisParticle.totalTime);
				}
			} else if (thisParticle.isBlended) {
				thisParticle.curColor = blend(thisParticle.endRGB, thisParticle.startRGB, thisParticle.timeLeft() / thisParticle.totalTime);
			}
				if (thisParticle.growth != 1.0f) {
				thisParticle.size = new Vector2f(thisParticle.size.getX() * thisParticle.growth, thisParticle.size.getY() * thisParticle.growth);
			}
			movement = new Vector2f(movement.getX() + (thisParticle.velocity.getX() * frameTime),
				movement.getY() + (thisParticle.velocity.getY() * frameTime));
		}
		thisParticle.position = movement;
    }

    public static Color blend(Color c1, Color c2, float ratio) {
		if (ratio > 1f) {
			ratio = 1f;
		} else if (ratio < 0f) {
			ratio = 0f;
		}
		float iRatio = 1.0f - ratio;

		int redOne = Math.min(255, Math.max(0, Math.round((c1.getRed() * iRatio) + (c2.getRed() * ratio))));
		int greenOne = Math.min(255, Math.max(0, Math.round((c1.getGreen() * iRatio) + (c2.getGreen() * ratio))));
		int blueOne = Math.min(255, Math.max(0, Math.round((c1.getBlue() * iRatio) + (c2.getBlue() * ratio))));
		int alphaOne = Math.min(255, Math.max(0, Math.round((c1.getAlpha() * iRatio) + (c2.getAlpha() * ratio))));

		return new Color(redOne, greenOne, blueOne, alphaOne);
    }

    public static void newParticle(Vector2f location, Vector2f velocity, float size, float ratio, float angle, boolean isAdditive, boolean isBlended, SpriteAPI sprite, Color startColor, Color endColor, float lifeTime, float fadeTime, float growth) {	
		//Don't add new particles if we've already exceeded the maximum number
		if(isAdditive){
			int listSize = 0;
			for (Object value : fx_SharedLib.additiveParticleList.values()) {
				ArrayList thisList = (ArrayList) value;
				listSize += thisList.size();
			}	
			if(listSize > MAX_ADDITIVE_PARTICLES){
				return;
			}
		} else {
			int listSize = 0;
			for (Object value : fx_SharedLib.particleList.values()) {
				ArrayList thisList = (ArrayList) value;
				listSize += thisList.size();
			}	
			if(listSize > MAX_PARTICLES){
				return;
			}			
		}

		//Don't build new particles outside the view-distance.
		if (location == null || centerPoint == null) {
			return;
		}
		if (MathUtils.getDistanceSquared(location, centerPoint) > LONG_DIST) {
			return;
		}
		//Weird fix for angle weirdness
		//angle = angle - 90f;
		Vector2f initialSize = new Vector2f(size / ratio, size * ratio);
		fx_Particle data = new fx_Particle(sprite, lifeTime, fadeTime, initialSize, location, velocity, angle, isAdditive, isBlended, startColor, endColor, growth);
		
		//Adds each particle to a sorted-by-texture-id ArrayList
		if(data.isAdditive){
			if(additiveParticleList.containsKey(data.textureID)){
				additiveParticleList.get(data.textureID).add(data);
			} else {
				additiveParticleList.put(data.textureID, new ArrayList<fx_Particle>());
				additiveParticleList.get(data.textureID).add(data);
			}
		} else {
			if(particleList.containsKey(data.textureID)){
				particleList.get(data.textureID).add(data);
			} else {
				particleList.put(data.textureID, new ArrayList<fx_Particle>());
				particleList.get(data.textureID).add(data);
			}
		}
    }


	//Additive; only puts 255 into Alpha
    public static Color[] getBeamColors(BeamAPI beam) {
		Color projColor = new Color(255, 255, 255, 255);
		Color projColorFade = new Color(255, 255, 255, 255);
		projCoreColorMap.put(beam.getWeapon().getShip().getId(), projColor);
		projFringeColorMap.put(beam.getWeapon().getShip().getId(), projColorFade);
		projSizeMap.put(beam.getWeapon().getId(), 1f);
		try {
			JSONObject json;
			try{
				json = Global.getSettings().loadJSON("data/weapons/" + beam.getWeapon().getId() + ".wpn");
			} catch(Throwable ex) {
				Color[] returnedColor = new Color[2];
				returnedColor[0] = projColor;
				returnedColor[1] = projColorFade;
				projCoreColorMap.put(beam.getWeapon().getShip().getId(), projColor);
				projFringeColorMap.put(beam.getWeapon().getShip().getId(), projColorFade);
				return returnedColor;
			}
			if(json.has("fxShotMult")){
				projSizeMap.put(beam.getWeapon().getId(), (float) json.getDouble("fxShotMult"));
			}
			//Support for custom Glows, Fringes and Mults.
			if(json.has("fxShotGlowCore") && json.has("fxShotGlowFringe")){
				int redCore = (int) json.getJSONArray("fxShotGlowCore").getInt(0);
				int greenCore = (int) json.getJSONArray("fxShotGlowCore").getInt(1);
				int blueCore = (int) json.getJSONArray("fxShotGlowCore").getInt(2);
				int alphaCore = (int) 255;

				int redFringe = (int) json.getJSONArray("fxShotGlowFringe").getInt(0);
				int greenFringe = (int) json.getJSONArray("fxShotGlowFringe").getInt(1);
				int blueFringe = (int) json.getJSONArray("fxShotGlowFringe").getInt(2);
				int alphaFringe = (int) 255;

				projCoreColorMap.put(beam.getWeapon().getId(), new Color(redCore, greenCore, blueCore, alphaCore));
				projFringeColorMap.put(beam.getWeapon().getId(), new Color(redFringe, greenFringe, blueFringe, alphaFringe));
				projColor = new Color(redCore, greenCore, blueCore, alphaCore);
				projColorFade = new Color(redFringe, greenFringe, blueFringe, alphaFringe);
				if (projColor.getRed() + projColor.getGreen() + projColor.getBlue() >= 764) {
					projColor = new Color(projColorFade.getRed(), projColorFade.getGreen(), projColorFade.getBlue(), 255);
				}
			} else {
				int redCore = (int) json.getJSONArray("coreColor").getInt(0);
				int greenCore = (int) json.getJSONArray("coreColor").getInt(1);
				int blueCore = (int) json.getJSONArray("coreColor").getInt(2);
				int alphaCore = (int) 255;

				int redFringe = (int) json.getJSONArray("fringeColor").getInt(0);
				int greenFringe = (int) json.getJSONArray("fringeColor").getInt(1);
				int blueFringe = (int) json.getJSONArray("fringeColor").getInt(2);
				int alphaFringe = (int) 255;

				projCoreColorMap.put(beam.getWeapon().getId(), new Color(redCore, greenCore, blueCore, alphaCore));
				projFringeColorMap.put(beam.getWeapon().getId(), new Color(redFringe, greenFringe, blueFringe, alphaFringe));
				projColor = (Color) projCoreColorMap.get(beam.getWeapon().getId());
				projColorFade = (Color) projFringeColorMap.get(beam.getWeapon().getId());
				if (projColor.getRed() + projColor.getGreen() + projColor.getBlue() >= 764) {
					projColor = projColorFade;
				}
			}
		} catch (Throwable ex) {
			Color[] returnedColor = new Color[2];
			returnedColor[0] = projColor;
			returnedColor[1] = projColorFade;
			projCoreColorMap.put(beam.getWeapon().getShip().getId(), projColor);
			projFringeColorMap.put(beam.getWeapon().getShip().getId(), projColorFade);
			return returnedColor;
			//Global.getLogger(EZFactionModPlugin.class).log(Level.ERROR,"ERROR: couldn't find the EZ Faction directory, etc.", ex);
		}

		Color[] returnedColor = new Color[2];
		returnedColor[0] = projColor;
		returnedColor[1] = projColorFade;
		return returnedColor;
    }

	//Additive; Alpha is 255
    public static Color[] getProjectileColors(DamagingProjectileAPI projectile) {
		Color projColor = new Color(255, 255, 128, 255);
		Color projColorFade = new Color(255, 64, 0, 255);
		projSizeMap.put(projectile.getWeapon().getId(), 1f);
		projCoreColorMap.put(projectile.getWeapon().getShip().getId(), projColor);
		projFringeColorMap.put(projectile.getWeapon().getShip().getId(), projColorFade);

		boolean defined = false;

		//Avoids bug with Flares; hack!
		if (projectile.getWeapon().hasAIHint(WeaponAPI.AIHints.SYSTEM)) {
			defined = true;
		}

		if (!defined) {
			try {
				JSONObject json;
				try{
					json = Global.getSettings().loadJSON("data/weapons/proj/" + projectile.getProjectileSpecId() + ".proj");
				} catch(Throwable ex) {
					Color[] returnedColor = new Color[2];
					returnedColor[0] = projColor;
					returnedColor[1] = projColorFade;
					return returnedColor;
				}
				if(json.has("fxShotMult")){
					projSizeMap.put(projectile, (float)json.getDouble("fxShotMult"));
				}
				if(json.has("fxShotGlowCore") && json.has("fxShotGlowFringe")){
					int redCore = (int) json.getJSONArray("fxShotGlowCore").getInt(0);
					int greenCore = (int) json.getJSONArray("fxShotGlowCore").getInt(1);
					int blueCore = (int) json.getJSONArray("fxShotGlowCore").getInt(2);
					int alphaCore = (int) 255;

					int redFringe = (int) json.getJSONArray("fxShotGlowFringe").getInt(0);
					int greenFringe = (int) json.getJSONArray("fxShotGlowFringe").getInt(1);
					int blueFringe = (int) json.getJSONArray("fxShotGlowFringe").getInt(2);
					int alphaFringe = (int) 255;

					projCoreColorMap.put(projectile.getProjectileSpecId(), new Color(redCore, greenCore, blueCore, alphaCore));
					projFringeColorMap.put(projectile.getProjectileSpecId(), new Color(redFringe, greenFringe, blueFringe, alphaFringe));
					projColor = new Color(redCore, greenCore, blueCore, alphaCore);
					projColorFade = new Color(redFringe, greenFringe, blueFringe, alphaFringe);
					if (projColor.getRed() + projColor.getGreen() + projColor.getBlue() >= 764) {
						projColor = new Color(projColorFade.getRed(), projColorFade.getGreen(), projColorFade.getBlue(), 255);
					}
				} else {
					//AUTOMATIC HANDLER
					boolean isEnergy = false;
					switch (projectile.getDamageType()) {
						case HIGH_EXPLOSIVE:
							projColor = new Color(255, 255, 128, 255);
							projColorFade = new Color(255, 64, 0, 255);
							projCoreColorMap.put(projectile.getProjectileSpecId(), projColor);
							projFringeColorMap.put(projectile.getProjectileSpecId(), projColorFade);
							break;
						case KINETIC:
							projColor = new Color(255, 255, 64, 255);
							projColorFade = new Color(128, 32, 0, 255);
							projCoreColorMap.put(projectile.getProjectileSpecId(), projColor);
							projFringeColorMap.put(projectile.getProjectileSpecId(), projColorFade);
							break;
						case FRAGMENTATION:
							projColor = new Color(255, 128, 0, 255);
							projColorFade = new Color(64, 0, 0, 255);
							projCoreColorMap.put(projectile.getProjectileSpecId(), projColor);
							projFringeColorMap.put(projectile.getProjectileSpecId(), projColorFade);
							break;
						case ENERGY:
							isEnergy = true;
							break;
						default:
							projColor = new Color(255, 128, 0, 255);
							projColorFade = new Color(64, 0, 0, 255);
							projCoreColorMap.put(projectile.getProjectileSpecId(), projColor);
							projFringeColorMap.put(projectile.getProjectileSpecId(), projColorFade);
							break;
					}
					if(isEnergy){
						int redCore = (int) json.getJSONArray("coreColor").getInt(0);
						int greenCore = (int) json.getJSONArray("coreColor").getInt(1);
						int blueCore = (int) json.getJSONArray("coreColor").getInt(2);
						int alphaCore = (int) 255;

						int redFringe = (int) json.getJSONArray("fringeColor").getInt(0);
						int greenFringe = (int) json.getJSONArray("fringeColor").getInt(1);
						int blueFringe = (int) json.getJSONArray("fringeColor").getInt(2);
						int alphaFringe = (int) 255;

						projCoreColorMap.put(projectile.getProjectileSpecId(), new Color(redCore, greenCore, blueCore, alphaCore));
						projFringeColorMap.put(projectile.getProjectileSpecId(), new Color(redFringe, greenFringe, blueFringe, alphaFringe));
						projColor = new Color(redCore, greenCore, blueCore, alphaCore);
						projColorFade = new Color(redFringe, greenFringe, blueFringe, alphaFringe);
						if (projColor.getRed() + projColor.getGreen() + projColor.getBlue() >= 764) {
							projColor = new Color(projColorFade.getRed(), projColorFade.getGreen(), projColorFade.getBlue(), 255);
						}
					}
				}
			} catch (Throwable ex) {
				Color[] returnedColor = new Color[2];
				returnedColor[0] = projColor;
				returnedColor[1] = projColorFade;
				return returnedColor;
			//Global.getLogger(EZFactionModPlugin.class).log(Level.ERROR,"ERROR: couldn't find the EZ Faction directory, etc.", ex);
			}
		}
		Color[] returnedColor = new Color[2];
		returnedColor[0] = projColor;
		returnedColor[1] = projColorFade;
		return returnedColor;
    }

    //Rewrite of LazyWizard's IsWithinArc
    public static boolean isWithinShieldArc(ShipAPI ship, Vector2f point) {
		Vector2f shipLoc = ship.getLocation();
		float shieldRange = ship.getShield().getRadius() + 45f;
		if (MathUtils.getDistanceSquared(shipLoc, point) > shieldRange * shieldRange) {
			return false;
		}

		float shieldAngle = ship.getShield().getFacing();
		float shieldArc = ship.getShield().getArc() / 2f;

		// Check if weapon is aimed at any part of the target
		Vector2f endArcLeft = MathUtils.getPointOnCircumference(shipLoc, shieldRange, shieldAngle - shieldArc);
		Vector2f endArcRight = MathUtils.getPointOnCircumference(shipLoc, shieldRange, shieldAngle + shieldArc);
		float radSquared = shieldRange * shieldRange;

		boolean partiallyInArc = (Line2D
			.ptSegDistSq(
				shipLoc.x,
				shipLoc.y,
				endArcLeft.x,
				endArcLeft.y,
				point.x,
				point.y) <= radSquared)
			|| (Line2D
			.ptSegDistSq(
				shipLoc.x,
				shipLoc.y,
				endArcRight.x,
				endArcRight.y,
				point.x,
				point.y) <= radSquared);
		return partiallyInArc;
    }
}
