package data.scripts;

import com.fs.starfarer.api.EveryFrameScript;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.MissionBoardAPI;
import com.fs.starfarer.api.campaign.MissionBoardAPI.MissionAvailabilityAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.util.IntervalUtil;
import com.fs.starfarer.api.util.Misc;
import java.util.ArrayList;
import java.util.List;

@SuppressWarnings("unchecked")
public class CommissionEndScript implements EveryFrameScript {
	
	private final MissionBoardAPI board;
	private final IntervalUtil tracker = new IntervalUtil(0.25f, .75f);
	
	public CommissionEndScript() {
		board = Global.getSector().getMissionBoard();
	}	
	
    //EveryFrameScripts must implement these
    @Override
    public boolean isDone()
    {
        return false;
    }

    @Override
    public boolean runWhilePaused()
    {
        return false;
    }

    @Override
    public void advance(float amount) 
    {
		float days = Global.getSector().getClock().convertToDays(amount);
		tracker.advance(days);
		if (tracker.intervalElapsed() && Misc.getCommissionFactionId() != null) {
			// which factions need to have commissions on offer?
			List<FactionAPI> factions = new ArrayList<>();
			for (FactionAPI faction : Global.getSector().getAllFactions()) {
				if(faction.getId().equalsIgnoreCase(Misc.getCommissionFactionId())) factions.add(faction);
			}
			
			// sync missions available with list of what needs to be on offer
			for (MissionAvailabilityAPI ma : board.getMissionsCopy()) {
				if (ma.getMission() instanceof FactionCommissionMissionAnnul) {
					FactionCommissionMissionAnnul fcm = (FactionCommissionMissionAnnul) ma.getMission();
					if (!factions.contains(fcm.getFaction())) {
						board.removeMission(fcm, true);
					} else {
						fcm.setTimestamp(Global.getSector().getClock().getTimestamp());
						factions.remove(fcm.getFaction());
					}
				}
			}
			//Just create this mission for the Faction we're allied with.
			for (FactionAPI faction : factions) {
				createMission(faction);
			}
		}
    }	
	
	protected void createMission(FactionAPI faction) {
		FactionCommissionMissionAnnul mission = new FactionCommissionMissionAnnul(faction.getId());
		for (MarketAPI market : Global.getSector().getEconomy().getMarketsCopy()) {
			if (market.getFaction() == faction) {
				board.makeAvailableAt(mission, market);
			}
		}
	}	
}
