package data.hullmods;

import com.fs.starfarer.api.combat.BaseHullMod;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShieldAPI;
import com.fs.starfarer.api.combat.ShieldAPI.ShieldType;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;

public class ShieldRemoved extends BaseHullMod {

	private float fluxBonus (HullSize hullSize){
		float fluxbonus;
		switch (hullSize) {
			case FIGHTER:
				fluxbonus = 500f;
				break;
			case FRIGATE:
				fluxbonus = 1000f;
				break;
			case DESTROYER:
				fluxbonus = 2000f;
				break;
			case CRUISER:
				fluxbonus = 4000f;
				break;
			case CAPITAL_SHIP:
				fluxbonus = 6000f;
				break;
			default:
				fluxbonus = 0f;
				break;
		}
		return fluxbonus;
	}

	private float disBonus (HullSize hullSize){
		float fluxbonus;
		switch (hullSize) {
			case FIGHTER:
				fluxbonus = 30f;
				break;
			case FRIGATE:
				fluxbonus = 25f;
				break;
			case DESTROYER:
				fluxbonus = 20f;
				break;
			case CRUISER:
				fluxbonus = 15f;
				break;
			case CAPITAL_SHIP:
				fluxbonus = 10f;
				break;
			default:
				fluxbonus = 0f;
				break;
		}
		return fluxbonus;
	}

    @Override
    public void applyEffectsBeforeShipCreation(HullSize hullSize, MutableShipStatsAPI stats, String id) {
		stats.getFluxCapacity().modifyFlat("Shield Removed", fluxBonus(hullSize), "Bonus Flux Capacity due to Shield Removal.");
		stats.getMaxSpeed().modifyFlat("Shield Removed", disBonus(hullSize), "Bonus speed due to Shield Removal.");
    }

	@Override
	public void applyEffectsAfterShipCreation(ShipAPI ship, String id) {
		ShieldAPI shield = ship.getShield();
		if (shield != null) {
			ship.setShield(ShieldType.NONE, 0f, 0f, 0f);
		}
	}

	@Override
	public String getDescriptionParam(int index, HullSize hullSize) {
		if(index == 0){
			return "" + (int) fluxBonus(hullSize);
		}
		if(index == 1){
			return "" + (int) disBonus(hullSize);
		}
		return null;
	}

	@Override
	public boolean isApplicableToShip(ShipAPI ship) {
		return ship != null && (ship.getHullSpec().getDefenseType() != ShieldType.NONE && ship.getHullSpec().getDefenseType() != ShieldType.PHASE);
	}

	@Override
	public String getUnapplicableReason(ShipAPI ship) {
		return "Ship does not have a shield to remove!";
	}
}
