package data.missions.atestforthebest;

import com.fs.starfarer.api.fleet.FleetGoal;
import com.fs.starfarer.api.fleet.FleetMemberType;
import com.fs.starfarer.api.impl.campaign.ids.StarTypes;
import com.fs.starfarer.api.mission.FleetSide;
import com.fs.starfarer.api.mission.MissionDefinitionAPI;
import com.fs.starfarer.api.mission.MissionDefinitionPlugin;

public class MissionDefinition implements MissionDefinitionPlugin {

	@Override
	public void defineMission(MissionDefinitionAPI api) {

		// Set up the fleets so we can add ships and fighter wings to them.
		// In this scenario, the fleets are attacking each other, but
		// in other scenarios, a fleet may be defending or trying to escape
		api.initFleet(FleetSide.PLAYER, "ISS", FleetGoal.ATTACK, false);
		api.initFleet(FleetSide.ENEMY, "EXP", FleetGoal.ATTACK, true);

//		api.getDefaultCommander(FleetSide.PLAYER).getStats().setSkillLevel(Skills.COORDINATED_MANEUVERS, 3);
//		api.getDefaultCommander(FleetSide.PLAYER).getStats().setSkillLevel(Skills.ELECTRONIC_WARFARE, 3);

		// Set a small blurb for each fleet that shows up on the mission detail and
		// mission results screens to identify each side.
		api.setFleetTagline(FleetSide.PLAYER, "We're just testing out some weapons in the Sim, man.");
		api.setFleetTagline(FleetSide.ENEMY, "We're the bad guys in this exercise.");

		// These show up as items in the bulleted list under
		// "Tactical Objectives" on the mission detail screen
		api.addBriefingItem("Live.");

		// Set up the player's fleet.  Variant names come from the
		// files in data/variants and data/variants/fighters
		api.addToFleet(FleetSide.PLAYER, "explorer_society_hawk_default", FleetMemberType.SHIP, "Pride of Ka'alth", true);
		api.addToFleet(FleetSide.PLAYER, "explorer_society_explorer_default", FleetMemberType.SHIP, "Vulcan Mind Probe", false);
		api.addToFleet(FleetSide.PLAYER, "explorer_society_aegean_default", FleetMemberType.SHIP, "Beam Me UP", false);

		// Set up the enemy fleet.
		api.addToFleet(FleetSide.ENEMY, "explorer_society_vespucci_default", FleetMemberType.SHIP, "Way, Way, Way Far Away", false);
		api.addToFleet(FleetSide.ENEMY, "explorer_society_crab_default", FleetMemberType.SHIP, "Crusty", false);
		api.addToFleet(FleetSide.ENEMY, "explorer_society_vespucci_default", FleetMemberType.SHIP, "Biter", false);

		api.defeatOnShipLoss("Pride of Ka'alth");

		// Set up the map.
		float width = 12000f;
		float height = 12000f;
		api.initMap((float)-width/2f, (float)width/2f, (float)-height/2f, (float)height/2f);

		float minX = -width/2;
		float minY = -height/2;

		// Add an asteroid field
		api.addAsteroidField(minX, minY + height / 2, 0, 8000f,
							 20f, 70f, 100);

		api.addPlanet(0, 0, 50f, StarTypes.RED_GIANT, 250f, true);

	}

}
