package data.scripts.plugins;

import com.fs.starfarer.api.combat.BeamAPI;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.CombatEntityAPI;
import com.fs.starfarer.api.combat.DamagingProjectileAPI;
import com.fs.starfarer.api.combat.EveryFrameCombatPlugin;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ViewportAPI;
import com.fs.starfarer.api.combat.DamageType;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.input.InputEventAPI;
import java.awt.Color;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import org.lazywizard.lazylib.MathUtils;

public class FormShieldPlugin implements EveryFrameCombatPlugin
{
	private final float shieldDamage = 0.25f;
	private final float shieldDamageShot = 1f;
	private final float shieldDamageMissile = 1f;
    CombatEngineAPI engine;

	public static HashSet<ShipAPI> reflectingMap = new HashSet<>();
	public static HashSet<ShipAPI> damagedMap = new HashSet<>();

	public static List<BeamAPI> beamList = new ArrayList<>();
	public static List<DamagingProjectileAPI> projList = new ArrayList<>();
	public static List<MissileAPI> seenMissiles = new ArrayList<>();
	public static List<DamagingProjectileAPI> deadProjectiles = new ArrayList<>();

	public static final Color JITTER_UNDER_COLOR = new Color(0,200,255,155);
	public static final Color JITTER_COLOR = new Color(0,200,255,16);


	private float dpsAfterDamageType(DamageType type, float dps){
			switch (type) {
			case ENERGY:
				break;
			case HIGH_EXPLOSIVE:
				dps *= 0.5f;
				break;
			case KINETIC:
				dps *= 1.5f;
				break;
			case FRAGMENTATION:
				dps *= 0.25f;
				break;
			default:
				break;
		}
		return dps;
	}


    @Override
    public void advance(float amount,List<InputEventAPI> events)
    {
		if (engine == null) {clearMe(); return;}
		if (engine.isPaused()) return;
		if(!engine.isInCampaign() && !engine.isInCampaignSim() && !engine.isMission() && !engine.isSimulation()) return;

        for (ShipAPI ship : engine.getShips()) {
			List hullmods = ship.getVariant().getSortedMods();
			if(hullmods.contains("shields_formshield")){
				if (!reflectingMap.contains(ship))
				{
					reflectingMap.add(ship);
					MutableShipStatsAPI stats = ship.getMutableStats();
					stats.getFragmentationDamageTakenMult().modifyFlatAfterMult("formshield", -1f);
					stats.getEnergyDamageTakenMult().modifyFlatAfterMult("formshield", -1f);
					stats.getHighExplosiveDamageTakenMult().modifyFlatAfterMult("formshield", -1f);
					stats.getKineticArmorDamageTakenMult().modifyFlatAfterMult("formshield", -1f);
					stats.getEmpDamageTakenMult().modifyFlatAfterMult("formshield", -1f);
				}
				if(reflectingMap.contains(ship) && ship.getFluxTracker().isOverloadedOrVenting() || ship.isHulk()){
					MutableShipStatsAPI stats = ship.getMutableStats();
					stats.getFragmentationDamageTakenMult().unmodify("formshield");
					stats.getEnergyDamageTakenMult().unmodify("formshield");
					stats.getHighExplosiveDamageTakenMult().unmodify("formshield");
					stats.getKineticArmorDamageTakenMult().unmodify("formshield");
					stats.getEmpDamageTakenMult().unmodify("formshield");
					reflectingMap.remove(ship);
				}
            }
        }

		//Don't bother doing expensive stuff if we don't need to.
		if(reflectingMap.isEmpty()) return;

		beamList = engine.getBeams();

        if(!beamList.isEmpty()){
			for (BeamAPI thisBeam : beamList)
			{
				if(thisBeam.getWeapon() == null) continue;
				if(thisBeam.getWeapon().getDerivedStats() == null) continue;
				CombatEntityAPI thisBeamTarget = thisBeam.getDamageTarget();
				if(thisBeamTarget != null)
				{
					if(thisBeamTarget instanceof ShipAPI)
					{
						ShipAPI shipTarget = (ShipAPI) thisBeamTarget;
						if(reflectingMap.contains(shipTarget))
						{
							float dps;
							if(thisBeam.getWeapon().isBurstBeam()){
								dps = thisBeam.getWeapon().getDerivedStats().getDps() * shieldDamage * amount;
								dpsAfterDamageType(thisBeam.getWeapon().getDamageType(), dps);

							} else {
								dps = thisBeam.getWeapon().getDerivedStats().getDps() * shieldDamage * amount;
								dpsAfterDamageType(thisBeam.getWeapon().getDamageType(), dps);
							}
							shipTarget.getFluxTracker().increaseFlux(dps, true);
							damagedMap.add(shipTarget);
						}
					}
				}
			}
		}

		List<DamagingProjectileAPI> projList2 = engine.getProjectiles();
		//projList2.addAll(engine.getMissiles());
		if(!projList.isEmpty()){
			for (DamagingProjectileAPI thisProj : projList)
			{
				if(thisProj.getWeapon() == null) continue;
				if(thisProj.getWeapon().getDerivedStats() == null) continue;
				if(thisProj.didDamage())
				{
					if(deadProjectiles.contains(thisProj)) continue;
					CombatEntityAPI thisProjTarget = thisProj.getDamageTarget();
					if(thisProj instanceof MissileAPI){
						for(ShipAPI possTarget : engine.getShips()){
							if(reflectingMap.contains(possTarget)){
								//Only do this step on valid ships!  EXPENSIVE!
								if(MathUtils.isPointWithinCircle(thisProj.getLocation(), possTarget.getLocation(), possTarget.getCollisionRadius() + 50f)){
									
									float dps = thisProj.getWeapon().getDerivedStats().getDamagePerShot() * shieldDamageMissile;
									dpsAfterDamageType(thisProj.getWeapon().getDamageType(), dps);
									possTarget.getFluxTracker().increaseFlux(dps, true);
									damagedMap.add(possTarget);
									deadProjectiles.add(thisProj);
								}
							}
						}
					} else {
						if(thisProjTarget instanceof ShipAPI){
							ShipAPI shipTarget = (ShipAPI) thisProjTarget;
							if(reflectingMap.contains(shipTarget))
							{
								float dps = thisProj.getWeapon().getDerivedStats().getDamagePerShot() * shieldDamageShot;
								dpsAfterDamageType(thisProj.getWeapon().getDamageType(), dps);
								shipTarget.getFluxTracker().increaseFlux(dps, true);
								damagedMap.add(shipTarget);
								deadProjectiles.add(thisProj);
							}
						}
					}
				}
			}
		}
		projList.clear();
		projList.addAll(projList2);

		for(ShipAPI damagedShip : damagedMap){
			damagedShip.setJitterUnder(this, JITTER_UNDER_COLOR, 3f, 4, 0f, 5f);
			damagedShip.setJitter(this, JITTER_COLOR, 3f, 4, 0f, 5f);
		}
		damagedMap.clear();
	}

    @Override
    public void init(CombatEngineAPI cEngine) {
        this.engine = cEngine;
		clearMe();
    }

	private void clearMe(){
		reflectingMap.clear();
		beamList.clear();
		projList.clear();
		deadProjectiles.clear();
	}

	@Override
	public void renderInWorldCoords(ViewportAPI vapi) {
	}

	@Override
	public void renderInUICoords(ViewportAPI vapi) {
	}
}
