package data.shipsystems.scripts.ai;

import com.fs.starfarer.api.combat.CollisionClass;
import org.lwjgl.util.vector.Vector2f;

import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipSystemAIScript;
import com.fs.starfarer.api.combat.ShipSystemAPI;
import com.fs.starfarer.api.combat.ShipwideAIFlags;
import com.fs.starfarer.api.util.IntervalUtil;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lazywizard.lazylib.combat.AIUtils;

public class EXP_EnergyRangeBoostAI implements ShipSystemAIScript {
	private ShipAPI ship;
	private CombatEngineAPI engine;
	private ShipSystemAPI system;

	private boolean shouldUseSystem = false;
	private final IntervalUtil tracker = new IntervalUtil(0.5f, 1f);

	@Override
	public void init(ShipAPI theShip, ShipSystemAPI theSystem, ShipwideAIFlags flags, CombatEngineAPI theEngine) {
		ship = theShip;
		engine = theEngine;
		system = theSystem;
	}

    @SuppressWarnings("unchecked")
    @Override
    public void advance(float amount, Vector2f missileDangerDir, Vector2f collisionDangerDir, ShipAPI target) {
        if(engine == null) return;
        if(engine.isPaused()) return;
		if(ship.getFluxTracker().isOverloadedOrVenting()) return;
		if(ship.getCollisionClass().equals(CollisionClass.NONE)) return;
        if(target == null) return;

        tracker.advance(amount);
        if (tracker.intervalElapsed())
        {
            shouldUseSystem = AIUtils.canUseSystemThisFrame(this.ship)
							&& !system.isOn()
							&& !system.isCoolingDown()
							&& !system.isActive()
							&& ship.getFluxTracker().getFluxLevel() < 0.7f
							;

			//Don't do this check unless the above is true, math-heavy
			if(shouldUseSystem){
				float strafeDistance = 1000f * 1000f;
				//Don't use System if we're not aimed at the target and the target's far-enough away!
				float AngleToCollision = VectorUtils.getAngle(ship.getLocation(), target.getLocation());
				float angDiff = getAngleDifference(ship.getFacing(), AngleToCollision);
				float AbsAng = Math.abs(angDiff);
				if(AbsAng > 45f || MathUtils.getDistanceSquared(ship, target) < strafeDistance){
					shouldUseSystem = false;
				}
			}
            // If system is inactive and should be active, enable it
            if(shouldUseSystem == true)
            {
                ship.useSystem();
            }
        }
    }
	
    //Gets angle difference between two angles, returns 0-180 +/-
    private static float getAngleDifference(float angle1, float angle2)
    {
        float distance = (angle2 - angle1) + 180f;
        distance = (distance / 360.0f);
        distance = ((distance - (float) Math.floor(distance)) * 360f) - 180f;
        return distance;
    } 
}
