package data.scripts;

import com.fs.starfarer.api.BaseModPlugin;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.ModSpecAPI;
import com.fs.starfarer.api.PluginPick;

import com.fs.starfarer.api.campaign.CampaignPlugin;
import com.fs.starfarer.api.combat.AutofireAIPlugin;
import com.fs.starfarer.api.combat.DroneLauncherShipSystemAPI;
import com.fs.starfarer.api.combat.MissileAIPlugin;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShipAIPlugin;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipSystemAIScript;
import com.fs.starfarer.api.combat.ShipwideAIFlags;
import com.fs.starfarer.api.combat.WeaponAPI;
import com.fs.starfarer.api.combat.WeaponAPI.WeaponSize;
import com.fs.starfarer.api.combat.WeaponAPI.WeaponType;

import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.loading.WingRole;

import data.scripts.shipai.StandardShipAI;

import data.scripts.weapons.ai.AntiFighterAutoFireAI;
import data.scripts.weapons.ai.BoardingShuttleAutoFireAI;
import data.scripts.weapons.ai.DoNotAimAI;
import data.scripts.weapons.ai.DoNotAimPDAI;
import data.scripts.weapons.ai.DoNotAimStrikeAI;
import data.scripts.weapons.ai.RepairDroneAutoFireAI;
import data.scripts.weapons.ai.StandardAutoFireAI;
import data.scripts.weapons.ai.StandardPDAI;
import data.scripts.weapons.ai.StrikeAutoFireAI;

import static data.scripts.shipai.BestStrafeDistance.BestStrafeDistance;
import data.scripts.weapons.ai.MissileAutoFireAI;
import data.scripts.weapons.ai.ThrusterAI;

import data.scripts.weapons.missileai.GuidedMissileAI_NoDrones;
import data.scripts.weapons.missileai.GuidedMissileAI_SmartPD;
import data.scripts.weapons.missileai.GuidedMissileAI_Strike;
import data.shipsystems.scripts.ai.AlwaysOnAI;
import data.shipsystems.scripts.ai.AmmoFeedAI;
import data.shipsystems.scripts.ai.BurnDriveAI;
import data.shipsystems.scripts.ai.DamperAI;
import data.shipsystems.scripts.ai.DisplacerAI;
import data.shipsystems.scripts.ai.EMPAI;
import data.shipsystems.scripts.ai.EntropyAmpAI;
import data.shipsystems.scripts.ai.FortressShieldAI;

import data.shipsystems.scripts.ai.PhaseCloakAI;
import data.shipsystems.scripts.ai.PhaseTeleporterAI;
import java.util.ArrayList;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Level;
import org.json.JSONArray;
import org.json.JSONObject;

public class SS_Al_ModPlugin extends BaseModPlugin
{
    //public static String REGISTERED_PLUGIN_KEY = "SimpleRebalModPlugin";
	private static final Map MISSILE_TYPE_MAP = new HashMap();
	private static final Map BARREL_MAP = new HashMap();
	private static final Map DRONE_MAP = new HashMap();
	private static final Map SYSTEM_MAP = new HashMap();
	private static final List BLACKLISTED = new ArrayList();

    @Override
    public void onEnabled(boolean wasEnabled) {
    }

    @Override
    public void onNewGame()
    {
    }
	
	private boolean getBlacklisted(ShipAPI ship){
		for(ModSpecAPI mod : Global.getSettings().getModManager().getAvailableModsCopy()){
			try{
				String modID = mod.getId();
				JSONObject thisAr = Global.getSettings().loadJSON("data/ai_config/blacklist.json",modID);
				if(thisAr.has(ship.getHullSpec().getBaseHullId())){
					BLACKLISTED.add(ship.getHullSpec().getBaseHullId());
					return true;					
				}
			} catch(Throwable ex) {}
		}
		return false;
	}	

    private ShipSystemAIScript pickSystemAIScript(ShipAPI ship){
        if(ship.getSystem() == null){
            return null;
		}
		if(BLACKLISTED.contains(ship.getHullSpec().getBaseHullId())) return null;
		if(getBlacklisted(ship)) return null;	

		String baseHullID = ship.getHullSpec().getBaseHullId();
		String shipSystemName = ship.getSystem().getId();
		String systemCodeID = "";
		//Tries to get our values from a Hashmap (fast); if it's not there yet, reads from JSON (slow).
		if(!SYSTEM_MAP.containsKey(baseHullID)){
			for(ModSpecAPI mod : Global.getSettings().getModManager().getAvailableModsCopy()){		
				try {
					String modID = mod.getId();	
					JSONObject json;

					json = Global.getSettings().loadJSON("data/shipsystems/" + shipSystemName + ".system",modID);
					systemCodeID = json.getString("aiScript");
				}catch (Throwable ex) {
					//Global.getLogger(EZFactionModPlugin.class).log(Level.ERROR,"ERROR: couldn't find the EZ Faction directory, etc.", ex);
				}
			}
			SYSTEM_MAP.put(baseHullID, systemCodeID);
		} else {
			systemCodeID = SYSTEM_MAP.get(baseHullID).toString();
		}

		//If we get here, we have found a string that points to the AI we need to use; invoke it.
		if(!systemCodeID.equals("")){
			Class clazz = null;
			try {
				clazz = Class.forName(systemCodeID);
			} catch (ClassNotFoundException ex) {
				Global.getLogger(SS_Al_ModPlugin.class).log(Level.ERROR,shipSystemName + " couldn't find the class for a System", ex);
			}
			if(clazz != null){
				try {
					//Yay, we found a valid AI script, we hope???
					//If it crashes but it got this far, you might want to check your path / package, yo.
					return (ShipSystemAIScript) clazz.newInstance();
				} catch (InstantiationException ex) {
					Global.getLogger(SS_Al_ModPlugin.class).log(Level.ERROR,shipSystemName + " couldn't instantiate class", ex);
				} catch (IllegalAccessException ex) {
					Global.getLogger(SS_Al_ModPlugin.class).log(Level.ERROR,shipSystemName + " illegal access", ex);
				}
			}
		}

		//VANILLA SYSTEM SUPPORT!
		//If we get here, we're dealing with a System that either:
		//A. Doesn't have an aiScript defined, or refers to some internal code we can't see or call...
		//B. Some lazy-ass modder who is using those Vanilla tags rather than writing their own AI.
		//Type B is NOT SUPPORTED AT ALL, because I can't call classes that are obfuscated, duh.
		//This code *also* means that Alex (or mods) can overwrite the .system JSONs (in terms of base stats or whatever)...
		//but they're still supported.
		//Mild future-proofing, basically.

		//Tries to get our values from a Hashmap (fast); if it's not there yet, reads from JSON (slow).
		if(!SYSTEM_MAP.containsKey(baseHullID)){
			for(ModSpecAPI mod : Global.getSettings().getModManager().getAvailableModsCopy()){
				String modID = mod.getId();	
				try {
					JSONObject json;

					json = Global.getSettings().loadJSON("data/shipsystems/" + shipSystemName + ".system",modID);
					systemCodeID = json.getString("aiType");
				}catch (Throwable ex) {
					//Global.getLogger(EZFactionModPlugin.class).log(Level.ERROR,"ERROR: couldn't find the EZ Faction directory, etc.", ex);
				}
			}
			SYSTEM_MAP.put(baseHullID, systemCodeID);
		} else {
			systemCodeID = SYSTEM_MAP.get(baseHullID).toString();
		}

		//Now that we have the aiType field's value... let's put the defaults into place.
		switch (systemCodeID){
			case "BURN_DRIVE":
				return new BurnDriveAI();
			case "WEAPON_BOOST":
				return new AmmoFeedAI();
			case "ACAUSAL_DISRUPTOR":
				return new EntropyAmpAI();
			case "DAMPER_FIELD":
				return new DamperAI();
			case "PHASE_DISPLACER":
				return new DisplacerAI();
			case "EMP":
				return new EMPAI();
			case "NONE":
				return null;
			case "ENTROPY_AMPLIFIER":
				return new EntropyAmpAI();
			case "FAST_RELOAD":
				return new AmmoFeedAI();
			case "FLARE":
				return new EMPAI();
			case "AMMO_RELOAD":
				return new AmmoFeedAI();
			case "FORTRESS_SHIELD":
				return new FortressShieldAI();
			case "INTERDICTOR":
				return new EntropyAmpAI();
			case "MANEUVERING_JETS":
				return new AlwaysOnAI();
			case "PHASE_CLOAK":
				return new PhaseCloakAI();
			case "PHASE_TELEPORTER":
				return new PhaseTeleporterAI();
			case "TEMPORAL_SHELL":
				return new AlwaysOnAI();
			default:
				return null;
		}
    }

    private boolean shouldStrafe(ShipAPI ship){
		//Everything thta isn't a Fighter can strafe.
        if(!ship.isFighter()) return true;
		if(ship.getWing() == null) return true;
		if(ship.getWing().getRole() == null) return true;
		//Bombers don't strafe; fighters do.  Pretty straightforward.
        return ship.getWing().getRole() != WingRole.BOMBER;
    }

    private ShipSystemAIScript pickCloakAIScript(ShipAPI ship){
        if(ship.getPhaseCloak() == null){
            return null;
        } else {
            //If we have a PhaseCloak... use this AI.  Or don't- write a clever custom exception here, if you dare.
            return new PhaseCloakAI();
        }
    }

    @Override
    public PluginPick<ShipAIPlugin> pickShipAI(FleetMemberAPI member, ShipAPI ship) {
		if(BLACKLISTED.contains(ship.getHullSpec().getBaseHullId())) return null;
		if(getBlacklisted(ship)) return null;
		
		//Explicitly sets / unsets the Vanilla AIFlags here
		for(ShipwideAIFlags.AIFlags flag : ShipwideAIFlags.AIFlags.values()){
			ship.getAIFlags().setFlag(flag);
			ship.getAIFlags().unsetFlag(flag);
		}
        //One code to rule them all.
        return new PluginPick(new StandardShipAI(ship, pickSystemAIScript(ship), pickCloakAIScript(ship), shouldStrafe(ship), BestStrafeDistance(ship,false)), CampaignPlugin.PickPriority.MOD_SPECIFIC);
		//return null;
	}


	//This code automatically picks out the "best" use case for a weapon, applying the weapon hints somewhat like Vanilla does.
	//It's not designed to cover every possible screwy use case, obviously, but it's pretty good at weapons that behave like Vanilla.
    @Override
    public PluginPick<AutofireAIPlugin> pickWeaponAutofireAI(WeaponAPI weapon) {
		if(weapon.getShip() == null) return null;
		if(BLACKLISTED.contains(weapon.getShip().getHullSpec().getBaseHullId())) return null;
		if(getBlacklisted(weapon.getShip())) return null;		
		
        //Set up tolerance values for various weapon types, based on Arc restrictions.  Small fix for fixed-mount guns
        float tolerance = 0f;
        if(weapon.getArc() <= 30f || weapon.getSlot().getArc() <= 30f || weapon.getSlot().isHidden() || weapon.getSlot().isHardpoint() || weapon.isBeam()){
            tolerance = Math.max(30f - weapon.getArc(),10f);
        }
		//Slow-fire weapons get low Tolerance, to avoid missing a lot.
        if(weapon.getDerivedStats().getRoF() < 1.0f && !weapon.getType().equals(WeaponType.MISSILE)){
            tolerance = 10f;
        }
		//Spray-and-pray weapons get extra Tolerance.
		if(weapon.getDerivedStats().getRoF() > 8f){
			tolerance += 20f;
		}


        float barrelLength = -500f;
        String weaponID = weapon.getId();
		//Tries to get our values from a Hashmap (fast); if it's not there yet, reads from JSON (slow).
		if(!BARREL_MAP.containsKey(weaponID)){
			for(ModSpecAPI mod : Global.getSettings().getModManager().getAvailableModsCopy()){
				String modID = mod.getId();				
				try {
					JSONObject json;
					json = Global.getSettings().loadJSON("data/weapons/" + weaponID + ".wpn",modID);
					barrelLength = (float) json.getJSONArray("turretOffsets").getLong(0);
				}catch (Throwable ex) {
					//Global.getLogger(EZFactionModPlugin.class).log(Level.ERROR,"ERROR: couldn't find the EZ Faction directory, etc.", ex);
				}
			}
			BARREL_MAP.put(weaponID, barrelLength);
		} else {
			barrelLength = (float) BARREL_MAP.get(weaponID);
		}

        boolean isPD = weapon.hasAIHint(WeaponAPI.AIHints.PD);
        boolean isStrike = weapon.hasAIHint(WeaponAPI.AIHints.STRIKE);
        boolean isAntiFighter = weapon.hasAIHint(WeaponAPI.AIHints.ANTI_FTR);
		boolean isDoNotConserve = weapon.hasAIHint(WeaponAPI.AIHints.DO_NOT_CONSERVE);
        boolean isDoNotAim = weapon.hasAIHint(WeaponAPI.AIHints.DO_NOT_AIM);
        boolean frigateKiller = weapon.hasAIHint(WeaponAPI.AIHints.USE_VS_FRIGATES);
		boolean guidedPoor = weapon.hasAIHint(WeaponAPI.AIHints.GUIDED_POOR);
		boolean conserve_1 = weapon.hasAIHint(WeaponAPI.AIHints.CONSERVE_1);
		boolean conserve_2 = weapon.hasAIHint(WeaponAPI.AIHints.CONSERVE_2);
		boolean conserve_3 = weapon.hasAIHint(WeaponAPI.AIHints.CONSERVE_3);
		boolean conserve_4 = weapon.hasAIHint(WeaponAPI.AIHints.CONSERVE_4);
		boolean conserve_5 = weapon.hasAIHint(WeaponAPI.AIHints.CONSERVE_5);
		boolean heatseeker = weapon.hasAIHint(WeaponAPI.AIHints.HEATSEEKER);
		
		//Missile Launcher AIs
		if(weapon.getType().equals(WeaponType.MISSILE)){
			//REAPERS
			if(conserve_2 && isStrike){
				//Tight aiming requirements, friendly-fire aware, etc.
				//(WeaponAPI weapon, f BarrelLength, f Tolerance, b AmmoConserve, b Strike, b FriendlyFire, b HE, b kinetic)
				return new PluginPick(new MissileAutoFireAI(weapon, barrelLength, 5f, true, true, true, true, false), CampaignPlugin.PickPriority.MOD_SPECIFIC);				
			}
			
			//HAMMERS
			if(conserve_1 && isStrike){
				//Tight aiming requirements, friendly-fire aware, etc.
				//(WeaponAPI weapon, f BarrelLength, f Tolerance, b AmmoConserve, b Strike, b FriendlyFire, b HE, b kinetic)
				return new PluginPick(new MissileAutoFireAI(weapon, barrelLength, 5f, true, true, true, true, false), CampaignPlugin.PickPriority.MOD_SPECIFIC);				
			}	
			
			//HARPOONS
			if(conserve_1 && frigateKiller){
				//Loose aiming requirements; conserves ammo
				//(WeaponAPI weapon, f BarrelLength, f Tolerance, b AmmoConserve, b Strike, b FriendlyFire, b HE, b kinetic)
				return new PluginPick(new MissileAutoFireAI(weapon, barrelLength, 120f, true, false, false, true, false), CampaignPlugin.PickPriority.MOD_SPECIFIC);			
			}				
			
			//ATROPOS
			if(conserve_2 && guidedPoor){
				//Moderate aiming requirements; strike
				//(WeaponAPI weapon, f BarrelLength, f Tolerance, b AmmoConserve, b Strike, b FriendlyFire, b HE, b kinetic)
				return new PluginPick(new MissileAutoFireAI(weapon, barrelLength, 30f, false, true, false, true, false), CampaignPlugin.PickPriority.MOD_SPECIFIC);			
			}				
			
			//SABOT
			if(conserve_3 && guidedPoor){
				//Moderate aiming requirements, kinetic
				//(WeaponAPI weapon, f BarrelLength, f Tolerance, b AmmoConserve, b Strike, b FriendlyFire, b HE, b kinetic)
				return new PluginPick(new MissileAutoFireAI(weapon, barrelLength, 30f, false, false, false, false, true), CampaignPlugin.PickPriority.MOD_SPECIFIC);				
			}				
			
			//SALAMANDERS
			if(isDoNotAim && heatseeker){
				//Weapons that don't aim go here, unless they're Strike.
				return new PluginPick(new DoNotAimAI(weapon, barrelLength, tolerance), CampaignPlugin.PickPriority.MOD_SPECIFIC);				
			}
		}

		//Special for the Thruster type
		if(weapon.getId().contains("thruster") && isDoNotConserve){
			return new PluginPick(new ThrusterAI(weapon, barrelLength, tolerance), CampaignPlugin.PickPriority.MOD_SPECIFIC);
		}

        //Special boarding weapon logic
        if(weapon.getId().contains("boarding")){
            return new PluginPick(new BoardingShuttleAutoFireAI(weapon, barrelLength, tolerance), CampaignPlugin.PickPriority.MOD_SPECIFIC);
        }

        //Special Repair Beam logic
        if(weapon.getId().contains("repair")){
            return new PluginPick(new RepairDroneAutoFireAI(weapon, barrelLength, tolerance), CampaignPlugin.PickPriority.MOD_SPECIFIC);
        }

		//Covers PD that is a missile, like Swarmers are in Rebal Mod
        if(isPD && isDoNotAim){
            return new PluginPick(new DoNotAimPDAI(weapon, barrelLength, tolerance), CampaignPlugin.PickPriority.MOD_SPECIFIC);
        }

        //Has the PD AI installed; forces Smalls to PD if it's not obviously silly.
        if(weapon.getShip().getVariant().getHullMods().contains("pointdefenseai")){
        if(weapon.getSize().equals(WeaponSize.SMALL) && weapon.getType() != WeaponType.MISSILE && weapon.getDerivedStats().getRoF() > 3f || weapon.isBeam() || weapon.isBurstBeam()){
                return new PluginPick(new StandardPDAI(weapon, barrelLength, tolerance), CampaignPlugin.PickPriority.MOD_SPECIFIC);
            }
        }

        if(isPD && !isAntiFighter){
            //PD but not Anti-Fighter; standard PD- ignore manual targets if a higher-priority target shows up
            return new PluginPick(new StandardPDAI(weapon, barrelLength, tolerance), CampaignPlugin.PickPriority.MOD_SPECIFIC);
        } else if (!isPD && isAntiFighter) {
            //This weapon is not PD but is Anti-Fighter; do not fire at missiles, but prioritize Fighters first, before manual targets
            return new PluginPick(new AntiFighterAutoFireAI(weapon, barrelLength, tolerance), CampaignPlugin.PickPriority.MOD_SPECIFIC);
        } else if (isPD && isAntiFighter) {
            //This weapon is both PD and Anti-Fighter; resolve by picking PD behaviors
            return new PluginPick(new StandardPDAI(weapon, barrelLength, tolerance), CampaignPlugin.PickPriority.MOD_SPECIFIC);
        } else if(isDoNotAim && !isStrike){
            //Weapons that don't aim go here, unless they're Strike.
            return new PluginPick(new DoNotAimAI(weapon, barrelLength, tolerance), CampaignPlugin.PickPriority.MOD_SPECIFIC);
        } else if(isDoNotAim && isStrike){
            //If we're a Strike weapon with DO_NOT_AIM, don't fire at Drones and Fighters
            return new PluginPick(new DoNotAimStrikeAI(weapon, barrelLength, tolerance), CampaignPlugin.PickPriority.MOD_SPECIFIC);
        } else if(isStrike || frigateKiller){
            //Anti-Frigate weapons currently just use Strike atm, but this can be changed later.
            return new PluginPick(new StrikeAutoFireAI(weapon, barrelLength, tolerance), CampaignPlugin.PickPriority.MOD_SPECIFIC);
        } else {
            //Catch-all for safety (we should never really reach this); reverts to SS autofire AI
            return new PluginPick(new StandardAutoFireAI(weapon, barrelLength, tolerance), CampaignPlugin.PickPriority.MOD_SPECIFIC);
        }
    }

    @Override
    public PluginPick<ShipAIPlugin> pickDroneAI(ShipAPI ship, ShipAPI mothership, DroneLauncherShipSystemAPI system) {
		//Use Vanilla's AI... for now.
		return null;
    }

    @Override
    public PluginPick<MissileAIPlugin> pickMissileAI(MissileAPI missile,ShipAPI launchingShip)
    {
		//If it's not guided, don't bother, duh
		if(!missile.isGuided()) return null;

		//Gets some basic stats about the missile so that it can make a few generic choices.
        String missileID = missile.getProjectileSpecId();
		int spawnCount = 0;
		float turnRate = 0f;
		float acceleration = 0f;
		float projectileSpawnDist = 0f;
		float projectileSpawnCount = 0f;
		//Populate our Hashmaps with this data (fast), so that we never ever have to get it from JSON (slow) ever again.
		if(!MISSILE_TYPE_MAP.containsKey(missileID)){
			for(ModSpecAPI mod : Global.getSettings().getModManager().getAvailableModsCopy()){
				String modID = mod.getId();				
				try {
					JSONObject json;
					json = Global.getSettings().loadJSON("data/weapons/proj/" + missileID + ".proj",modID);
					JSONObject engineArray = json.getJSONObject("engineSpec");
					acceleration = (float) engineArray.getLong("acc");
					turnRate = (float) engineArray.getLong("turnRate");
					if(json.has("behaviorSpec")){
						JSONObject behaviorArray = json.getJSONObject("behaviorSpec");
						boolean mirv = behaviorArray.getString("behavior").equalsIgnoreCase("mirv");
						if(mirv){
							projectileSpawnDist = (float) behaviorArray.getLong("splitRange");
							projectileSpawnCount = (float) behaviorArray.getLong("numShots");
						}				
					}
					//Global.getLogger(SS_Al_ModPlugin.class).log(Level.ERROR,missileID + "found a missile's basic spec:" + missileID);
				}catch (Throwable ex) {
					//Global.getLogger(EZFactionModPlugin.class).log(Level.ERROR,"ERROR: couldn't find the EZ Faction directory, etc.", ex);
				}
			}

			float[] missileValues;
			missileValues = new float[4];
			missileValues[0] = acceleration;
			missileValues[1] = turnRate;
			missileValues[2] = projectileSpawnDist;
			missileValues[3] = projectileSpawnCount;
			MISSILE_TYPE_MAP.put(missileID, missileValues);
		} else {
			float[] missileValues;
			missileValues = (float[]) MISSILE_TYPE_MAP.get(missileID);
			projectileSpawnDist = missileValues[2];
			projectileSpawnCount = missileValues[3];
		}
		if(projectileSpawnCount > 0.1f){
			spawnCount = Math.round(projectileSpawnCount);
		}
		if(spawnCount > 0){
			//MIRVs aren't supported because we can't get certain data out of 'em... yet.
			return null;
		}
		if(missile.isFromMissile()){
			spawnCount = 0;
			projectileSpawnDist = 0f;
		}

		if(missile.getProjectileSpecId().contains("flare")){
			return new PluginPick(new GuidedMissileAI_SmartPD(missile,0.5f,spawnCount,projectileSpawnDist,false), CampaignPlugin.PickPriority.MOD_SPECIFIC);
		}

		if(missile.getWeapon().hasAIHint(WeaponAPI.AIHints.PD)){
			return new PluginPick(new GuidedMissileAI_SmartPD(missile,0.5f,spawnCount,projectileSpawnDist,false), CampaignPlugin.PickPriority.MOD_SPECIFIC);
		} else if(missile.getWeapon().hasAIHint(WeaponAPI.AIHints.ANTI_FTR)){
			return new PluginPick(new GuidedMissileAI_NoDrones(missile,0.25f,spawnCount,projectileSpawnDist,false), CampaignPlugin.PickPriority.MOD_SPECIFIC);
		} else if(missile.getWeapon().hasAIHint(WeaponAPI.AIHints.GUIDED_POOR)){
			return new PluginPick(new GuidedMissileAI_Strike(missile,0.01f,spawnCount,projectileSpawnDist,false), CampaignPlugin.PickPriority.MOD_SPECIFIC);
		} else if(missile.getWeapon().hasAIHint(WeaponAPI.AIHints.HEATSEEKER)){
			return new PluginPick(new GuidedMissileAI_NoDrones(missile,0.25f,spawnCount,projectileSpawnDist,true), CampaignPlugin.PickPriority.MOD_SPECIFIC);
		} else if(missile.getWeapon().hasAIHint(WeaponAPI.AIHints.STRIKE)){
			return new PluginPick(new GuidedMissileAI_Strike(missile,0.25f,spawnCount,projectileSpawnDist,false), CampaignPlugin.PickPriority.MOD_SPECIFIC);
		} else {
			return new PluginPick(new GuidedMissileAI_NoDrones(missile,0.2f,spawnCount,projectileSpawnDist,false), CampaignPlugin.PickPriority.MOD_SPECIFIC);
		}
    }

    @Override
    public void onApplicationLoad()
    {
        boolean hasLazyLib = Global.getSettings().getModManager().isModEnabled("lw_lazylib");
        if (!hasLazyLib)
        {
            throw new RuntimeException("The AI Overhaul Mod requires LazyLib.");
        }
    }

    @Override
    public void beforeGameSave() {
    }

    @Override
    public void afterGameSave() {
    }
}
