package data.scripts.plugins;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.FleetDataAPI;
import com.fs.starfarer.api.characters.MutableCharacterStatsAPI;
import com.fs.starfarer.api.combat.CollisionClass;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.DamageType;
import com.fs.starfarer.api.combat.DeployedFleetMemberAPI;
import com.fs.starfarer.api.combat.EveryFrameCombatPlugin;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipHullSpecAPI;
import com.fs.starfarer.api.combat.ViewportAPI;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.input.InputEventAPI;
import com.fs.starfarer.api.mission.FleetSide;
import com.fs.starfarer.api.util.IntervalUtil;
import data.scripts.shipai.AIData;
import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.lazywizard.lazylib.MathUtils;
import org.lwjgl.util.vector.Vector2f;

public class ShipThreatPlugin implements EveryFrameCombatPlugin {

    private CombatEngineAPI engine;
    private final IntervalUtil checkInterval = new IntervalUtil(0.1f,0.2f);
    public static Map shipMap = new HashMap();
	public static Map closeShipNotFighterMap = new HashMap();
	public static Map closeShipIncludingFighterMap = new HashMap();
    public static ArrayList hulkList = new ArrayList();
    public static Map friendlyMap = new HashMap();
    public static Map missileMap = new HashMap();
    public static Map shipDest = new HashMap();
    public static Map shipTarget = new HashMap();
    public static Map captureMap = new HashMap();
	public static Map threatMap = new HashMap();

    public static ArrayList getDestination(ShipAPI ship){
        if(shipTarget != null && !shipTarget.isEmpty() && shipTarget.containsKey(ship)){
            ArrayList returnList = new ArrayList();
            ShipAPI target = (ShipAPI) shipTarget.get(ship);
            returnList.add(0,target);
            return returnList;
        }
        if(shipDest != null && !shipDest.isEmpty() && shipDest.containsKey(ship)){
            ArrayList returnList = new ArrayList();
            Vector2f dest = (Vector2f) shipDest.get(ship);
            returnList.add(0,dest);
            return returnList;
        }
        return null;
    }

    public static MissileAPI getClosestMissileThreat(ShipAPI ship){
        if(missileMap.isEmpty()) return null;
        if(!missileMap.containsKey(ship)) return null;

        Vector2f shipLoc = ship.getLocation();
        MissileAPI closest = null;
        float distance, closestDistance = Float.MAX_VALUE;
        List<MissileAPI> tmpList = (List<MissileAPI>) missileMap.get(ship);
		if(tmpList.get(0) instanceof MissileAPI == false) return null;

        for (MissileAPI tmp : tmpList)
        {
            float radiusSquared = tmp.getCollisionRadius() * tmp.getCollisionRadius();
            distance = MathUtils.getDistanceSquared(tmp.getLocation(),shipLoc) - radiusSquared;
            if (distance < closestDistance)
            {
                closest = tmp;
                closestDistance = distance;
            }
        }

        return closest;
    }

    public static ShipAPI getClosestThreat(ShipAPI ship){
		List<ShipAPI> tmpList;
		if(ship.isDestroyer() || ship.isCruiser() || ship.isCapital()){
			tmpList = (List<ShipAPI>) closeShipNotFighterMap.get(ship);
		} else {
			tmpList = (List<ShipAPI>) closeShipIncludingFighterMap.get(ship);
		}
		if(tmpList == null || tmpList.isEmpty()) tmpList = (List<ShipAPI>)shipMap.get(ship);
		if(tmpList == null || tmpList.isEmpty()) return null;
		if(tmpList.get(0) instanceof ShipAPI == false) return null;

        Vector2f shipLoc = ship.getLocation();
        ShipAPI closest = null;
        float distance, closestDistance = Float.MAX_VALUE;
        for (ShipAPI tmp : tmpList)
		{
            float radiusSquared = tmp.getCollisionRadius() * tmp.getCollisionRadius();
            distance = MathUtils.getDistanceSquared(tmp.getLocation(),shipLoc) - radiusSquared;
            if (distance < closestDistance)
            {
                closest = tmp;
                closestDistance = distance;
            }
        }

        return closest;
    }

	public static float getShipThreatLevel(ShipAPI ship){
		float threatLevel = 0f;
		if(!ship.isHulk() && ship.isAlive()){
			if(ship.isFighter()){
				threatLevel += 1f;
			} else if (ship.isFrigate()){
				if(ship.getHullSpec().getHints().contains(ShipHullSpecAPI.ShipTypeHints.CIVILIAN) || ship.getHullSpec().getHints().contains(ShipHullSpecAPI.ShipTypeHints.CARRIER)){
					threatLevel += 2f;
				} else {
					threatLevel += 3f;
				}
				if(ship.getShield() == null){
					threatLevel -= 0.5f;
				}
				if(ship.getFluxTracker().isOverloadedOrVenting()){
					threatLevel -= 1.0f;
				}
			} else if (ship.isDestroyer()){
				if(ship.getHullSpec().getHints().contains(ShipHullSpecAPI.ShipTypeHints.CIVILIAN) || ship.getHullSpec().getHints().contains(ShipHullSpecAPI.ShipTypeHints.CARRIER)){
					threatLevel += 3f;
				} else {
					threatLevel += 5f;
				}
				if(ship.getShield() == null){
					threatLevel -= 1.0f;
				}
				if(ship.getFluxTracker().isOverloadedOrVenting()){
					threatLevel -= 2.5f;
				}
			} else if (ship.isCruiser()) {
				if(ship.getHullSpec().getHints().contains(ShipHullSpecAPI.ShipTypeHints.CIVILIAN) || ship.getHullSpec().getHints().contains(ShipHullSpecAPI.ShipTypeHints.CARRIER)){
					threatLevel += 5f;
				} else {
					threatLevel += 8f;
				}
				if(ship.getShield() == null){
					threatLevel -= 2.0f;
				}
				if(ship.getFluxTracker().isOverloadedOrVenting()){
					threatLevel -= 4.0f;
				}
			} else if (ship.isCapital()){
				if(ship.getHullSpec().getHints().contains(ShipHullSpecAPI.ShipTypeHints.CIVILIAN) || ship.getHullSpec().getHints().contains(ShipHullSpecAPI.ShipTypeHints.CARRIER)){
					threatLevel += 7f;
				} else {
					threatLevel += 12f;
				}
				if(ship.getShield() == null){
					threatLevel -= 3.0f;
				}
				if(ship.getFluxTracker().isOverloadedOrVenting()){
					threatLevel -= 7.0f;
				}
			}
			float enemyLevelAggro;
			FleetSide side;
			if(ship.getOwner() == 0 || ship.getOwner() == 2){
				side = FleetSide.PLAYER;
			} else {
				side = FleetSide.ENEMY;
			}
			MutableCharacterStatsAPI stats = null;
			if(Global.getCombatEngine().getFleetManager(side) != null){
				if(Global.getCombatEngine().getFleetManager(side).getDeployedFleetMember(ship) != null){
					if(Global.getCombatEngine().getFleetManager(side).getDeployedFleetMember(ship).getMember() != null){
						if(Global.getCombatEngine().getFleetManager(side).getDeployedFleetMember(ship).getMember().getFleetCommander() != null){
							stats = Global.getCombatEngine().getFleetManager(side).getDeployedFleetMember(ship).getMember().getFleetCommander().getStats();
						}
					}
				}
			}
			float enemyLevelMult = 1f;
			DeployedFleetMemberAPI deployedEnemy = Global.getCombatEngine().getFleetManager(ship.getOwner()).getDeployedFleetMember(ship);
			if(deployedEnemy != null){
				if(deployedEnemy.getMember() != null){
					if(deployedEnemy.getMember().getCaptain() != null){
						enemyLevelMult = (deployedEnemy.getMember().getCaptain().getStats().getLevel() * 0.03f);
						threatLevel += enemyLevelMult;
					}
				}
			}
			if(stats != null){
				float maxOP = ship.getVariant().getHullSpec().getOrdnancePoints(stats);
				enemyLevelAggro = (maxOP * 0.05f);
				threatLevel += enemyLevelAggro;
			}
			//If the player's in control of this ship, make it less "threatening", and therefore more attractive to attack / focus on
			ShipAPI possPlayer = getPlayerShip();
			if(possPlayer != null){
				if(ship == possPlayer){
					threatLevel *= 0.75f;
				}
			}
		}
		return threatLevel;
	}

	public static ShipAPI getPlayerShip(){
		ShipAPI possPlayer = Global.getCombatEngine().getPlayerShip();
		if(possPlayer != null) return possPlayer;
		return null;
	}

	public static float getTotalThreatValue(ShipAPI ship, List<ShipAPI>threats){
		float level = 0f;
		for(ShipAPI threat : threats){
			level += (float) threatMap.get(threat);
		}
		return level;
	}

	public static ShipAPI getLeastDangerousThreat(ShipAPI ship, List<ShipAPI>threats){
		if(threatMap.isEmpty()) return null;
		if(threats == null) return null;
		if(threats.isEmpty()) return null;
		float threatLevel;
		float lowestThreatLevel = 999999999999f;
		ShipAPI lowestThreat = null;
		for(ShipAPI threat : threats){
			if(threatMap.get(threat) == null) continue;
			threatLevel = (float) threatMap.get(threat);
			float dist = MathUtils.getDistanceSquared(ship, threat);
			//Prioritize closer targets, in terms of "weakness"; beats using Collections.sort() by a long shot here!
			if(dist < (1000f * 1000f) && dist > (750f * 750f)){
				threatLevel *= 0.8;
			} else if (dist <= (750f * 750f)){
				threatLevel *= 0.5;
			}
			if(threatLevel < lowestThreatLevel){
				lowestThreat = threat;
				lowestThreatLevel = threatLevel;
			}
		}
		return lowestThreat;
	}

	public static ShipAPI getMostDangerousThreat(ShipAPI ship, List<ShipAPI>threats){
		//If the player's in control of this ship... adjust
		ShipAPI possPlayer = getPlayerShip();
		if(threatMap.isEmpty()) return null;
		if(threats == null) return null;
		if(threats.isEmpty()) return null;
		float threatLevel;
		float highestThreatLevel = 0f;
		ShipAPI highestThreat = null;
		for(ShipAPI threat : threats){
			threatLevel = (float) threatMap.get(threat);
			if(threat == possPlayer) threatLevel *= 4f;
			if(threatLevel > highestThreatLevel){
				highestThreat = threat;
				highestThreatLevel = threatLevel;
			}
		}
		return highestThreat;
	}

    public static List<ShipAPI> getHulks(){
        if(hulkList.isEmpty()) return null;
        return (List<ShipAPI>) hulkList;
    }

    public static List<ShipAPI> getFriendlies(ShipAPI ship){
        if(friendlyMap.isEmpty()) return null;
        if(!friendlyMap.containsKey(ship)) return null;
        return (List<ShipAPI>) friendlyMap.get(ship);
    }

    public static List<ShipAPI> getThreats(ShipAPI ship){
        if(shipMap.isEmpty()) return null;
        if(!shipMap.containsKey(ship)) return null;
        return (List<ShipAPI>) shipMap.get(ship);
    }

    public static List<ShipAPI> getCloseNotFighterThreats(ShipAPI ship){
        if(closeShipNotFighterMap.isEmpty()) return null;
        if(!closeShipNotFighterMap.containsKey(ship)) return null;
        return (List<ShipAPI>) closeShipNotFighterMap.get(ship);
    }

	public static List<ShipAPI> getCloseThreatsIncludeFighters(ShipAPI ship){
        if(closeShipIncludingFighterMap.isEmpty()) return null;
        if(!closeShipIncludingFighterMap.containsKey(ship)) return null;
        return (List<ShipAPI>) closeShipIncludingFighterMap.get(ship);
    }

    public static List<MissileAPI> getMissiles(ShipAPI ship){
        if(missileMap.isEmpty()) return null;
        if(!missileMap.containsKey(ship)) return null;
        return (List<MissileAPI>) missileMap.get(ship);
    }

    @Override
    public void init(CombatEngineAPI engine) {
        this.engine = engine;
    }

    public static ShipAPI getNearestEnemyToPoint(ShipAPI ship, Vector2f point){
        if(shipMap.isEmpty()) return null;
        if(!shipMap.containsKey(ship)) return null;
        float bestRange = 1000000000f;
        List<ShipAPI> enemies = (List<ShipAPI>)shipMap.get(ship);
        ShipAPI theEnemy = null;
        for(ShipAPI enemy : enemies){
            float range = MathUtils.getDistanceSquared(enemy,point);
            if(range < bestRange){
                theEnemy = enemy;
                bestRange = range;
            }
        }
        return theEnemy;
    }

    public static ShipAPI getNearestFriendlyToPoint(ShipAPI ship, Vector2f point){
        if(friendlyMap.isEmpty()) return null;
        if(!friendlyMap.containsKey(ship)) return null;
        float bestRange = 1000000000f;
        List<ShipAPI> friends = (List<ShipAPI>)friendlyMap.get(ship);
        ShipAPI theFriend = null;
        for(ShipAPI friendly : friends){
            float range = MathUtils.getDistanceSquared(friendly,point);
            if(range < bestRange){
                theFriend = friendly;
                bestRange = range;
            }
        }
        return theFriend;
    }

    public static List<ShipAPI> vacGetNearbyFriendlyShips(ShipAPI ship, float range, List<ShipAPI> theShips){
        List<ShipAPI> friendlies = new ArrayList<>();
        float rangesq = range * range;
        Vector2f loc = ship.getLocation();
        for(ShipAPI posFriend : theShips){
            if(ship == posFriend){
                continue;
            }
            if(posFriend.isAlive() && !posFriend.getCollisionClass().equals(CollisionClass.NONE) && MathUtils.getDistanceSquared(loc,posFriend.getLocation()) < rangesq){
                friendlies.add((ShipAPI)posFriend);
            }
        }
        if(!friendlies.isEmpty()) return friendlies;
        return null;
    }

    public static void vacGetNearbyEnemyShips(ShipAPI ship, float range, List<ShipAPI> theShips){
        ArrayList<ShipAPI> enemies = new ArrayList<>();
		ArrayList<ShipAPI> closeEnemies = new ArrayList<>();
		ArrayList<ShipAPI> closeEnemiesIncludingFighters = new ArrayList<>();
        float rangesq = range * range;
		float closeRangesq = 2000f * 2000f;
        Vector2f loc = ship.getLocation();
        for(ShipAPI posEnemy : theShips){
            if(posEnemy.isAlive() && !posEnemy.getCollisionClass().equals(CollisionClass.NONE) && !posEnemy.isHulk() && Global.getCombatEngine().isEntityInPlay(posEnemy)){
				float dist = MathUtils.getDistanceSquared(loc,posEnemy.getLocation());
				if(dist < rangesq){
					enemies.add((ShipAPI) posEnemy);
				}
				if(dist < closeRangesq){
					closeEnemiesIncludingFighters.add((ShipAPI) posEnemy);
				}
				if(dist < closeRangesq && !posEnemy.isFighter() && !posEnemy.isDrone()){
					closeEnemies.add((ShipAPI) posEnemy);
				}
			}
        }
		if(!enemies.isEmpty()){
			shipMap.put(ship, enemies);
		}
		if(!closeEnemies.isEmpty()){
			closeShipNotFighterMap.put(ship, enemies);
		}
		if(!closeEnemiesIncludingFighters.isEmpty()){
			closeShipIncludingFighterMap.put(ship, enemies);
		}
    }

    public static List<MissileAPI> vacGetNearbyEnemyMissiles(ShipAPI ship, float range, ArrayList<MissileAPI> allMissiles){
        List<MissileAPI> enemies = new ArrayList<>();
        float rangesq = range * range;
        Vector2f loc = ship.getLocation();
        for(MissileAPI posEnemy : allMissiles){
            if(!posEnemy.getCollisionClass().equals(CollisionClass.NONE) && MathUtils.getDistanceSquared(loc,posEnemy.getLocation()) < rangesq){
                enemies.add((MissileAPI) posEnemy);
            }
        }
        if(!enemies.isEmpty()) return enemies;
        return null;
    }

    private void captureStuff(){
        for(ShipAPI target : engine.getShips()){
            if(target.isHulk() && captureMap.containsKey(target)){
                float theseThings[] = (float[]) captureMap.get(target);
                float owner = (float) theseThings[0];
                float timeLeft = (float) theseThings[1] - engine.getElapsedInLastFrame();
                float array[] = new float[2];
                array[0] = owner;
                array[1] = timeLeft;
                captureMap.put(target,array);
                if(timeLeft <= 0f){
                    if(owner < 0.01)
                    {
                        //Here we're checking to make sure we're in a Campaign mission context; if not, halt!
                        if(engine.isInCampaign())
                        {
                            Vector2f loc = target.getLocation();
                            float facing = target.getFacing();

                            String variantID; //Safety code here
                            ShipHullSpecAPI shipHull = target.getHullSpec();
                            String hullName = shipHull.getHullId();
                            variantID = target.getVariant().getHullVariantId();
                            String shipName = target.getVariant().getDesignation();//Get the ship name to prevent duplicates

                            if(variantID.equals("")) variantID = hullName + "_Standard";

                            //Give our fleet the captured ship.
                            ShipAPI spawned = engine.getFleetManager(FleetSide.PLAYER).spawnShipOrWing(variantID, loc, facing);
                            DeployedFleetMemberAPI deployed = engine.getFleetManager(FleetSide.PLAYER).getDeployedFleetMember(spawned);
                            FleetMemberAPI thisMember = deployed.getMember();
                            thisMember.setStatUpdateNeeded(true);
							engine.getFleetManager(FleetSide.PLAYER).getTaskManager(true).orderRetreat(deployed, false, true);
                            //engine.getFleetManager(FleetSide.PLAYER).orderRetreat(deployed,false);
                            if (spawned == null) return;
                            spawned.setCurrentCR(MathUtils.getRandomNumberInRange(0.25f,0.5f));
                            //Sets the name back to the same name
                            spawned.getVariant().setVariantDisplayName(shipName);
                            Global.getSoundPlayer().playSound("boarding_torp_hit", 1.0f, 1.0f, target.getLocation(), target.getVelocity());
                            AIData.captures += 1;

                            //This code just moves the captured enemy ship a long way off the map and blows it up.
                            //We ~must~ blow it up; using remove() results in an un-winnable battle state, because it doesn't count as "dead".
                            loc.x += 100000f;
                            loc.y += 100000f;
                            for(int i = 0; i < 4; i++){
                            engine.spawnEmpArc(target, loc, target, target,
                                               DamageType.ENERGY,
                                               10000000000f,
                                               0f, // emp
                                               10000000000f, // max range
                                               "tachyon_lance_emp_impact",
                                               30f, // thickness
                                               new Color(255,100,0,255),
                                               new Color(255,200,0,255)
                                               );
                            }

                            //If in campaign, we need to keep the ship we've captured after the battle, because ships spawned during the battle aren't...
                            //...being carried over to the cargo at the end of the battle.
                            //This also means that if the player loses the captured ship during the battle, they'll keep it anyway.
                            //Fixing that would probably require some list of captured ships and a post-battle state check...
                            //...or a cyclic script that would remove them if they become Hulks during the battle.
                            //For now, this suffices, even if it's not totally polished.

                            //STOP HERE if we're in the sim!
                            if(!engine.isInCampaign()) return;
                            FleetDataAPI fleet = Global.getSector().getPlayerFleet().getFleetData();
                            fleet.addFleetMember(thisMember);
                            thisMember.getRepairTracker().setMothballed(true);
                        } else {

                            Vector2f loc = target.getLocation();
                            float facing = target.getFacing();

                            String variantID; //Safety code here
                            ShipHullSpecAPI shipHull = target.getHullSpec();
                            String hullName = shipHull.getHullId();
                            variantID = target.getVariant().getHullVariantId();
                            String shipName = target.getVariant().getDesignation();//Get the ship name to prevent duplicates
                            //shipName = shipName + ship.getVariant().getHullSpec().getHullName();

                            if(variantID.equals("")) variantID = hullName + "_Standard";

                            //Give our fleet the captured ship.
                            ShipAPI spawned = engine.getFleetManager(FleetSide.PLAYER).spawnShipOrWing(variantID, loc, facing);
                            DeployedFleetMemberAPI deployed = engine.getFleetManager(FleetSide.PLAYER).getDeployedFleetMember(spawned);
                            FleetMemberAPI thisMember = deployed.getMember();
                            thisMember.setStatUpdateNeeded(true);
							engine.getFleetManager(FleetSide.PLAYER).getTaskManager(true).orderRetreat(deployed, false, true);
                            //engine.getFleetManager(FleetSide.PLAYER).orderRetreat(deployed,false);
                            if (spawned == null) return;
                            spawned.setCurrentCR(MathUtils.getRandomNumberInRange(0.25f,0.5f));
                            //Sets the name back to the same name
                            spawned.getVariant().setVariantDisplayName(shipName);
                            Global.getSoundPlayer().playSound("boarding_torp_hit", 1.0f, 1.0f, target.getLocation(), target.getVelocity());

                            //This code just moves the captured enemy ship a long way off the map and blows it up.
                            //We ~must~ blow it up; using remove() results in an un-winnable battle state, because it doesn't count as "dead".
                            loc.x += 100000f;
                            loc.y += 100000f;
                            engine.spawnEmpArc(target, loc, target, target,
                                               DamageType.ENERGY,
                                               10000000000f,
                                               0f, // emp
                                               10000000f, // max range
                                               "tachyon_lance_emp_impact",
                                               30f, // thickness
                                               new Color(255,100,0,255),
                                               new Color(255,200,0,255)
                                               );

                        }
                    } else {
                        Vector2f loc = target.getLocation();
                        float facing = target.getFacing();
                        String variantID; //Safety code here
                        ShipHullSpecAPI shipHull = target.getHullSpec();
                        String hullName = shipHull.getHullId();

                        variantID = hullName + "_Standard";

                        //Give the enemy fleet the captured ship.
                        engine.getFleetManager(FleetSide.ENEMY).spawnShipOrWing(variantID, loc, facing);
                        Global.getSoundPlayer().playSound("boarding_torp_hit", 1.0f, 1.0f, target.getLocation(), target.getVelocity());

                        //This code just moves the captured enemy ship a long way off the map and blows it up.
                        //We ~must~ blow it up; using remove() results in an un-winnable battle state, because it doesn't count as "dead".
                        loc.x += 100000f;
                        loc.y += 100000f;
                        engine.spawnEmpArc(target, loc, target, target,
                                           DamageType.ENERGY,
                                           10000000000f,
                                           0f, // emp
                                           100000f, // max range
                                           "tachyon_lance_emp_impact",
                                           30f, // thickness
                                           new Color(255,100,0,255),
                                           new Color(255,200,0,255)
                                           );

                    }
                }
            }
        }

    }

    @Override
    public void advance(float amount,   List<InputEventAPI> events) {
        if(Global.getCombatEngine() == null) return;
        if (Global.getCombatEngine().isPaused()) return;

        if(!captureMap.isEmpty()){
            captureStuff();
        }

        checkInterval.advance(amount);
        if(checkInterval.intervalElapsed()
				|| shipMap.isEmpty()
				|| missileMap.isEmpty()
				|| closeShipNotFighterMap.isEmpty()
				|| closeShipIncludingFighterMap.isEmpty()
				|| threatMap.isEmpty()
				){
            List<ShipAPI> allShips = engine.getShips();

            //Clear out our list of ships
            hulkList.clear();
            shipMap.clear();
			closeShipNotFighterMap.clear();
			closeShipIncludingFighterMap.clear();
            friendlyMap.clear();
            missileMap.clear();
			threatMap.clear();

            //Divide up the list into hulks, side 0/1 and Neutrals
            ArrayList<ShipAPI> allShipsZero;
            allShipsZero = new ArrayList<>();
            ArrayList<ShipAPI> allShipsOne;
            allShipsOne = new ArrayList<>();
            ArrayList<ShipAPI> allNeutrals;
            allNeutrals = new ArrayList<>();

            for(ShipAPI ship : allShips){
                int owner = ship.getOwner();
                boolean hulk = ship.isHulk();
                if(!ship.isFighter() && !ship.isDrone() && hulk == true && Global.getCombatEngine().isEntityInPlay(ship)){
                    hulkList.add(ship);
                } else {
					threatMap.put(ship, getShipThreatLevel(ship));
				}
                if((owner == 0 || owner == 2) && ship.isAlive() && Global.getCombatEngine().isEntityInPlay(ship)){
                    allShipsZero.add(ship);
                }else if (owner == 1 && ship.isAlive() && Global.getCombatEngine().isEntityInPlay(ship)){
                    allShipsOne.add(ship);
                } else if (ship.isAlive() && Global.getCombatEngine().isEntityInPlay(ship)){
                    allNeutrals.add(ship);
                }
            }

            //List missiles generated by sides 0/1
            List<MissileAPI> allMissiles = engine.getMissiles();
            ArrayList<MissileAPI> allMissilesZero;
            allMissilesZero = new ArrayList<>();
            ArrayList<MissileAPI> allMissilesOne;
            allMissilesOne = new ArrayList<>();
            for(MissileAPI missile : allMissiles){
                int owner = missile.getOwner();
                if(owner == 1){
                    allMissilesOne.add(missile);
                } else {
                    allMissilesZero.add(missile);
                }
            }

            for(ShipAPI ship : allShips){
				if(ship.isDrone()) continue;
                int owner = ship.getOwner();
				float searchDist = 10000f;
				if (ship.isCapital()) { searchDist = 25000f; }
				else if (ship.isCruiser()) { searchDist = 15000f; }
				//else if (ship.isDestroyer()) { searchDist = 15000f; }
                if(owner != 1){
                    vacGetNearbyEnemyShips(ship,searchDist,allShipsOne);
                } else {
                    vacGetNearbyEnemyShips(ship,searchDist,allShipsZero);
                }

                List<ShipAPI> localFriends;
                //localFriends = new ArrayList<>();
                if(owner != 1){
                    localFriends = vacGetNearbyFriendlyShips(ship,1500f,allShipsZero);
                } else {
                    localFriends = vacGetNearbyFriendlyShips(ship,1500f,allShipsOne);
                }
                if(localFriends != null){
                    friendlyMap.put(ship,localFriends);
                }

                List<MissileAPI> localMissiles;
                //localMissiles = new ArrayList<>();
                if(owner != 1){
                    localMissiles = vacGetNearbyEnemyMissiles(ship,1500f,allMissilesOne);
                } else {
                    localMissiles = vacGetNearbyEnemyMissiles(ship,1500f,allMissilesZero);
                }
                if(localMissiles != null){
                    missileMap.put(ship,localMissiles);
                }
            }
            checkInterval.randomize();
        }
    }

    @Override
    public void renderInWorldCoords(ViewportAPI viewport) {
        //throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public void renderInUICoords(ViewportAPI viewport) {
        //throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
}
