package data.scripts.shipai;

import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.WeaponAPI;
import com.fs.starfarer.api.combat.WeaponAPI.WeaponType;

import java.util.List;

import org.lazywizard.lazylib.MathUtils;

public class BestStrafeDistance {
    //Gets the "best" strafing distance, with a maximum distance that makes some sense.
	//Now averages the ranges to get a better value.
    public static Float BestStrafeDistance(ShipAPI ship, boolean isUnderOrders){
        List<WeaponAPI> weapons = ship.getAllWeapons();
        float totalRange = 0f;
		float weaponRange = 0f;
		int validWeapon = 0;
		boolean foundNonPD = false;
		boolean foundStrikeShortRange = false;
        for (WeaponAPI weapon : weapons) {
			WeaponAPI.WeaponType type = weapon.getSpec().getType();
            if(type != WeaponAPI.WeaponType.DECORATIVE
				){

				//PD weapons are checked for here, to support all-PD special case, below.
				if(!weapon.hasAIHint(WeaponAPI.AIHints.PD)){
					foundNonPD = true;
				}
				
				//Weapon's out of action for now; ignore!
				if(weapon.getCooldownRemaining() > 5f){
					continue;
				}
				
				//Weapon's out of ammo; ignore!
				if(weapon.usesAmmo() && weapon.getAmmo() == 0){
					continue;
				}				
				
				validWeapon += 1;

				//Station Modules should think their range is large
				if(type.equals(WeaponType.STATION_MODULE)){
					totalRange += 2000f;
					continue;
				}
				//Carrier launch bays should increase range
				if(type.equals(WeaponType.LAUNCH_BAY)){
					totalRange += 1500f;
					continue;
				}

				float testRange = 0f;
				switch (type) {
					case BALLISTIC:
						weaponRange = weapon.getRange();
						testRange = ship.getMutableStats().getBallisticWeaponRangeBonus().computeEffective(weapon.getRange());
						if(testRange > weaponRange) weaponRange = testRange;
						break;
					case ENERGY:
						weaponRange = weapon.getRange();
						testRange = ship.getMutableStats().getEnergyWeaponRangeBonus().computeEffective(weapon.getRange());
						if(testRange > weaponRange) weaponRange = testRange;
						break;
					case MISSILE:
						weaponRange = weapon.getRange();
						testRange = ship.getMutableStats().getMissileWeaponRangeBonus().computeEffective(weapon.getRange());
						if(testRange > weaponRange) weaponRange = testRange;
						break;
					default:
						weaponRange = weapon.getRange();
						break;
				}
				if (weaponRange <= 500f) weaponRange = 500f;
				
				totalRange += weaponRange;

				//Catches short-range Strike weapons, so that we can weight them a bit more.
				if(weapon.hasAIHint(WeaponAPI.AIHints.STRIKE)){
					if(weaponRange < 600f){
						foundStrikeShortRange = true;
					}
				}				

            }
        }
		
		//No weapons?  Stay far, far away.
		if(validWeapon <= 0) return 2000f;

		//If all we have are PD weapons and we're not a Fighter, stay out of range!
		if(!ship.isFighter() && !ship.isDrone()){
			if(!foundNonPD)	return 1500f;
		}
		//If we're a Fighter and all we have is PD, get close.
		if(ship.isFighter() || ship.isDrone()){
			if(!foundNonPD)	return 450f;
		}

		//Average the ranges of non-PD weapons here, buffing out a bit to prevent too-short engagement
		totalRange = ((float) totalRange / validWeapon);
		//Shorten the average if we have short-ranged strike guns
		if(foundStrikeShortRange){
			totalRange *= 0.9;
		}
		
        if(totalRange >= 400f){
            //Use some random noise here so that ships don't always stay out at max kiting distances
            float finalRange = Math.max(500f,Math.min(totalRange - MathUtils.getRandomNumberInRange(100,150f),2200f - MathUtils.getRandomNumberInRange(0f,100f)));
			if(isUnderOrders) finalRange = Math.max(500f, finalRange -50f);
			return finalRange;
        } else {
            //Safety catch-all for suckage
            return 800f;
        }
    }
}
