package data.scripts.shipai;

import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipCommand;
import com.fs.starfarer.api.combat.ShipwideAIFlags;
import data.scripts.plugins.ShipThreatPlugin;
import static data.scripts.plugins.ShipThreatPlugin.getTotalThreatValue;
import data.scripts.weapons.ai.WeaponAIUtilities;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lwjgl.util.vector.Vector2f;

public class FluxManager {

	private static void friendlyCheck(ShipAPI ship, float friendlies){
		float threshold;
		boolean friendsAround = false;
		switch (ship.getHullSize()){
			case FRIGATE:
				threshold = 1500f;
				friendsAround = friendlies > 8f;
				break;
			case DESTROYER:
				threshold = 1200f;
				friendsAround = friendlies > 12f;
				break;
			case CRUISER:
				threshold = 1100f;
				friendsAround = friendlies > 14f;
				break;
			case CAPITAL_SHIP:
				threshold = 1000f;
				friendsAround = friendlies > 15f;
				break;
			default:
				threshold = 1500f;
				break;
		}

		ShipAPI nearThreat = ShipThreatPlugin.getClosestThreat(ship);
		if(nearThreat != null){
			if(MathUtils.getDistanceSquared(ship, nearThreat) > (threshold * threshold) && friendsAround){
				if(!ship.getFluxTracker().isVenting())ship.giveCommand(ShipCommand.VENT_FLUX,null,0);
			}
		}
	}

    public static boolean FluxManager(ShipAPI ship, ShipAPI target)
    {
            //FLUX CONTROL
            //If Hard Flux is greater than 75% of max or Soft Flux is greater than 90%, Vent if possible, or get to safety.
			//If we have Flux too high: if friends nearby and enemies aren't too close, back up and Vent;
			//Otherwise, back up and try to Vent later.

			//Really high Flux?  Vent immediately.
			if (ship.getFluxTracker().getHardFlux() >= ship.getFluxTracker().getMaxFlux() * 0.75f || (ship.getFluxTracker().getCurrFlux() >= ship.getFluxTracker().getMaxFlux() * 0.9f)){
				if(!ship.getFluxTracker().isVenting())ship.giveCommand(ShipCommand.VENT_FLUX,null,0);
				if(ship.getAIFlags() != null){
					ship.getAIFlags().setFlag(ShipwideAIFlags.AIFlags.DO_NOT_AUTOFIRE_NON_ESSENTIAL_GROUPS);
				}
				return true;
			}
			//Moderate Flux, but we're far away from enemies, or have friends around?  Vent.
			if(ShipThreatPlugin.getFriendlies(ship) != null && ship.getFluxTracker().getCurrFlux() >= ship.getFluxTracker().getMaxFlux() * 0.6f){
				float friendLevel = getTotalThreatValue(ship,ShipThreatPlugin.getFriendlies(ship));
				friendlyCheck(ship,friendLevel);
				if(ship.getAIFlags() != null){
					ship.getAIFlags().setFlag(ShipwideAIFlags.AIFlags.DO_NOT_AUTOFIRE_NON_ESSENTIAL_GROUPS);
				}
				return true;
			}
			if(ship.getAIFlags() != null){
				ship.getAIFlags().unsetFlag(ShipwideAIFlags.AIFlags.DO_NOT_AUTOFIRE_NON_ESSENTIAL_GROUPS);
			}
			return false;
    }

	//Tests whether a location is in a ship's frontal arc or not
    public static boolean isWithinShieldArc(Vector2f loc, ShipAPI ship)
    {
		Vector2f shipLoc = ship.getLocation();
		float facing = ship.getFacing();

        // Check if weapon is aimed at any part of the target
        return WeaponAIUtilities.getAngleDifference(VectorUtils.getAngle(shipLoc, loc), facing) < 90f;
    }

}
