package data.scripts.shipai;

import com.fs.starfarer.api.combat.DamagingProjectileAPI;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShieldAPI.ShieldType;
import com.fs.starfarer.api.combat.ShipAPI;
import data.scripts.plugins.ShipThreatPlugin;
import static data.scripts.plugins.ShipThreatPlugin.getCloseThreatsIncludeFighters;
import data.scripts.weapons.ai.WeaponAIUtilities;
import org.lazywizard.lazylib.CollisionUtils;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lazywizard.lazylib.combat.CombatUtils;
import org.lwjgl.util.vector.Vector2f;

public class ShieldManager {

	public static boolean missileFound(MissileAPI nearMissile, ShipAPI ship){
		if(nearMissile != null && MathUtils.getDistanceSquared(ship,nearMissile) < (300f * 300f)){
			Vector2f loc = nearMissile.getLocation();
			if(CollisionUtils.getCollides(loc, MathUtils.getPointOnCircumference(loc, 300f, nearMissile.getFacing()), ship.getLocation(), ship.getCollisionRadius())){
				ship.getMouseTarget().set(loc.getX(), loc.getY());
				if(!isWithinShieldArc(loc,ship)){
					ship.getShield().toggleOff();
				}
				return true;
			}
		}
		return false;
	}

    public static void ShieldManager(ShipAPI ship, ShipAPI target, Vector2f destination)
    {
        //DRONE SHIELD CONTROL
        //If shields are down, raise them.
		//If the Drone has a Phase Cloak, let the Phase Cloak manager handle that (Tempest drone, etc.)
        //Also points the shields at the best possible target, period.
        if(ship.isDrone()){
            if(ship.getShield() != null && ship.getPhaseCloak() == null){
                if(ship.getShield().isOff()){
                    ship.getShield().toggleOn();
                }else{
					//Rotates Omni shields to face the target, etc.
                    if(ship.getShield() != null && ship.getShield().getType() == ShieldType.OMNI){
                        MissileAPI nearMissile = ShipThreatPlugin.getClosestMissileThreat(ship);
						if(!missileFound(nearMissile,ship)){
                            ShipAPI nearestEnemy = ShipThreatPlugin.getMostDangerousThreat(ship, getCloseThreatsIncludeFighters(ship));
                            if(nearestEnemy != null && nearestEnemy.isAlive()){
                                Vector2f loc = nearestEnemy.getLocation();
								ship.getMouseTarget().set(loc.getX(), loc.getY());
								if(!isWithinShieldArc(loc,ship)){
									ship.getShield().toggleOff();
								}
                            } else if (target != null) {
								Vector2f loc = target.getLocation();
								ship.getMouseTarget().set(loc.getX(), loc.getY());
								if(!isWithinShieldArc(loc,ship)){
									ship.getShield().toggleOff();
								}
                            } else if (destination != null) {
								Vector2f loc = destination;
								ship.getMouseTarget().set(loc.getX(), loc.getY());
								if(!isWithinShieldArc(loc,ship)){
									ship.getShield().toggleOff();
								}
                            }
                        }
                    }
                }
            }
            //Don't bother continuing.
            return;
        }

        //SHIELD CONTROL
        //If shields are down, raise them, unless enemies aren't in range.
        //Also points the shields at the best possible target, and lowers them if it's going to overload if hit
        if(ship.getShield() != null && !ship.isDrone()){
			ShipAPI nearestEnemy = ShipThreatPlugin.getClosestThreat(ship);
			MissileAPI nearestEnemyMissile = ShipThreatPlugin.getClosestMissileThreat(ship);
			boolean Omni = ship.getShield().getType() == ShieldType.OMNI;
			boolean finished = false;

			//Shuts down the shields on Capital ships if Flux is above half but ship is un-injured.  May backfire in some cases, so there's a catch for low Armor.
			if((ship.isCruiser() || ship.isCapital()) && (ship.getFluxTracker().getHardFlux() / ship.getFluxTracker().getMaxFlux() > 0.6f) && (ship.getHullLevel() >= 0.9f)){
				ship.getShield().toggleOff();
				finished = true;
			}

			//Raise shields, if missiles are near
			if(!finished && nearestEnemyMissile != null && MathUtils.getDistanceSquared(ship,nearestEnemyMissile) < (800f * 800f)){
				 ship.getShield().toggleOn();
				 finished = true;
			}

			//If nearest enemy < 1500 (squared); else keep shields down, for speed
			if(!finished && nearestEnemy != null && MathUtils.getDistanceSquared(ship,nearestEnemy) < (1500f * 1500f)){
				ship.getShield().toggleOn();
				finished = true;
			}
			//Raise shields, if big projectiles are incoming
			float possDamage = 0f;
			if(!finished){
				for(DamagingProjectileAPI shot : CombatUtils.getProjectilesWithinRange(ship.getLocation(),200f + ship.getCollisionRadius()))
				{
					if(shot.getOwner() != ship.getOwner()){
						possDamage += (shot.getDamageAmount() + shot.getEmpAmount()) * ship.getMutableStats().getShieldDamageTakenMult().getModifiedValue();
					}
				}
				if(possDamage + ship.getFluxTracker().getHardFlux() >= ship.getFluxTracker().getMaxFlux() && possDamage > 1000f){
					ship.getShield().toggleOn();
					finished = true;
				}
			}

			//Shut down the shield if no enemies, shots, etc. are nearby
			if(!finished && nearestEnemy != null && MathUtils.getDistanceSquared(ship,nearestEnemy) > (1200f * 1200f)){
				if(possDamage < 500f){
					if(nearestEnemyMissile != null && MathUtils.getDistanceSquared(ship,nearestEnemyMissile) > (800f * 800f)){
						ship.getShield().toggleOff();
						finished = true;
					}
				}
			}

			if(Omni && ship.getShield().isOn()){
				MissileAPI nearMissile = ShipThreatPlugin.getClosestMissileThreat(ship);
				if(!missileFound(nearMissile,ship)){
					if(nearestEnemy != null && nearestEnemy.isAlive()){
						Vector2f loc = nearestEnemy.getLocation();
						ship.getMouseTarget().set(loc.getX(), loc.getY());
						if(!isWithinShieldArc(loc,ship)){
							ship.getShield().toggleOff();
						}
					} else if (target != null && target.isAlive()) {
						Vector2f loc = target.getLocation();
						ship.getMouseTarget().set(loc.getX(), loc.getY());
						if(!isWithinShieldArc(loc,ship)){
							ship.getShield().toggleOff();
						}
					} else if (destination != null) {
						Vector2f loc = destination;
						ship.getMouseTarget().set(loc.getX(), loc.getY());
						if(!isWithinShieldArc(loc,ship)){
							ship.getShield().toggleOff();
						}
					}
				}
			}
        }
    }

	//Tests whether an object is in a shield's arc or not
    public static boolean isWithinShieldArc(Vector2f loc, ShipAPI ship)
    {
		Vector2f shipLoc = ship.getLocation();
		float facing = ship.getShield().getFacing();

        // Check if weapon is aimed at any part of the target
        return WeaponAIUtilities.getAngleDifference(VectorUtils.getAngle(shipLoc, loc), facing) < 90f;
    }

}
