package data.scripts.shipai;

import com.fs.starfarer.api.combat.FluxTrackerAPI;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShieldAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import data.scripts.plugins.ShipThreatPlugin;
import java.util.ArrayList;
import java.util.List;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lwjgl.util.vector.Vector2f;

public class SteerMeStandard {
    public static boolean[] SteerMeStandard(ShipAPI ship, ShipAPI target, float strafeDistance, Vector2f TLoc, Vector2f TVel, boolean kiteable, boolean shouldStrafe, boolean runAway){
        boolean strafeLeft = false;
        boolean strafeRight = false;
        boolean turnRight = false;
        boolean turnLeft = false;
        boolean isAccelerating = true;
        boolean canCircleStrafe = false;
		boolean isDodging = false;
		boolean canKite = kiteable;
		boolean isFleeing = runAway;
        
		//Get ship's velocity and location
		Vector2f MVel = ship.getVelocity();
		Vector2f MLoc = ship.getLocation();
		float MFace = ship.getFacing();
      
		//Get the lead point
		Vector2f TLead;
		if(TVel.length() < 300f){
			TLead = TLoc;
		} else {
			if(target == null){
				TLead = TLoc;
			} else {
				float TCol = target.getCollisionRadius();
				Vector2f TVelShort = VectorDirection(TVel, TCol);
				TLead = new Vector2f(TLoc.getX() + TVelShort.getX(),TLoc.getY() + TVelShort.getY());
			}
		}
      
        //MISSILE AVOIDANCE, FRONT SHIELD
        //Do we have a big missile inbound, and our target is currently far away?  Aim at that, if we have a frontal shield that is not 360. 
        if(ship.getShield() != null && ship.getShield().getType() == ShieldAPI.ShieldType.FRONT  && ship.getShield().getArc() < 359f){
            ShipAPI nearestEnemy = ShipThreatPlugin.getClosestThreat(ship);
            MissileAPI nearestEnemyMissile = ShipThreatPlugin.getClosestMissileThreat(ship);
            if(nearestEnemyMissile != null && nearestEnemy != null  && nearestEnemyMissile.getDamageAmount() > 400f && MathUtils.getDistanceSquared(ship,nearestEnemy) > 640000 && MathUtils.getDistanceSquared(ship,nearestEnemyMissile) < 1000000){
                TLead = nearestEnemyMissile.getLocation();
                //isFleeing = true;
				isDodging = true;
            } else if (nearestEnemyMissile != null && nearestEnemy == null && nearestEnemyMissile.getDamageAmount() > 400f && MathUtils.getDistanceSquared(ship,nearestEnemyMissile) < 1000000){
                TLead = nearestEnemyMissile.getLocation();
                //isFleeing = true;
				isDodging = true;
            }
			if(nearestEnemy != null && MathUtils.getDistanceSquared(ship, nearestEnemy) < 16000f){
				TLead = nearestEnemy.getLocation();
				isDodging = true;
			}
        } 
		
        //MISSILE AVOIDANCE, NO SHIELD
        //Do we have a big missile inbound, and our target is currently far away?  Turn away and go forward, if we don't have a shield at all.
		if((ship.getShield() == null || ship.getShield().getType() == ShieldAPI.ShieldType.NONE)){
            ShipAPI nearestEnemy = ShipThreatPlugin.getClosestThreat(ship);
            MissileAPI nearestEnemyMissile = ShipThreatPlugin.getClosestMissileThreat(ship);
            if((nearestEnemyMissile != null && nearestEnemy != null && nearestEnemyMissile.getDamageAmount() > 400f && MathUtils.getDistanceSquared(ship,nearestEnemy) > 250000) || (nearestEnemyMissile != null && nearestEnemyMissile.getDamageAmount() > 400f) && MathUtils.getDistanceSquared(ship,nearestEnemyMissile) < 1000000){
                Vector2f shipLoc = ship.getLocation();
                Vector2f targetLoc = nearestEnemyMissile.getLocation();
                float shipFace = ship.getFacing();

                //main aiming (to find angle you are off target by)
                float AngleToCollision = VectorUtils.getAngle(shipLoc, targetLoc);
                //Gets the difference in the angle to the enemy vs. the missile's current angle
                float angDiff = getAngleDifference(shipFace, AngleToCollision);
                float AbsAng = Math.abs(angDiff);
                if(AbsAng < 90f){
                    //Strafe and back
                    if(angDiff > 0){
                        strafeRight = true;
                        strafeLeft = false;
                        turnRight = false;
                        turnLeft = false;
                        canCircleStrafe = false;
						isDodging = true;
                    }else{
                        strafeRight = false;
                        strafeLeft = true;                        
                        turnLeft = false;
                        turnRight = false;
                        canCircleStrafe = false;
						isDodging = true;
                    }
                } else if (AbsAng < 160f){
                    //Turn and burn
                    if(angDiff > 0){
                        strafeRight = false;
                        strafeLeft = false;
                        turnRight = true;
                        turnLeft = false;
                        canCircleStrafe = false;
                        isAccelerating = true;
						isDodging = false;
                    }else{
                        strafeRight = false;
                        strafeLeft = false;                        
                        turnLeft = true;
                        turnRight = false;
                        canCircleStrafe = false;
                        isAccelerating = true;
						isDodging = false;
                    }                    
                }               
            }
        }		
		
		//COLLISION AVOIDANCE SYSTEM
		// Basically, this just checks for collisions to our front 90 arc and avoids them, if possible.
		//If Dodging / Fleeing, look behind us, rather than ahead.
		//Get nearby things we should be dodging      
		if(!ship.isFighter() && !ship.isDrone()){
			float bestDistance = 100000000f;
			ArrayList<ShipAPI> possibleTarget = new ArrayList<>();
			//lazylib is convenient, gets everything you want in Searchrange     
			List<ShipAPI> nearbyEnemies = ShipThreatPlugin.getCloseNotFighterThreats(ship); 
			if(nearbyEnemies != null && !nearbyEnemies.isEmpty()){
				for (ShipAPI enemy : nearbyEnemies) {
					//criteria to add to list (you can put anything that will return a true/false here)
					if (enemy.isAlive()  && isBiggerOrEqual(ship,enemy) || ship.getShield() == null)
					{
						float radius = Math.max(125f, enemy.getCollisionRadius() * 1.1f);
						radius = radius * radius;
						float distance = MathUtils.getDistanceSquared(ship,enemy);
						if(radius > distance && distance < bestDistance){
							possibleTarget.add(enemy);  //add to possibles
							bestDistance = distance;
						}
					} 
				}
			}
			List<ShipAPI> nearbyFriendlies = ShipThreatPlugin.getFriendlies(ship);
			if(nearbyFriendlies != null && !nearbyFriendlies.isEmpty()){
				for (ShipAPI friend : nearbyFriendlies) {
					//criteria to add to list (you can put anything that will return a true/false here)
					if (!friend.isFighter() && !friend.isDrone() && !friend.isHulk() && friend.isAlive() && isBiggerOrEqual(ship,friend) || ship.getShield() == null)
					{
						float radius = Math.max(125f, friend.getCollisionRadius() * 1.25f);
						radius = radius * radius;
						float distance = MathUtils.getDistanceSquared(ship,friend);
						if(radius > distance && distance < bestDistance){
							if(friend.getPhaseCloak() != null){
								if(!friend.getPhaseCloak().isOn() || !friend.getPhaseCloak().isActive()){
									possibleTarget.add(friend);  //add to possibles
									bestDistance = distance;
								}
							} else {
								possibleTarget.add(friend);  //add to possibles
								bestDistance = distance;
							}
						}
					}
				}
			}

			//Get the closest instance of a collision target, dodge it.
			if(!possibleTarget.isEmpty()){
				ShipAPI colTarget = (ShipAPI) possibleTarget.get(possibleTarget.size() - 1);
				Vector2f shipLoc = ship.getLocation();
				Vector2f targetLoc = colTarget.getLocation();
				float shipFace = ship.getFacing();

				//main aiming (to find angle you are off target by)
				float AngleToCollision = VectorUtils.getAngle(shipLoc, targetLoc);
				//Gets the difference in the angle to the enemy vs. the missile's current angle
				float angDiff = getAngleDifference(shipFace, AngleToCollision);
				float AbsAng = Math.abs(angDiff);
				if(!isDodging && !isFleeing){
					if(AbsAng < 120f){
						if(angDiff > 0){
							turnRight = true;
							turnLeft = false;
							strafeRight = true;
						}else{
							turnLeft = true;
							turnRight = false;
							strafeLeft = true;
						}
					}
				} else {
					if(AbsAng > 120f){
						if(angDiff > 0){
							turnRight = false;
							turnLeft = true;
							strafeRight = false;
						}else{
							turnLeft = false;
							turnRight = true;
							strafeLeft = false;
						}
					}				
				}
				//We're dodging, skip the next section.
				isDodging = true;
			}
		}//END OF COLLISION DETECTION SYSTEM		
   


		//AIMING AT A TARGET, NOT FLEEING / DODGING
		//////////////////////////////////////////////////////////////////////////////////////////////
		//Strafe a little bit to correct our aim, unless we're dodging.
		//If we're fleeing (trying to Vent) then keep facing the target.
		if(!isDodging){
			//Main aiming logic
			float AngleToEnemy = VectorUtils.getAngle(MLoc, TLead);
			//How closely aligned are we with the target?
			float angleToTarget = getAngleDifference(MFace, AngleToEnemy); 

			float AbsAngD = Math.abs(angleToTarget);		  
		  
			if (AbsAngD > 5)
			{
				//course correct for ship velocity vector (some bugs when severely off target)
				float MFlightAng = VectorUtils.getAngle(new Vector2f(0, 0), MVel);
				float MFlightCC = getAngleDifference(MFace, MFlightAng);
				if (Math.abs(MFlightCC)>20)
				{
					if(MFlightCC < 0){
						strafeLeft = true;
						strafeRight = false;
						canKite = false;
					} else {
						strafeRight = true;
						strafeLeft = false;
						canKite = false;
					}
				}
			}

			//point towards target
			if (AbsAngD >= 0.5)
			{
				if(angleToTarget > 0){
					turnLeft = true;
					turnRight = false;
					if(kiteable){
						canKite = true;
					}
				}else{
					turnRight = true;
					turnLeft = false;
					if(kiteable){
						canKite = true;
					}
				}
			}
			
			//Damp angular velocity if we're getting close to the target angle, to cut down on wiggle
			if(AbsAngD < Math.abs(ship.getAngularVelocity())){
				if(angleToTarget > 0){
					ship.setAngularVelocity(AbsAngD); 
				}else if (angleToTarget < 0){
					ship.setAngularVelocity(-AbsAngD);
				}                
			}

			//Acceleration code; if we're close to our target and we're facing it and it's suitable for kiting, back up and kite a bit.
			if (AbsAngD < 25 && canKite && shouldStrafe == true && MathUtils.getDistanceSquared(TLoc,MLoc) <= (strafeDistance * strafeDistance))
			{
				canCircleStrafe = true;
				isAccelerating = false;
			} else if (AbsAngD > 5 && !canKite && shouldStrafe == true && MathUtils.getDistanceSquared(TLoc,MLoc) <= (strafeDistance * strafeDistance)){
				canCircleStrafe = false;
				isAccelerating = false;
			}else {
				canCircleStrafe = false;
				isAccelerating = true;
			} 			
		}
 
        //Final stages; get whether ship is Overloaded or Venting and back up while wiggling if necessary.
        FluxTrackerAPI tracker = ship.getFluxTracker();
        if(tracker.isOverloadedOrVenting() || isFleeing){
            canCircleStrafe = true;
            isAccelerating = false;
        }      
        
        boolean[] commandList = new boolean[6];
        commandList[0] = turnLeft;
        commandList[1] = turnRight;
        commandList[2] = strafeLeft;
        commandList[3] = strafeRight;
        commandList[4] = canCircleStrafe;
        commandList[5] = isAccelerating;
        return commandList;
    }
	
	public static boolean isBiggerOrEqual(ShipAPI ship, ShipAPI target){
		int shipSize = 0;
		int targetSize = 0;
		if(ship.isFrigate()){
			shipSize = 1;
		} else if (ship.isDestroyer()){
			shipSize = 2;
		} else if (ship.isCruiser()){
			shipSize = 3;
		} else if (ship.isCapital()){
			//Capital ships ~always~ see themselves as "bigger"
			return true;
		}
		if(target.isFrigate()){
			targetSize = 1;
		} else if (target.isDestroyer()){
			targetSize = 2;
		} else if (target.isCruiser()){
			targetSize = 3;
		} else if (target.isCapital()){
			targetSize = 4;
		}
		return targetSize >= shipSize;
	}
    
   //Returns a new vector with original direction but new length
    private static Vector2f VectorDirection(Vector2f vec, float size)
    {
        float x = vec.getX();
        float y = vec.getY();

        float length =(float) Math.hypot(x,y);
        return new Vector2f(x*size/length, y*size/length);
    }           

    //Gets angle difference between two angles, returns 0-180 +/-
    public static float getAngleDifference(float angle1, float angle2)
    {
        float distance = (angle2 - angle1) + 180f;
        distance = (distance / 360.0f);
        distance = ((distance - (float) Math.floor(distance)) * 360f) - 180f;
        return distance;
    }         
}
