package data.scripts.shipai;

import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipHullSpecAPI;
import com.fs.starfarer.api.combat.ShipHullSpecAPI.ShipTypeHints;
import com.fs.starfarer.api.loading.WingRole;

import data.scripts.plugins.ShipThreatPlugin;
import static data.scripts.plugins.ShipThreatPlugin.getLeastDangerousThreat;
import java.util.ArrayList;

import java.util.List;

import org.lazywizard.lazylib.MathUtils;

public class TargetManager {
    public static ShipAPI TargetManager(ShipAPI ship, ShipAPI target, boolean tryingToVent)
    {
		if(ship == null){
			return null;
		}
        //If we have a target and we're close and we're not trying to Vent, don't bother sorting again.
		//If the target is small and we're not, keep moving here, even though that's expensive
		if(!ship.isFighter() && !tryingToVent) {
			if(target != null){
				if(!target.isFighter() && !target.isDrone() && MathUtils.getDistanceSquared(ship,target) < (1000f * 1000f)){return target;}
			}
		}

		if(tryingToVent && target != null){
			return target;
		}

		//If the above isn't satisfied, then it's time to evaluate.
        //Get our nearby enemies and sort them by distance.
		//Optimized here to save considerable amounts of CPU.
		List<ShipAPI> nearbyEnemiesCopy;
		if(ship.isFighter() || ship.isFrigate()){
			nearbyEnemiesCopy = ShipThreatPlugin.getCloseThreatsIncludeFighters(ship);
		} else {
			nearbyEnemiesCopy = ShipThreatPlugin.getCloseNotFighterThreats(ship);
		}
		if(nearbyEnemiesCopy == null || nearbyEnemiesCopy.isEmpty() || !(nearbyEnemiesCopy.get(0) instanceof ShipAPI)){
			nearbyEnemiesCopy = ShipThreatPlugin.getThreats(ship);
		}

        if(nearbyEnemiesCopy != null && !nearbyEnemiesCopy.isEmpty()){
			//String test = nearbyEnemies.get(0).getId();
			//Global.getLogger(TargetManager.class).log(Level.ERROR, "nearbyEnemies is NOT EMPTY OR NULL:  ship " + ship.getName() + " is targetting " + test);
			//Phased ships are sorted out of nearbyEnemies here, to cut down on computation steps later on
			//We make a shallow copy of nearbyEnemies and cut out the Phased ships and other weird cases here.
            //Collections.sort(nearbyEnemiesCopy,new CollectionUtils.SortEntitiesByDistance(ship.getLocation()));
			//Collections.reverse(nearbyEnemiesCopy);

			List<ShipAPI> fighterList = new ArrayList<>();
			List<ShipAPI> frigateList = new ArrayList<>();
			List<ShipAPI> destroyerList = new ArrayList<>();
			List<ShipAPI> cruiserList = new ArrayList<>();
			List<ShipAPI> capitalList = new ArrayList<>();
			List<ShipAPI> civilianList = new ArrayList<>();

			if(!ship.isAlly()){
				ShipAPI playerShip = ShipThreatPlugin.getPlayerShip();
				if(playerShip != null){
					if(ship.getOwner() != playerShip.getOwner() && MathUtils.getDistanceSquared(ship, playerShip) < 5000f * 5000f){
						frigateList.add(playerShip);
						destroyerList.add(playerShip);
						cruiserList.add(playerShip);
						capitalList.add(playerShip);
						civilianList.add(playerShip);
					}
				}
			}

			for (ShipAPI enemy : nearbyEnemiesCopy){
				switch (enemy.getHullSize()){
					case FIGHTER:
						if(!ship.getHullSpec().getHints().contains(ShipTypeHints.CIVILIAN)
						&& !ship.getHullSpec().getHints().contains(ShipTypeHints.CARRIER)){
							fighterList.add(enemy);
						}
						break;
					case FRIGATE:
						if(!ship.getHullSpec().getHints().contains(ShipTypeHints.CIVILIAN)
						&& !ship.getHullSpec().getHints().contains(ShipTypeHints.CARRIER)){
							frigateList.add(enemy);
						}
						break;
					case DESTROYER:
						if(!ship.getHullSpec().getHints().contains(ShipTypeHints.CIVILIAN)
						&& !ship.getHullSpec().getHints().contains(ShipTypeHints.CARRIER)){
							destroyerList.add(enemy);
						}
						break;
					case CRUISER:
						if(!ship.getHullSpec().getHints().contains(ShipTypeHints.CIVILIAN)
						&& !ship.getHullSpec().getHints().contains(ShipTypeHints.CARRIER)){
							cruiserList.add(enemy);
						}
						break;
					case CAPITAL_SHIP:
						if(!ship.getHullSpec().getHints().contains(ShipTypeHints.CIVILIAN)
						&& !ship.getHullSpec().getHints().contains(ShipTypeHints.CARRIER)){
							capitalList.add(enemy);
						}
						break;
				}
				if(ship.getHullSpec().getHints().contains(ShipTypeHints.CIVILIAN)
				|| ship.getHullSpec().getHints().contains(ShipTypeHints.CARRIER)){
					civilianList.add(enemy);
				}
			}

			if( ship.getHullSpec().getHints().contains(ShipHullSpecAPI.ShipTypeHints.CIVILIAN)
				|| ship.getHullSpec().getHints().contains(ShipHullSpecAPI.ShipTypeHints.CARRIER) ){
				if(!frigateList.isEmpty()) return getLeastDangerousThreat(ship,frigateList);
				if(!civilianList.isEmpty()) return getLeastDangerousThreat(ship,civilianList);
				//Catch-all, in case we can't find anything better to do.
				return nearbyEnemiesCopy.get(0);
			}
			if(ship.isFighter()){
				switch (ship.getWing().getRole()){
					case ASSAULT:
						if(!destroyerList.isEmpty()) return getLeastDangerousThreat(ship,destroyerList);
						if(!civilianList.isEmpty()) return getLeastDangerousThreat(ship,civilianList);
						if(!frigateList.isEmpty()) return getLeastDangerousThreat(ship,frigateList);
						if(!cruiserList.isEmpty()) return getLeastDangerousThreat(ship,cruiserList);
						if(!capitalList.isEmpty()) return getLeastDangerousThreat(ship,capitalList);
						//Catch-all, in case we can't find anything better to do.
						return nearbyEnemiesCopy.get(0);
					case BOMBER:
						if(!civilianList.isEmpty()) return getLeastDangerousThreat(ship,civilianList);
						if(!cruiserList.isEmpty()) return getLeastDangerousThreat(ship,cruiserList);
						if(!capitalList.isEmpty()) return getLeastDangerousThreat(ship,capitalList);
						if(!destroyerList.isEmpty()) return getLeastDangerousThreat(ship,destroyerList);
						if(!frigateList.isEmpty()) return getLeastDangerousThreat(ship,frigateList);
						//Catch-all, in case we can't find anything better to do.
						return nearbyEnemiesCopy.get(0);
					case FIGHTER:
						if(!frigateList.isEmpty()) return getLeastDangerousThreat(ship,frigateList);
						if(!fighterList.isEmpty()) return getLeastDangerousThreat(ship,fighterList);
						if(!civilianList.isEmpty()) return getLeastDangerousThreat(ship,civilianList);
						if(!destroyerList.isEmpty()) return getLeastDangerousThreat(ship,destroyerList);
						//Catch-all, in case we can't find anything better to do.
						return nearbyEnemiesCopy.get(0);
					case INTERCEPTOR:
						if(!fighterList.isEmpty()) return getLeastDangerousThreat(ship,fighterList);
						if(!civilianList.isEmpty()) return getLeastDangerousThreat(ship,civilianList);
						if(!frigateList.isEmpty()) return getLeastDangerousThreat(ship,frigateList);
						if(!destroyerList.isEmpty()) return getLeastDangerousThreat(ship,destroyerList);
						//Catch-all, in case we can't find anything better to do.
						return nearbyEnemiesCopy.get(0);
					case SUPPORT:
						if(!destroyerList.isEmpty()) return getLeastDangerousThreat(ship,destroyerList);
						if(!cruiserList.isEmpty()) return getLeastDangerousThreat(ship,cruiserList);
						if(!frigateList.isEmpty()) return getLeastDangerousThreat(ship,frigateList);
						//Catch-all, in case we can't find anything better to do.
						return nearbyEnemiesCopy.get(0);
					default:
						if(!fighterList.isEmpty()) return getLeastDangerousThreat(ship,fighterList);
						if(!frigateList.isEmpty()) return getLeastDangerousThreat(ship,frigateList);
						if(!destroyerList.isEmpty()) return getLeastDangerousThreat(ship,destroyerList);
						//Catch-all, in case we can't find anything better to do.
						return nearbyEnemiesCopy.get(0);
				}
			}
			if(ship.isFrigate()){
				if(!frigateList.isEmpty()) return getLeastDangerousThreat(ship,frigateList);
				if(!destroyerList.isEmpty()) return getLeastDangerousThreat(ship,destroyerList);
				if(!civilianList.isEmpty()) return getLeastDangerousThreat(ship,civilianList);
				//Catch-all, in case we can't find anything better to do.
				return nearbyEnemiesCopy.get(0);
			}
			if(ship.isDestroyer()){
				if(!destroyerList.isEmpty()) return getLeastDangerousThreat(ship,destroyerList);
				if(!frigateList.isEmpty()) return getLeastDangerousThreat(ship,frigateList);
				if(!civilianList.isEmpty()) return getLeastDangerousThreat(ship,civilianList);
				//Catch-all, in case we can't find anything better to do.
				return nearbyEnemiesCopy.get(0);
			}
			if(ship.isCruiser()){
				if(!cruiserList.isEmpty()) return getLeastDangerousThreat(ship,cruiserList);
				if(!destroyerList.isEmpty()) return getLeastDangerousThreat(ship,destroyerList);
				if(!civilianList.isEmpty()) return getLeastDangerousThreat(ship,civilianList);
				if(!frigateList.isEmpty()) return getLeastDangerousThreat(ship,frigateList);
				if(!capitalList.isEmpty()) return getLeastDangerousThreat(ship,capitalList);
				//Catch-all, in case we can't find anything better to do.
				return nearbyEnemiesCopy.get(0);
			}
			if(ship.isCapital()){
				if(!capitalList.isEmpty()) return getLeastDangerousThreat(ship,capitalList);
				if(!cruiserList.isEmpty()) return getLeastDangerousThreat(ship,cruiserList);
				if(!civilianList.isEmpty()) return getLeastDangerousThreat(ship,civilianList);
				if(!destroyerList.isEmpty()) return getLeastDangerousThreat(ship,destroyerList);
				//Catch-all, in case we can't find anything better to do.
				return nearbyEnemiesCopy.get(0);
			}
		}
		//WE SHOULD NEVER GET HERE UNLESS THERE AREN'T ANY TARGETS WITHIN SEARCH RANGE

		//if(finalEnemy != null)Global.getLogger(TargetManager.class).log(Level.ERROR, "nearbyEnemies is NOT EMPTY OR NULL:  ship " + ship.getName() + " is targetting " + finalEnemy.getHullSpec().getBaseHullId());
		//if(finalEnemy != null) return finalEnemy;

		/*
        //Still no target?  Cheat!
		float mapX = Math.abs(Global.getCombatEngine().getMapWidth());
		float mapY = Math.abs(Global.getCombatEngine().getMapHeight());
        nearbyEnemies = Global.getCombatEngine().getShips();
        //Collections.sort(nearbyEnemies,new CollectionUtils.SortEntitiesByDistance(ship.getLocation()));
        for (ShipAPI thisEnemy : nearbyEnemies) {
            //Target any non-Hulk enemy ships that aren't Drones.
            if(((thisEnemy.getOwner() != 1 && ship.getOwner() == 1) || (ship.getOwner() != 1 && thisEnemy.getOwner() == 1))
				&& !thisEnemy.isHulk() && thisEnemy.isAlive()
				&& !thisEnemy.isDrone()
				&& !thisEnemy.isFighter()
				&& Global.getCombatEngine().isEntityInPlay(thisEnemy)
				&& Math.abs(thisEnemy.getLocation().getX()) < mapX && Math.abs(thisEnemy.getLocation().getY()) < mapY
				//&& MathUtils.getDistance(ship, thisEnemy) < 15000f
				){
                return thisEnemy;
            }
        }
		*/
        //Still nothing?  Return a null.  Should never actually happen unless all enemy ships are dead this frame.
        return null;
    }
}
