package data.scripts.shipai;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.characters.PersonalityAPI;
import com.fs.starfarer.api.combat.DeployedFleetMemberAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipHullSpecAPI.ShipTypeHints;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.impl.campaign.ids.Personalities;
import com.fs.starfarer.api.mission.FleetSide;
import static data.scripts.plugins.ShipThreatPlugin.getTotalThreatValue;
import java.util.List;

public class ThreatChecker {
    public static boolean ThreatChecker(ShipAPI ship, List<ShipAPI> nearbyEnemies, List<ShipAPI> nearbyFriends, boolean isUnderOrders)
    {
		if(nearbyEnemies == null) return false;
		if(nearbyEnemies.isEmpty()) return false;

		float threatLevel;
		float threatMult = 0.5f;
		//Attempts to gauge personalities of the Captains and adjusts the global threat accordingly
		DeployedFleetMemberAPI deployed;
		if(ship.getOwner() != 1){
			deployed = Global.getCombatEngine().getFleetManager(FleetSide.PLAYER).getDeployedFleetMember(ship);
		} else {
			deployed = Global.getCombatEngine().getFleetManager(FleetSide.ENEMY).getDeployedFleetMember(ship);
		}
		//float levelAggro = 1.0f;
		if(deployed != null){
			FleetMemberAPI thisMember = deployed.getMember();
			if(thisMember != null && thisMember.getCaptain() != null){
			//levelAggro = 1f - (thisMember.getCaptain().getStats().getLevel() * 0.03f);
			PersonalityAPI capPersonality = thisMember.getCaptain().getPersonalityAPI();
				if(capPersonality != null){
					switch (capPersonality.getId()) {
						case Personalities.RECKLESS:
							threatMult = 0.3f;
							break;
						case Personalities.AGGRESSIVE:
							threatMult = 0.4f;
							break;
						case Personalities.CAUTIOUS:
							threatMult = 0.75f;
							break;
						case Personalities.TIMID:
							threatMult = 1.0f;
							break;
						default:
							break;
					}
				}
			}

			//Takes OPs of the ship into account
			//if(thisMember != null && thisMember.getFleetCommander() != null){
				//MutableCharacterStatsAPI myStat = thisMember.getFleetCommander().getStats();
				//float maxOP = ship.getVariant().getHullSpec().getOrdnancePoints(myStat);
				//levelAggro -= (maxOP * 0.02f);
			//}
		}
		//Civvie / carrier ships are especially afraid of getting into deep combat, regardless of Captains.
		if(ship.getHullSpec().getHints().contains(ShipTypeHints.CIVILIAN) || ship.getHullSpec().getHints().contains(ShipTypeHints.CARRIER)){
			threatMult *= 1.5f;
		}
		//If we don't have a shield, stay at sniping distances; don't get surrounded.
		if(ship.getShield() == null){
			threatMult *= 1.25f;
		}

		threatLevel = getTotalThreatValue(ship,nearbyEnemies) * threatMult;
		if(nearbyFriends != null && !nearbyFriends.isEmpty()){
			threatLevel -= getTotalThreatValue(ship,nearbyFriends) * 0.75f;
		}

		if(isUnderOrders) threatLevel *= 0.5f;

		switch (ship.getHullSize()){
			case FRIGATE:
				if(threatLevel > 10f) return true;
				break;
			case DESTROYER:
				if(threatLevel > 15f) return true;
				break;
			case CRUISER:
				if(threatLevel > 30f) return true;
				break;
			case CAPITAL_SHIP:
				if(threatLevel > 40f) return true;
				break;
			default:
				return false;
		}
		return false;
    }
}
