package data.scripts.weapons.ai;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.AutofireAIPlugin;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipwideAIFlags;
import com.fs.starfarer.api.combat.WeaponAPI;
import com.fs.starfarer.api.util.IntervalUtil;
import data.scripts.plugins.ShipThreatPlugin;
import java.util.ArrayList;
import java.util.List;
import org.lazywizard.lazylib.MathUtils;
import org.lwjgl.util.vector.Vector2f;

public class DoNotAimAI implements AutofireAIPlugin {
    
    private Vector2f theWeaponLoc;
    private final WeaponAPI theWeapon;
    private boolean shouldFire = false;
    private final IntervalUtil checkInterval = new IntervalUtil(0.20f, 0.30f);

    
    //Initial setup.  If waitTime = 0f, the guidance begins one frame after launch.  
    //Otherwise, drifts until waitTime = 0f (useful for anime-style launch systems)
    public DoNotAimAI (WeaponAPI weapon, float theBarrelLength, float theTolerance)
    {
        this.theWeapon = weapon;
    }  
    
    private List<ShipAPI> getEnemiesInRange(WeaponAPI weapon)
    {
        List<ShipAPI> enemies = new ArrayList<>();
        float range = weapon.getRange() * 1.1f;
        
        //Gets all nearby ships that may be valid threats.
        List<ShipAPI> threats = ShipThreatPlugin.getCloseThreatsIncludeFighters(weapon.getShip());
		if(weapon.getShip().isDrone()){
			if(weapon.getShip().getDroneSource() != null){
				threats = ShipThreatPlugin.getCloseThreatsIncludeFighters(weapon.getShip().getDroneSource());
			}
		}			
        if(threats == null || threats.isEmpty()) return null;        
        
        for (ShipAPI ship : threats)
        {
            if (
					MathUtils.getDistanceSquared(theWeaponLoc,ship.getLocation()) <= (range + ship.getCollisionRadius()) * (range + ship.getCollisionRadius())
                ) 
            {
                enemies.add(ship);
            }
        }
        return enemies;
    }      

    @Override
    public void advance(float amount) {
        if(Global.getCombatEngine() == null) return;
        if (Global.getCombatEngine().isPaused()) return;
        //No ammo?  Don't bother.
        if(theWeapon.usesAmmo() && theWeapon.getAmmo() == 0){
            shouldFire = false;
            return;
        }
		
		//Quits firing if the ship's Flux levels are too high.
		if(theWeapon.getShip().getAIFlags() != null){
			if(theWeapon.usesEnergy() && theWeapon.getShip().getAIFlags().hasFlag(ShipwideAIFlags.AIFlags.DO_NOT_AUTOFIRE_NON_ESSENTIAL_GROUPS)){
				shouldFire = false;
				return;
			}		
		}
        
        theWeaponLoc = theWeapon.getLocation();
        
        checkInterval.advance(amount);
        if(checkInterval.intervalElapsed()){
            shouldFire = false;
            checkInterval.randomize();
            List<ShipAPI> nearbyEnemies = getEnemiesInRange(theWeapon);
            if(nearbyEnemies != null && !nearbyEnemies.isEmpty()){
                //for (ShipAPI enemy : nearbyEnemies) {
                    //criteria to add to list (you can put anything that will return a true/false here)
					shouldFire = true;
                //}
            }
        }
    }

    @Override
    public boolean shouldFire() {
        return shouldFire;
    }

    @Override
    public void forceOff() {
        //nothing
    }

    @Override
    public Vector2f getTarget() {
        return null;
    }

    @Override
    public ShipAPI getTargetShip() {       
        return null;
    }

    @Override
    public WeaponAPI getWeapon() {
        return theWeapon;
    }
}

