package data.scripts.weapons.ai;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.AutofireAIPlugin;
import com.fs.starfarer.api.combat.CollisionClass;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipwideAIFlags;
import com.fs.starfarer.api.combat.WeaponAPI;
import com.fs.starfarer.api.util.IntervalUtil;
import data.scripts.plugins.ShipThreatPlugin;
import java.util.List;
import org.lazywizard.lazylib.MathUtils;
import org.lwjgl.util.vector.Vector2f;

public class DoNotAimPDAI implements AutofireAIPlugin {
    
    private Vector2f theWeaponLoc;
    private ShipAPI shipOwner;
    private int weaponSide = 0;
    private float weaponRange = 0f;
    private final WeaponAPI theWeapon;
    private boolean shouldFire = false;
    private final IntervalUtil checkInterval = new IntervalUtil(0.10f, 0.30f);

    
    //Initial setup.  If waitTime = 0f, the guidance begins one frame after launch.  
    //Otherwise, drifts until waitTime = 0f (useful for anime-style launch systems)
    public DoNotAimPDAI (WeaponAPI weapon, float theBarrelLength, float theTolerance)
    {
        this.theWeapon = weapon;
    }  
    
    private boolean areEnemyShipsInRangeOfPoint(Vector2f point, int mySide, ShipAPI owner, float range)
    {
		List<ShipAPI> threats = ShipThreatPlugin.getCloseThreatsIncludeFighters(owner);
		if(owner.isDrone()){
			if(owner.getDroneSource() != null){
				threats = ShipThreatPlugin.getCloseThreatsIncludeFighters(owner.getDroneSource());
			}
		}			
        if(threats == null || threats.isEmpty()) return false;
        for (ShipAPI ship : threats)
        {
            if (MathUtils.getDistanceSquared(point,ship.getLocation()) <= (range + ship.getCollisionRadius()) * (range + ship.getCollisionRadius()))
            {
				return true;
			}
        }
        return false;
    }  
    
    private boolean areEnemyMissilesInRangeOfPoint(Vector2f point, int mySide, ShipAPI owner, float range)
    {
        List<MissileAPI> threats = ShipThreatPlugin.getMissiles(owner);
		if(owner.isDrone()){
			if(owner.getDroneSource() != null){
				threats = ShipThreatPlugin.getMissiles(owner.getDroneSource());
			}
		}			
        if(threats == null || threats.isEmpty()) return false;
        for (MissileAPI missile : threats)
        {
            if (MathUtils.getDistanceSquared(point,missile.getLocation()) <= (range + missile.getCollisionRadius()) * (range + missile.getCollisionRadius())){
                return true;
            }
        }
        return false;
    }     

    @Override
    public void advance(float amount) {
        if(Global.getCombatEngine() == null) return;
        if (Global.getCombatEngine().isPaused()) return;
        //No ammo?  Don't bother.
        if(theWeapon.usesAmmo() && theWeapon.getAmmo() == 0){
            shouldFire = false;
            return;
        }
		
		//Quits firing if the ship's Flux levels are too high.
		if(theWeapon.getShip().getAIFlags() != null){
			if(theWeapon.usesEnergy() && theWeapon.getShip().getAIFlags().hasFlag(ShipwideAIFlags.AIFlags.DO_NOT_AUTOFIRE_NON_ESSENTIAL_GROUPS)){
				shouldFire = false;
				return;
			}
		}

        checkInterval.advance(amount);
        if(checkInterval.intervalElapsed()){
            shouldFire = false;
            checkInterval.randomize();
            
            theWeaponLoc = theWeapon.getLocation();
            shipOwner = theWeapon.getShip();            
            weaponSide = shipOwner.getOwner();
            weaponRange = theWeapon.getRange();
            if(areEnemyShipsInRangeOfPoint(theWeaponLoc,weaponSide,shipOwner,weaponRange) 
				|| areEnemyMissilesInRangeOfPoint(theWeaponLoc,weaponSide,shipOwner,weaponRange)){
				shouldFire = true;
			}
        }
    }

    @Override
    public boolean shouldFire() {
        return shouldFire;
    }

    @Override
    public void forceOff() {
        //nothing
    }

    @Override
    public Vector2f getTarget() {
        return null;
    }

    @Override
    public ShipAPI getTargetShip() {       
        return null;
    }

    @Override
    public WeaponAPI getWeapon() {
        return theWeapon;
    }
}

