package data.scripts.weapons.ai;

import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.WeaponAPI;
import data.scripts.plugins.ShipThreatPlugin;
import java.util.ArrayList;
import java.util.List;
import org.lazywizard.lazylib.CollisionUtils;
import org.lazywizard.lazylib.MathUtils;
import org.lwjgl.util.vector.Vector2f;

public class WeaponAIUtilities {
    //Friendly-Fire Checker
    //Look for friendlies and don't fire if found
    public static boolean friendlyFireCheck(ShipAPI source, Vector2f theWeaponLoc, float theWeaponRange, float theWeaponAngle){
        List<ShipAPI> posFriendlies = ShipThreatPlugin.getFriendlies(source);
		List<ShipAPI> neutrals = ShipThreatPlugin.hulkList;
		List<ShipAPI> friendlies = new ArrayList();
		if(posFriendlies != null){
			for(ShipAPI ship : posFriendlies){
				friendlies.add(ship);
			}
		}
		if(neutrals != null){
			for(ShipAPI ship : neutrals){
				friendlies.add(ship);
			}
		}
        if(source != null && !friendlies.isEmpty()){
            Vector2f offset = MathUtils.getPointOnCircumference(theWeaponLoc,theWeaponRange,theWeaponAngle); 
            for(ShipAPI friendly : friendlies){
                Vector2f friendLoc = friendly.getLocation();      
                //First test, because it's cheaper.
                if(CollisionUtils.isPointWithinCollisionCircle(offset,friendly) ||
                   CollisionUtils.getCollides(theWeaponLoc,
                                    offset,
                                    friendLoc,
                                    friendly.getCollisionRadius())){
                    return false;
                }
            }
        }
        return true;
    }
    
    public static float getEnergyCost(WeaponAPI theWeapon){
            float energyCost = 0f;
            float costOne = theWeapon.getDerivedStats().getFluxPerSecond();
            float costTwo = theWeapon.getDerivedStats().getSustainedFluxPerSecond();
            float costThree = theWeapon.getDerivedStats().getFluxPerDam();
            float costFour = theWeapon.getFluxCostToFire();
            
            if(costOne >= costTwo && costOne >= costThree && costOne >= costFour){
                energyCost = costOne;
            } else if (costTwo >= costOne && costTwo >= costThree && costTwo >= costFour){
                energyCost = costTwo;
            } else if (costThree >= costOne && costThree >= costTwo && costThree >= costFour){
                energyCost = costThree;
            } else {
                energyCost = costFour;
            }
            if(energyCost < 1.0f){
                energyCost = 0.0f;
            }
            return energyCost;
    }
    
    public static Vector2f getFiringSolution(Vector2f source, Vector2f destLocation, Vector2f destVelocity, float projVel)
    {
        float tx = destLocation.getX() - source.getX();
        float ty = destLocation.getY() - source.getY();
        float tvx = destVelocity.getX();
        float tvy = destVelocity.getY();

        // Get quadratic equation components
        float a = tvx*tvx + tvy*tvy - projVel*projVel;
        float b = 2 * (tvx * tx + tvy * ty);
        float c = tx*tx + ty*ty;    

        // Solve quadratic
        Vector2f ts = quad(a, b, c); // See quad(), below

        // Find smallest positive solution
        Vector2f finalSolution = new Vector2f();
        if(ts != null) 
        {
            float t0 = ts.getX();
            float t1 = ts.getY();
            float t = Math.min(t0, t1);
            if (t < 0) t = Math.max(t0, t1);    
            if (t > 0) 
            {
                finalSolution.x = destLocation.getX() + destVelocity.getX() * t;
                finalSolution.y = destLocation.getY() + destVelocity.getY() * t;
            }
        }
        return finalSolution; 
    }
    
    public static Vector2f quad(float a, float b,float c) {
      Vector2f sol = new Vector2f();
      if (Math.abs(a) < 1e-6) {
            if (Math.abs(b) < 1e-6) {
                if(Math.abs(c) < 1e-6){
                    sol.set(0f,0f);
                } else {
                    sol = null;
                }
            } else {
                sol.x = -c / b;
                sol.y = -c / b;
            }
        } else {
            float disc = b*b - 4*a*c;
            if (disc >= 0f) {
                disc = (float) Math.sqrt(disc);
                a = 2f*a;
                sol.set((-b-disc)/a, (-b+disc)/a);
            }
      }
      return sol;
    } 
    
    // Will be in next LazyLib version
    public static float getAngleDifference(float angle1, float angle2)
    {
        float distance = (angle2 - angle1) + 180f;
        distance = (distance / 360.0f);
        distance = ((distance - (float) Math.floor(distance)) * 360f) - 180f;
        return distance;
    }
}
