    package data.scripts.weapons.helperUtils;  
      
    import com.fs.starfarer.api.combat.BoundsAPI;  
    import com.fs.starfarer.api.combat.ShipAPI;
	import java.util.ArrayList;
	import java.util.List;
    import org.lazywizard.lazylib.MathUtils;  
    import org.lwjgl.util.vector.Vector2f;  
	import org.lwjgl.util.vector.Vector4f;
      
    public class CollisionCheck {  
        //Gets the world position of the intersection between our location and a ShipAPI's bounds segment.  
        //Very close to exact.      
        //Returns null if no matches.
        public static Vector2f getBoundsHitExact(Vector2f lineStart, ShipAPI target){  
            Vector2f lineEnd = target.getLocation();  
            BoundsAPI bounds = target.getExactBounds();  
            bounds.update(lineEnd,target.getFacing());
              
            Vector2f closestIntersection = null;
            Vector2f lastIntersection = new Vector2f(-100000,-100000);

            // Convert all segments to lines, do collision checks to find closest hit
            for (BoundsAPI.SegmentAPI tmp : bounds.getSegments())
            {
                Vector2f intersection = intersectSegments(lineStart, lineEnd, tmp.getP1(), tmp.getP2());
                // Collision = true
                if (intersection != null)
                {
                    if (MathUtils.getDistanceSquared(lineStart, intersection)
                        <= MathUtils.getDistanceSquared(lineStart, lastIntersection))
                    {
                        lastIntersection = intersection;
                        closestIntersection = intersection;
                    }
                }
            }
            return closestIntersection;
        }
        
        public static boolean isInBoundsExact(Vector2f lineStart, ShipAPI target){  
            Vector2f lineEnd = target.getLocation();  
            BoundsAPI bounds = target.getExactBounds();  
            bounds.update(lineEnd,target.getFacing());

            // Convert all segments to lines, do collision checks to find closest hit
            for (BoundsAPI.SegmentAPI tmp : bounds.getSegments())
            {
                Vector2f intersection = intersectSegments(lineStart, lineEnd, tmp.getP1(), tmp.getP2());
                // Collision = true
                if (intersection != null)
                {
                    return true;
                }
            }
            return false;
        }        
        
        //Alex's implementation of segment collisions for Java
        private static Vector2f intersectSegments(Vector2f a1, Vector2f a2, Vector2f b1, Vector2f b2) {  
           float denom = (b2.y - b1.y) * (a2.x - a1.x) - (b2.x - b1.x) * (a2.y - a1.y);  
           float numUa = (b2.x - b1.x) * (a1.y - b1.y) - (b2.y - b1.y) * (a1.x - b1.x);  
           float numUb = (a2.x - a1.x) * (a1.y - b1.y) - (a2.y - a1.y) * (a1.x - b1.x);  

           if (denom == 0 && !(numUa == 0 && numUb == 0)) { // parallel, not coincident  
              return null;  
           }  

           if (denom == 0 && numUa == 0 && numUb == 0) { // coincident  
              float minX, minY, maxX, maxY;  
              if (a1.x < a2.x) {  
                 minX = a1.x;  
                 maxX = a2.x;  
              } else {  
                 minX = a2.x;  
                 maxX = a1.x;  
              }  
              if (a1.y < a2.y) {  
                 minY = a1.y;  
                 maxY = a2.y;  
              } else {  
                 minY = a2.y;  
                 maxY = a1.y;  
              }  
              // if either one of the endpoints in segment b is between the points in segment a,  
              // return that endpoint as the intersection.  Otherwise, no intersection.  
              if (b1.x >= minX && b1.x <= maxX && b1.y >= minY && b1.y <= maxY) {  
                 return new Vector2f(b1);  
              } else if (b2.x >= minX && b2.x <= maxX && b2.y >= minY && b2.y <= maxY) {  
                 return new Vector2f(b2);  
              } else {  
                 return null;  
              }  
           }  

           float Ua = numUa / denom;  
           float Ub = numUb / denom;  
           if (Ua >=0 && Ua <= 1 && Ub >= 0 && Ub <= 1) { // segments intersect  
              Vector2f result = new Vector2f();  
              result.x = a1.x + Ua * (a2.x - a1.x);  
              result.y = a1.y + Ua * (a2.y - a1.y);  
              return result;  
           } else { // lines intersect, but segments do not  
              return null;  
           }  

        }
        
        //Returns Vector4f
        public static Vector4f getClosestCellLocationToPoint(Vector2f point,ShipAPI target){
            Vector4f closestPoint = null;
            
            List<Vector4f> gridLocs = getCellLocations(target);
            float currentBestDistance = 1000000000f;

            for(Vector4f gridPoint : gridLocs){
                float distance = MathUtils.getDistanceSquared(point,new Vector2f (gridPoint.x, gridPoint.y));
                if(distance < currentBestDistance){
                    closestPoint = gridPoint;
                    currentBestDistance = distance;
                }
            }
            return closestPoint;
        }
        
        
        //Sundog's cell locations finder
        public static Vector2f getCellLocation(ShipAPI ship, float x, float y) {  
            x -= ship.getArmorGrid().getGrid().length / 2f;  
            y -= ship.getArmorGrid().getGrid()[0].length / 2f;  
            float cellSize = ship.getArmorGrid().getCellSize();  
            Vector2f cellLoc = new Vector2f();  
            float theta = (float)(((ship.getFacing() - 90) / 360f) * (Math.PI * 2));  
            cellLoc.x = (float)(x * Math.cos(theta) - y * Math.sin(theta)) * cellSize + ship.getLocation().x;  
            cellLoc.y = (float)(x * Math.sin(theta) + y * Math.cos(theta)) * cellSize + ship.getLocation().y;  

            return cellLoc;  
        }         
        
        public static List<Vector4f> getCellLocations(ShipAPI ship) {  
            int width = ship.getArmorGrid().getGrid().length;  
            int height = ship.getArmorGrid().getGrid()[0].length;  
            //List cellLocations = new ArrayList(width * height);  
            // Not sure if above works the way I think it does.  
            List cellLocations = new ArrayList();  

            for(int x = 0; x < width; ++x) {  
                for(int y = 0; y < height; ++y) {  
                    Vector2f loc = getCellLocation(ship, x, y);
                    cellLocations.add(new Vector4f (loc.x, loc.y, (float) x, (float) y));  
                }             
            }
            return cellLocations;  
        } 
    }  