package data.scripts.weapons.helperUtils;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.CombatEntityAPI;
import com.fs.starfarer.api.combat.DamageType;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.WeaponAPI;
import java.awt.Color;
import java.awt.geom.Line2D;
import java.util.ArrayList;
import java.util.List;
import org.lazywizard.lazylib.CollisionUtils;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lwjgl.util.vector.Vector2f;
import org.lwjgl.util.vector.Vector4f;


public class VacuumWeaponUtils {  
    private static final float maxDrawDist = 2500 * 2500;
    
    public static boolean vacuumDestroyThisThingOrNot(CombatEngineAPI engine, CombatEntityAPI target, float damage){
        if(damage >= target.getHitpoints()){
            Vector2f screenLoc = engine.getViewport().getCenter();
            if(target instanceof ShipAPI){
                ShipAPI ship = (ShipAPI) target;
                engine.applyDamage
                (
                    ship, //dead Entity
                    ship.getLocation(), //Our 2D vector to the exact world-position of the collision
                    10000000000f, //Insta-death!
                    DamageType.ENERGY, //Using the damage type here, so that Kinetic / Explosive / Fragmentation AOE works.
                    10000000000f, //EMP 
                    true, //Ignores shields
                    false, //Does not do Soft Flux damage (unless you want it to for some strange reason)
                    ship  //The ship owns the damage
                );
                engine.applyDamage
                (
                    ship, //dead Entity
                    ship.getLocation(), //Our 2D vector to the exact world-position of the collision
                    10000000000f, //Insta-death!
                    DamageType.ENERGY, //Using the damage type here, so that Kinetic / Explosive / Fragmentation AOE works.
                    10000000000f, //EMP 
                    true, //Ignores shields
                    false, //Does not do Soft Flux damage (unless you want it to for some strange reason)
                    ship  //The ship owns the damage
                ); 
            } else {
                if(MathUtils.getDistanceSquared(target.getLocation(),screenLoc) < maxDrawDist){
                    doExplosion(engine, target.getLocation(), Math.min(5000f,target.getMaxHitpoints()));
                }                
                engine.removeEntity(target);
            }
            return true;
        }
        return false;
    }
    
    private static Vector2f randomCircularVelocity(float velocity)
    {
            Vector2f newVel = MathUtils.getRandomPointOnCircumference(null, velocity);
            return newVel;
    }    
    
    private static void doExplosion(CombatEngineAPI engine, Vector2f point, float dam) 
    {
        int repeat = Math.max(3,(int) (dam / 200f));
        for(int i = 0; i < repeat; i++)
        {
            Vector2f randPoint = MathUtils.getRandomPointInCircle(point,dam/35f);
            float randSize = MathUtils.getRandomNumberInRange(dam/25f,Math.max(dam / 20f,15f));
            Vector2f randVec = randomCircularVelocity(MathUtils.getRandomNumberInRange(30f,Math.max(dam / 50f,31f)));
            float randDur = 0.1f + MathUtils.getRandomNumberInRange(0.0f,0.3f);
            int yelVal = (int) (Math.random() * 200f + 32f);
            int randTrans = (int) MathUtils.getRandomNumberInRange(64f,128f);

            engine.addSmoothParticle(randPoint, randVec, randSize, 1f, randDur, new Color(255,yelVal,0,randTrans));
        }
        for(int i = 0; i < repeat; i++)
        {

            int yelVal = (int) (Math.random() * 128f + 64f);

            //Random Vectors, in order of speed; generally, we need smoke and central poof to be slower than others
            Vector2f randVecFast = randomCircularVelocity(MathUtils.getRandomNumberInRange(30f,Math.max(dam / 2f,31f)));
            Vector2f randVecFastTwo = randomCircularVelocity(MathUtils.getRandomNumberInRange(30f,Math.max(dam / 2f,31f)));
            Vector2f randVec = randomCircularVelocity(MathUtils.getRandomNumberInRange(30f,Math.max(dam / 25f,31f)));
            Vector2f randVecTwo = randomCircularVelocity(MathUtils.getRandomNumberInRange(30f,Math.max(dam / 35f,31f)));
            Vector2f randVecThree = randomCircularVelocity(MathUtils.getRandomNumberInRange(15f,Math.max(dam / 40f,16f)));

            float randSize = MathUtils.getRandomNumberInRange(10f,Math.max(dam / 50f,15f));
            float randSizeTwo = MathUtils.getRandomNumberInRange(5f,10f);
            float randDur = 1f + MathUtils.getRandomNumberInRange(-0.5f,3f);
            float randDurTwo = MathUtils.getRandomNumberInRange(0.5f,1f);
            int randTrans = (int) MathUtils.getRandomNumberInRange(32f,200f);
            int randGray = (int) MathUtils.getRandomNumberInRange(32f,64f);

            engine.addHitParticle(point, randVecFast, randSizeTwo, 1f, randDurTwo * 0.5f, new Color(255,yelVal + 64,0,randTrans + 55));
            engine.addHitParticle(point, randVecFastTwo, randSizeTwo, 1f, randDurTwo * 0.65f, new Color(255,yelVal + 64,0,randTrans + 55));
            engine.addHitParticle(point, randVec, randSize, 1f, randDurTwo, new Color(255,yelVal-64,0,randTrans));
            engine.addHitParticle(point, randVecTwo, randSize, 1f, randDurTwo * 0.75f, new Color(255,yelVal-32,0,randTrans));
            engine.addHitParticle(point, randVecThree, randSize, 1f, randDurTwo * 0.5f, new Color(255,yelVal,0,randTrans));

            engine.addSmokeParticle(point, randVecThree, MathUtils.getRandomNumberInRange(dam / 100f,dam / 50f), MathUtils.getRandomNumberInRange(0.5f,1f), randDur, new Color(randGray,randGray,randGray,randTrans));
        }	
    }
    
    public static void vacuumDamageThisEntity(Vector2f loc, CombatEntityAPI enemy, float damage, float emp, DamageType damageType, CombatEngineAPI engine, ShipAPI owner){
        Vector2f point;
        if(enemy instanceof ShipAPI == false){
            if(!vacuumDestroyThisThingOrNot(engine, enemy, damage)){
                point = enemy.getLocation();
                engine.applyDamage(
                      enemy, //enemy Entity
                      point, //Our 2D vector to the exact world-position of the collision
                      damage, //DPS modified by the damage multiplier
                      damageType, //Using the damage type here, so that Kinetic / Explosive / Fragmentation AOE works.
                      emp, //EMP 
                      false, //Does not bypass shields.
                      false, //Does not do Soft Flux damage (unless you want it to for some strange reason)
                      owner  //Who owns this projectile?
                );   
            }
        } else {
            ShipAPI ship = (ShipAPI) enemy;
            //If not already in the bounds... work our way into the center of the ship.
            if(CollisionUtils.isPointWithinBounds(loc,enemy)) {
                point = loc;
            } else {
                point = CollisionUtils.getCollisionPoint(loc,enemy.getLocation(),enemy);
                if(point == null) point = CollisionCheck.getBoundsHitExact(loc,(ShipAPI) enemy);
                boolean collided = false;
                if(point == null){
                    point = loc;
                    float angle = VectorUtils.getAngle(point,ship.getLocation());
                    for(int i = 0; i < 5; i++){                    
                        if(!collided && CollisionUtils.isPointWithinBounds(point,enemy) == false){
                            float distance = MathUtils.getDistance(point,enemy.getLocation());
                            point = MathUtils.getPointOnCircumference(point,distance * 0.95f,angle);
                        } else {
                            collided = true;
                        }
                    }
                }
                if(!collided){
                    Vector4f intersection = CollisionCheck.getClosestCellLocationToPoint(point,ship);
                    point = new Vector2f(intersection.x,intersection.y);
                }
            }
            engine.applyDamage(
                  enemy, //enemy Entity
                  point, //Our 2D vector to the exact world-position of the collision
                  damage, //DPS modified by the damage multiplier
                  damageType, //Using the damage type here, so that Kinetic / Explosive / Fragmentation AOE works.
                  emp, //EMP 
                  false, //Does not bypass shields.
                  false, //Does not do Soft Flux damage (unless you want it to for some strange reason)
                  owner  //Who owns this projectile?
            );            
        }
    }    
    
    //Rewrite of LazyWizard's IsWithinArc
    public static boolean vacuumIsWithinArc(CombatEntityAPI entity, WeaponAPI weapon, Vector2f weaponLoc, float weaponRange, float weaponArc, float weaponAngle)
    {
        // Check if weapon is in range
        if (!MathUtils.isWithinRange(entity, weaponLoc, weaponRange))
        {
            return false;
        }

        // Check if weapon is aimed at the target's center
        if (weapon.distanceFromArc(entity.getLocation()) == 0)
        {
            return true;
        }

        // Check if weapon is aimed at any part of the target
        Vector2f endArcLeft = MathUtils.getPointOnCircumference(weaponLoc, weaponRange,
                weaponAngle - weaponArc);
        Vector2f endArcRight = MathUtils.getPointOnCircumference(weaponLoc, weaponRange,
                weaponAngle + weaponArc);
        float radSquared = entity.getCollisionRadius() * entity.getCollisionRadius();
        Vector2f target = entity.getLocation();
        boolean partiallyInArc = (Line2D
                .ptSegDistSq(
                        weaponLoc.x,
                        weaponLoc.y,
                        endArcLeft.x,
                        endArcLeft.y,
                        target.x,
                        target.y) <= radSquared)
                || (Line2D
                .ptSegDistSq(
                        weaponLoc.x,
                        weaponLoc.y,
                        endArcRight.x,
                        endArcRight.y,
                        target.x,
                        target.y) <= radSquared);
        return partiallyInArc;
    }
    
    //Returns one of the nearest enemy MissileAPI objects to a point.  Not guaranteed to get the nearest object, for speed purposes.
    public static MissileAPI getNearestMissileNearPoint(int owner, Vector2f point, float range)
    {
        List<MissileAPI> enemies = new ArrayList<>();
        for (MissileAPI missile : Global.getCombatEngine().getMissiles())
        {
            if (    
                    missile.getOwner() != owner
                    && MathUtils.isWithinRange(missile,missile,range)
                )
            {
                enemies.add(missile);
            }
        }
        if(!enemies.isEmpty()){
            return enemies.get(0);
        } else {
            return null;
        }
    } 
    
}