package data.shipsystems.scripts.ai;

import com.fs.starfarer.api.combat.CollisionClass;
import org.lwjgl.util.vector.Vector2f;

import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipSystemAIScript;
import com.fs.starfarer.api.combat.ShipSystemAPI;
import com.fs.starfarer.api.combat.ShipwideAIFlags;
import com.fs.starfarer.api.util.IntervalUtil;
import static data.scripts.shipai.BestStrafeDistance.BestStrafeDistance;
import data.scripts.shipai.SteerMeStandard;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lazywizard.lazylib.combat.AIUtils;

public class AmmoFeedAI implements ShipSystemAIScript {

    private ShipAPI ship;
    private CombatEngineAPI engine;
    private ShipSystemAPI system;

    private boolean shouldUseSystem = false;
    private final IntervalUtil tracker = new IntervalUtil(0.35f, 0.5f);

    @Override
    public void init(ShipAPI ship, ShipSystemAPI system, ShipwideAIFlags flags, CombatEngineAPI engine) {
        this.ship = ship;
        this.engine = engine;
        this.system = system;
    }

    @Override
    public void advance(float amount, Vector2f missileDangerDir, Vector2f collisionDangerDir, ShipAPI target)
    {
        if(engine == null) return;
        if(engine.isPaused()) return;
		if(ship.getFluxTracker().isOverloadedOrVenting()) return;
		if(ship.getCollisionClass().equals(CollisionClass.NONE)) return;
        if(target == null) return;

        tracker.advance(amount);
        if (tracker.intervalElapsed())
        {
            shouldUseSystem = AIUtils.canUseSystemThisFrame(this.ship)
							&& !system.isOn()
							&& !system.isCoolingDown()
							&& !system.isActive()
							&& ship.getFluxTracker().getFluxLevel() < 0.5f
							;

			//Don't do this check unless the above is true, math-heavy
			if(shouldUseSystem){
				float strafeDistance = BestStrafeDistance(ship,false);
				strafeDistance = (strafeDistance + 100f) * (strafeDistance + 100f);
				//Don't use System if we're not aimed at the target and reasonably close!
				float AngleToCollision = VectorUtils.getAngle(ship.getLocation(), target.getLocation());
				float angDiff = SteerMeStandard.getAngleDifference(ship.getFacing(), AngleToCollision);
				float AbsAng = Math.abs(angDiff);
				if(AbsAng > 45f || MathUtils.getDistanceSquared(ship, target) > strafeDistance){
					shouldUseSystem = false;
				}
			}
            // If system is inactive and should be active, enable it
            if(shouldUseSystem == true)
            {
                ship.useSystem();
            }
        }
    }
}
