package data.shipsystems.scripts.ai;

import com.fs.starfarer.api.combat.CollisionClass;
import org.lwjgl.util.vector.Vector2f;

import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipSystemAIScript;
import com.fs.starfarer.api.combat.ShipSystemAPI;
import com.fs.starfarer.api.combat.ShipwideAIFlags;
import com.fs.starfarer.api.util.IntervalUtil;
import data.scripts.shipai.SteerMeStandard;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lazywizard.lazylib.combat.AIUtils;

public class BurnDriveAI implements ShipSystemAIScript {

	private ShipwideAIFlags flags;
    private ShipAPI ship;
    private CombatEngineAPI engine;
    private final float strafeDistance = 1000f * 1000f;

    private boolean shouldUseSystem = false;
    private final IntervalUtil tracker = new IntervalUtil(0.1f, 0.2f);

    @Override
    public void init(ShipAPI thisShip, ShipSystemAPI thisSystem, ShipwideAIFlags thisFlags, CombatEngineAPI thisEngine) {
        ship = thisShip;
        engine = thisEngine;
		flags = thisFlags;
    }

    @Override
    public void advance(float amount, Vector2f missileDangerDir, Vector2f collisionDangerDir, ShipAPI target)
    {
        if(engine == null) return;
        if(engine.isPaused()) return;
		if(ship.getCollisionClass().equals(CollisionClass.NONE)) return;
        if(target == null) return;

		//Don't activate if this ship should be retreating
		if(flags.hasFlag(ShipwideAIFlags.AIFlags.BACKING_OFF)){
			return;
		}
		
        tracker.advance(amount);
        if (tracker.intervalElapsed())
        {
            shouldUseSystem = AIUtils.canUseSystemThisFrame(ship)
			&& ship.getHullLevel() > 0.75f //Don't use this if injured
			//&& SteerMeStandard.isBiggerOrEqual(ship, target)
            && MathUtils.getDistanceSquared(target,ship) > strafeDistance
			;

			//Don't do this check unless the above is true, math-heavy
			if(shouldUseSystem){
				//Don't use System if we're not aimed at the target!
				float AngleToCollision = VectorUtils.getAngle(ship.getLocation(), target.getLocation());
				float angDiff = SteerMeStandard.getAngleDifference(ship.getFacing(), AngleToCollision);
				float AbsAng = Math.abs(angDiff);
				if(AbsAng > 20f){
					shouldUseSystem = false;
				}
			}
            // If system is inactive and should be active, enable it
            if(shouldUseSystem)
            {
                ship.useSystem();
            }
        }
    }
}
