package data.shipsystems.scripts.ai;

import com.fs.starfarer.api.combat.CollisionClass;
import java.util.List;

import org.lwjgl.util.vector.Vector2f;

import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.FluxTrackerAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipCommand;
import com.fs.starfarer.api.combat.ShipSystemAIScript;
import com.fs.starfarer.api.combat.ShipSystemAPI;
import com.fs.starfarer.api.combat.ShipwideAIFlags;
import com.fs.starfarer.api.util.IntervalUtil;
import data.scripts.plugins.ShipThreatPlugin;
import org.lazywizard.lazylib.combat.AIUtils;

public class DisplacerAI implements ShipSystemAIScript {
	private CombatEngineAPI engine;
	private ShipAPI ship;
	private ShipSystemAPI system;
	
	private final IntervalUtil tracker = new IntervalUtil(0.35f, 0.6f);
	
        @Override
	public void init(ShipAPI ship, ShipSystemAPI system, ShipwideAIFlags flags, CombatEngineAPI engine) {
		this.engine = engine;
		this.ship = ship;
		this.system = system;
	}

    @Override
    public void advance(float amount, Vector2f missileDangerDir, Vector2f collisionDangerDir, ShipAPI target) {
        if(engine == null) return;
        if(engine.isPaused()) return;
		if(ship.getCollisionClass().equals(CollisionClass.NONE)) return;
		if(ship.getFluxTracker().isOverloadedOrVenting()) return;
		
		tracker.advance(amount);
		
        if (tracker.intervalElapsed()) {
            // Can we even use the system right now?
            if (!AIUtils.canUseSystemThisFrame(ship))
            {
                return;
            }
            boolean shouldUseSystem = false;
            List nearbyEnemies = ShipThreatPlugin.getCloseNotFighterThreats(ship);
            List nearbyMissiles = ShipThreatPlugin.getMissiles(ship);
            if(nearbyMissiles == null || nearbyEnemies == null || nearbyEnemies.isEmpty() || nearbyMissiles.isEmpty()){
                shouldUseSystem = true;
            }

            if (ship.getHullLevel() < 0.75f)//Don't when damaged, you're probably in the middle of a firefight
            {
				shouldUseSystem = false;
            }		

			FluxTrackerAPI flux = ship.getFluxTracker();
			if (flux.getFluxLevel() > 0.5f)//We're probably about to overload, may as well jump out
			{
				ship.giveCommand(ShipCommand.ACCELERATE_BACKWARDS, null, 0);
				shouldUseSystem = true;
			}			
			
			// If system is inactive and should be active, enable it
			// If system is active and shouldn't be, disable it
			if (system.isActive() ^ shouldUseSystem)
			{
				ship.useSystem();
			}			
        }
    }
}
