package data.shipsystems.scripts.ai;


import com.fs.starfarer.api.combat.CollisionClass;
import org.lwjgl.util.vector.Vector2f;

import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipCommand;
import com.fs.starfarer.api.combat.ShipSystemAIScript;
import com.fs.starfarer.api.combat.ShipSystemAPI;
import com.fs.starfarer.api.combat.ShipwideAIFlags;
import com.fs.starfarer.api.util.IntervalUtil;
import data.scripts.plugins.ShipThreatPlugin;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.combat.AIUtils;

public class EMPAI implements ShipSystemAIScript {

    private ShipAPI ship;
    private CombatEngineAPI engine;
    private ShipSystemAPI system;
	private final float enemyDist = 700f * 700f;
	private final float missileDist = 500f * 500f;
    private boolean shouldUseSystem = false;
    private final IntervalUtil tracker = new IntervalUtil(0.2f, 0.4f);
	
    @Override
    public void init(ShipAPI ship, ShipSystemAPI system, ShipwideAIFlags flags, CombatEngineAPI engine) {
        this.ship = ship;
        this.engine = engine;
        this.system = system;
    }

    @Override
    public void advance(float amount, Vector2f missileDangerDir, Vector2f collisionDangerDir, ShipAPI target) 
    {
        if(engine == null) return;
        if(engine.isPaused()) return;    
		if(ship.getFluxTracker().isOverloadedOrVenting()) return;
		if(ship.getCollisionClass().equals(CollisionClass.NONE)) return;
        
		tracker.advance(amount);
		
        if (tracker.intervalElapsed()){
			shouldUseSystem = AIUtils.canUseSystemThisFrame(ship);
			shouldUseSystem =	shouldUseSystem 
								&& !system.isOn() 
								&& !system.isActive() 
								&& !system.isChargeup()
								&& ship.getFluxTracker().getFluxLevel() < 0.5f
								;
			//Don't get to this part unless true already; math-heavy.
			if(shouldUseSystem){
				ShipAPI closestEnemy = ShipThreatPlugin.getClosestThreat(ship);
				MissileAPI closestMissile = ShipThreatPlugin.getClosestMissileThreat(ship);
				if(closestEnemy != null){
					shouldUseSystem =	shouldUseSystem && MathUtils.getDistanceSquared(ship, closestEnemy) < enemyDist;
				}
				if(closestMissile != null){
					shouldUseSystem =	shouldUseSystem && MathUtils.getDistanceSquared(ship, closestMissile) < missileDist;
				}
			}
			if(shouldUseSystem) ship.useSystem();
			tracker.randomize();
        }
    }
}
