package data.shipsystems.scripts.ai;

import com.fs.starfarer.api.combat.CollisionClass;
import org.lwjgl.util.vector.Vector2f;

import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.FluxTrackerAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipSystemAIScript;
import com.fs.starfarer.api.combat.ShipSystemAPI;
import com.fs.starfarer.api.combat.ShipwideAIFlags;
import com.fs.starfarer.api.util.IntervalUtil;

public class FortressShieldAI implements ShipSystemAIScript {

	private ShipAPI ship;
	private CombatEngineAPI engine;
	private ShipSystemAPI system;
	
	private float trackerPlus = 0.0f;
	private boolean shouldUseSystem = false;
	private final IntervalUtil tracker = new IntervalUtil(0.35f + trackerPlus, 0.6f + trackerPlus);
	
	@Override
	public void init(ShipAPI ship, ShipSystemAPI system, ShipwideAIFlags flags, CombatEngineAPI engine) {
		this.ship = ship;
		this.engine = engine;
		this.system = system;
	}

    @SuppressWarnings("unchecked")
    @Override
    public void advance(float amount, Vector2f missileDangerDir, Vector2f collisionDangerDir, ShipAPI target) {
        if(engine == null) return;
        if(engine.isPaused()) return;   
		if(ship.getFluxTracker().isOverloadedOrVenting()) return;
		if(ship.getCollisionClass().equals(CollisionClass.NONE)) return;
		
        tracker.advance(amount);
        if (tracker.intervalElapsed()) {
            FluxTrackerAPI flux = ship.getFluxTracker();
            float level = flux.getFluxLevel();
            if (level > 0.45f)//We're probably heading towards overload, start it up.
            {
                shouldUseSystem = true;
            }
            if (level > 0.9f)//We're probably heading towards overload, shut it down!
            {
                shouldUseSystem = false;
            }	
            if (level < 0.1f)//Shut down when we've lost all Hard Flux.
            {
                shouldUseSystem = false;
            }			
			
            // If system is inactive and should be active, enable it
            // If system is active and shouldn't be, disable it
            if (system.isActive() == true && shouldUseSystem == false)
            {
                ship.useSystem();
                trackerPlus = 0.0f;
            }	
            if (system.isActive() == true && shouldUseSystem == true)
            {
                trackerPlus = 0.0f;
            }
            if (system.isActive() == false && shouldUseSystem == false)
            {
                trackerPlus = 0.0f;
            }			
            if(system.isActive() == false && shouldUseSystem == true)
            {
                ship.useSystem();
                trackerPlus = 5.0f;//Wait a minimum of 5 seconds after this to turn it off!
            }		
            tracker.randomize();
        }
    }
}
