package data.shipsystems.scripts.ai;


import org.lwjgl.util.vector.Vector2f;

import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipCommand;
import com.fs.starfarer.api.combat.ShipSystemAIScript;
import com.fs.starfarer.api.combat.ShipSystemAPI;
import com.fs.starfarer.api.combat.ShipwideAIFlags;
import com.fs.starfarer.api.util.IntervalUtil;
import data.scripts.plugins.ShipThreatPlugin;
import static data.scripts.shipai.BestStrafeDistance.BestStrafeDistance;
import org.lazywizard.lazylib.MathUtils;

public class PhaseCloakAI implements ShipSystemAIScript {

	private ShipAPI ship;
	private CombatEngineAPI engine;

	private final IntervalUtil tracker = new IntervalUtil(0.2f, 0.3f);

    @Override
    public void init(ShipAPI ship, ShipSystemAPI system, ShipwideAIFlags flags, CombatEngineAPI engine) {
            this.ship = ship;
            this.engine = engine;
    }

    @Override
    public void advance(float amount, Vector2f missileDangerDir, Vector2f collisionDangerDir, ShipAPI target) {
        if(engine == null) return;
        if(engine.isPaused()) return;
		if(ship.getFluxTracker().isOverloadedOrVenting()) return;

        tracker.advance(amount);
        if (tracker.intervalElapsed()) {
			float strafeDistance = BestStrafeDistance(ship,false);
			strafeDistance = strafeDistance * strafeDistance;
            boolean shouldUseSystem = false;

			ShipAPI closestEnemy = ShipThreatPlugin.getClosestThreat(ship);
			MissileAPI closestMissile = ShipThreatPlugin.getClosestMissileThreat(ship);
			if (closestEnemy == null && closestMissile == null){
				shouldUseSystem = true;
			}
			if(closestEnemy != null){
				//Don't de-cloak until in range of something.
				if(MathUtils.getDistanceSquared(ship, closestEnemy) > strafeDistance){
					shouldUseSystem = true;
				} else {
					shouldUseSystem = false;
				}
			}
			//Stay cloaked if we're being hunted by missiles.
			if(closestMissile != null){
				if(MathUtils.getDistanceSquared(ship, closestMissile) < strafeDistance){
					shouldUseSystem = true;
				}
			}
			if(ship.getFluxTracker().getFluxLevel() > 0.6f) shouldUseSystem = false;

            // If system is inactive and should be active, enable it
            // If system is active and shouldn't be, disable it
            if (!ship.getPhaseCloak().isOn() || !ship.getPhaseCloak().isActive())
            {
                if(shouldUseSystem) ship.giveCommand(ShipCommand.TOGGLE_SHIELD_OR_PHASE_CLOAK,null,0);
            } else if (ship.getPhaseCloak().isOn() || ship.getPhaseCloak().isActive()){
                if(!shouldUseSystem) ship.giveCommand(ShipCommand.TOGGLE_SHIELD_OR_PHASE_CLOAK,null,0);
            }
			tracker.randomize();
        }
    }
}
