package data.shipsystems.scripts.ai;

import com.fs.starfarer.api.combat.CollisionClass;
import org.lwjgl.util.vector.Vector2f;

import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipSystemAIScript;
import com.fs.starfarer.api.combat.ShipSystemAPI;
import com.fs.starfarer.api.combat.ShipwideAIFlags;
import com.fs.starfarer.api.util.IntervalUtil;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.combat.AIUtils;

public class PhaseTeleporterAI implements ShipSystemAIScript {

    private ShipAPI ship;
    private CombatEngineAPI engine;
    private ShipSystemAPI system;

    private boolean shouldUseSystem = false;
    private IntervalUtil tracker = new IntervalUtil(0.1f, 1.5f);
	
    @Override
    public void init(ShipAPI ship, ShipSystemAPI system, ShipwideAIFlags flags, CombatEngineAPI engine) {
        this.ship = ship;
        this.engine = engine;
        this.system = system;
    }

    @Override
    public void advance(float amount, Vector2f missileDangerDir, Vector2f collisionDangerDir, ShipAPI target) 
    {
        if(engine == null) return;
        if(engine.isPaused()) return;
		if(ship.getFluxTracker().isOverloadedOrVenting()) return;
		if(ship.getCollisionClass().equals(CollisionClass.NONE)) return;
        tracker.advance(amount);
		
        if (tracker.intervalElapsed()) 
        {
            shouldUseSystem = AIUtils.canUseSystemThisFrame(ship) && !system.isOn() && !system.isCoolingDown();
            // If system is inactive and should be active, enable it		
            if(shouldUseSystem == true)
            {   
                if(target != null && target instanceof ShipAPI && MathUtils.getDistanceSquared(ship, target) < (1800f * 1800f)){
                    float angle = target.getFacing();
                    float getAngleAway = MathUtils.clampAngle(angle - 180f + MathUtils.getRandomNumberInRange(-15f,15f));
                    Vector2f targetPoint = MathUtils.getPointOnCircumference(target.getLocation(),target.getCollisionRadius() + 600f,getAngleAway);
                    ship.getMouseTarget().set(targetPoint);
                    ship.useSystem();
                }
            }
			tracker.randomize();
        }
    }
}
