package data.campaign.terrain;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.FleetAssignment;
import com.fs.starfarer.api.campaign.LocationAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactoryV2;
import com.fs.starfarer.api.impl.campaign.fleets.FleetParams;
import com.fs.starfarer.api.impl.campaign.ids.FleetTypes;
import com.fs.starfarer.api.impl.campaign.terrain.HyperspaceTerrainPlugin;
import com.fs.starfarer.api.impl.campaign.terrain.HyperspaceTerrainPlugin.CellStateTracker;
import com.fs.starfarer.api.loading.FleetCompositionDoctrineAPI;
import org.lazywizard.lazylib.MathUtils;
import org.lwjgl.util.vector.Vector2f;

public class HyperspaceTerrainPluginNew extends HyperspaceTerrainPlugin { 
	private int combatCheck = 0;
	private int randomVal = 0;
	private boolean createFleet = false;
	
	//Gets a Faction that hates the player, obviously.
	private FactionAPI findRandomFactionThatHatesThePlayer(){
		for(int i = 0; i < 100; i++){
			for(FactionAPI faction : Global.getSector().getAllFactions()){
				if(faction.getRelToPlayer().isHostile() && MathUtils.getRandomNumberInRange(0f,100f) >= 95f){
					return faction;
				}
			}
		}
		//Just in case... but most of the time, we'll see the Remnants, if nothing else.
		return Global.getSector().getFaction("pirates");
	}
	
	private MarketAPI findBestMarket(FactionAPI faction){
		MarketAPI thisMarket = null;
		int currSize = 0;
		for(MarketAPI market : Global.getSector().getEconomy().getMarketsCopy()){
			if(market.getFaction().equals(faction)){
				if(market.getSize() > currSize){
					thisMarket = market;
					currSize = market.getSize();
				}
			}
		}
		if(thisMarket != null) return thisMarket;
		return null;
	}
	
	@Override
	public void applyEffect(SectorEntityToken entity, float days) {
		if(randomVal == 0) randomVal = MathUtils.getRandomNumberInRange(1500, 5000);
		combatCheck += 1;
		if(combatCheck > randomVal) combatCheck = randomVal;
		if (entity instanceof CampaignFleetAPI) {
			CampaignFleetAPI fleet = (CampaignFleetAPI) entity;
			
			boolean inCloud = isInClouds(fleet);
			int [] tile = getTilePreferStorm(fleet.getLocation(), fleet.getRadius());
			CellStateTracker cell = null; 
			if (tile != null) {
				cell = super.activeCells[tile[0]][tile[1]];
			}
					
			if (!inCloud) {
				// open, do nothing
			} else if (cell == null || !cell.isStorming()) {
				// deep
				fleet.getStats().addTemporaryModMult(0.1f, getModId() + "_1",
						"In deep hyperspace", VISIBLITY_MULT, 
						fleet.getStats().getDetectedRangeMod());
	
				float penalty = getBurnPenalty(fleet);
				fleet.getStats().addTemporaryModMult(0.1f, getModId() + "_2",
						"In deep hyperspace", 1f - penalty, 
						fleet.getStats().getFleetwideMaxBurnMod());
			} else if (cell.isStorming()) {
				if(entity.equals(Global.getSector().getPlayerFleet())){
					if(combatCheck >= randomVal){
						createFleet = true;
					}
				}
				
				// storm
				if (STORM_SENSOR_RANGE_MULT != 1) {
					fleet.getStats().addTemporaryModMult(0.1f, getModId() + "_storm_sensor",
							"In deep hyperspace (storm)", STORM_SENSOR_RANGE_MULT, 
							fleet.getStats().getSensorRangeMod());
				}
				
				if (STORM_VISIBILITY_FLAT != 0) {
					fleet.getStats().addTemporaryModFlat(0.1f, getModId() + "_storm_visibility",
							"In deep hyperspace (storm)", STORM_VISIBILITY_FLAT,
							fleet.getStats().getDetectedRangeMod());
				}
				if (STORM_SPEED_MULT != 1) {			
					fleet.getStats().addTemporaryModPercent(0.1f, getModId() + "_storm_speed",
							"In deep hyperspace (storm)", -100f * (1f - getAdjustedSpeedMult(fleet, STORM_SPEED_MULT)), 
							fleet.getStats().getFleetwideMaxBurnMod());				
				}
			}
		}
	}
	
	@Override
	public void advance(float amount) {
		CampaignFleetAPI playerFleet = Global.getSector().getPlayerFleet();
		if (playerFleet != null) {
			LocationAPI loc = playerFleet.getContainingLocation();
			if(!loc.isHyperspace()){
				for(CampaignFleetAPI fleet : Global.getSector().getHyperspace().getFleets()){
					//If the fleet's not in a battle and meets these conditons...
					if(fleet.getBattle() == null){
						if(fleet.isAlive() && fleet.hasTag("dragon_fleet") 
								|| fleet.isAlive() && (
								fleet.getAI().isCurrentAssignment(FleetAssignment.PATROL_SYSTEM)
								|| fleet.getAI().isCurrentAssignment(FleetAssignment.HOLD)
								|| fleet.getAI().isCurrentAssignment(FleetAssignment.RAID_SYSTEM)
								)){
							//Delete these fleets on this frame
							for(FleetMemberAPI member : fleet.getMembersWithFightersCopy()){
								fleet.removeFleetMemberWithDestructionFlash(member);
							}
						}
					}
				}
			}
		}
		
		//if (true) return;
		super.advance(amount);

		playStormStrikeSoundsIfNeeded();
		
		float days = Global.getSector().getClock().convertToDays(amount);

		auto.advance(days * 1f);
		
		int [][] cells = auto.getCells();
		
		Vector2f test = new Vector2f();
		if (playerFleet != null) {
			test = playerFleet.getLocation();
		}
		
		float x = this.entity.getLocation().x;
		float y = this.entity.getLocation().y;
		float size = getTileSize();
		
		float w = tiles.length * size;
		float h = tiles[0].length * size;
		x -= w/2f;
		y -= h/2f;
		int xIndex = (int) ((test.x - x) / size);
		int yIndex = (int) ((test.y - y) / size);
		if (xIndex < 0) xIndex = 0;
		if (yIndex < 0) yIndex = 0;
		if (xIndex >= tiles.length) xIndex = tiles.length - 1;
		if (yIndex >= tiles[0].length) yIndex = tiles[0].length - 1;
		
		int subgridSize = (int) ((10000 / size + 1) * 2f);
		
		int minX = Math.max(0, xIndex - subgridSize/2);
		int maxX = xIndex + subgridSize/2 ;
		int minY = Math.max(0, yIndex - subgridSize/2);
		int maxY = yIndex + subgridSize/2;
		
		// clean up area around the "active" area so that as the player moves around,
		// they don't leave frozen storm cells behind (which would then make it into the savefile)
		int pad = 4;
		for (int i = minX - pad; i <= maxX + pad && i < tiles.length; i++) {
			for (int j = minY - pad; j <= maxY + pad && j < tiles[0].length; j++) {
				if (i < minX || j < minY || i > maxX || j > maxY) {
					if (i >= 0 && j >= 0) {
						activeCells[i][j] = null;
					}
				}
			}
		}
		
		for (int i = minX; i <= maxX && i < tiles.length; i++) {
			for (int j = minY; j <= maxY && j < tiles[0].length; j++) {
				if (tiles[i][j] < 0) continue;
				
				CellStateTracker curr = activeCells[i][j];
				int val = cells[i][j];
				float interval = auto.getInterval().getIntervalDuration();
				
				if (val == 1 && curr == null) {
					curr = activeCells[i][j] = new CellStateTracker(i, j, 
							interval * 0f + interval * 1.5f * (float) Math.random(),
							interval * 0.5f + interval * 0.5f * (float) Math.random());
				}
				
				if (curr != null) {
					if (val != 1 && curr.isStorming() && !curr.isWaning()) {
						curr.wane(interval * 0.5f + interval * 0.5f * (float) Math.random());
					}
					
					curr.advance(days);
					if (curr.isOff()) {
						activeCells[i][j] = null;
					}
				}
			}
		}
		
		if(createFleet){
			boolean fastFlyers = MathUtils.getRandomNumberInRange(0, 10) > 8;
			FactionAPI enemyFaction = findRandomFactionThatHatesThePlayer();
			MarketAPI enemyMarket = findBestMarket(enemyFaction);
			FleetCompositionDoctrineAPI doctrine = enemyFaction.getCompositionDoctrine();
			float points = doctrine.getMinPointsForCombatCapital() -1f;
			
			//Creates fleets- either one huge heavy fleet or a horde of small ones surrounding the player's fleet.
			int repeats = MathUtils.getRandomNumberInRange(0, (int) Math.max(1f, Global.getSector().getPlayerPerson().getStats().getLevel() /5f));
			for(int i = 0; i < repeats; i++){
				Vector2f enemyLoc = MathUtils.getRandomPointOnCircumference(Global.getSector().getPlayerFleet().getLocation(), MathUtils.getRandomNumberInRange(200f, 350f));
				FleetParams flParams = new FleetParams();
				flParams.factionId = enemyFaction.getId();						
				flParams.withOfficers = true;
				flParams.market = enemyMarket;
				flParams.factionIdForShipPicking = enemyFaction.getId();
				flParams.levelLimit = 50;
				flParams.qualityBonus = 10f;
				if(fastFlyers){
					flParams.combatPts = points;
				} else {
					flParams.combatPts = MathUtils.getRandomNumberInRange(points, points*2f) + (float) Global.getSector().getPlayerPerson().getStats().getLevel();
				}
				flParams.hyperspaceLocation = enemyLoc;
				flParams.civilianPts = 0;
				flParams.freighterPts = 0;
				flParams.tankerPts = 0;
				flParams.freighterPts = 0;
				flParams.utilityPts = 0;
				flParams.random = MathUtils.getRandom();
				if(!fastFlyers){
					flParams.fleetType = FleetTypes.PATROL_LARGE;
				} else {
					flParams.fleetType = FleetTypes.PATROL_MEDIUM;
				}
				CampaignFleetAPI enemyFleet = FleetFactoryV2.createFleet(flParams);
				enemyFleet.setLocation(enemyLoc.getX(), enemyLoc.getY());
				enemyFleet.addTag("dragon_fleet");
				//FleetFactoryV2.addRandomCombatShips(flParams.combatPts, flParams.qualityBonus, enemyFleet, MathUtils.getRandom(), enemyMarket, true, 20f, flParams);
				enemyFleet.knowsWhoPlayerIs();
				combatCheck = 0;
				//Resets the randomVal to a random-enough range to provide a jump-scare
				randomVal = MathUtils.getRandomNumberInRange(15000, 150000);
				Global.getSector().getHyperspace().addEntity(enemyFleet);	
			}
			createFleet = false;
		}
		
	}	

}








