//What this does:  scales Beam and Energy damage w/ range, so that closer ranges mean more damage for these weapon types.
package data.scripts.plugins;

import com.fs.starfarer.api.combat.BeamAPI;
import com.fs.starfarer.api.combat.WeaponAPI;
import com.fs.starfarer.api.combat.WeaponAPI.DerivedWeaponStatsAPI;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.DamageType;
import com.fs.starfarer.api.combat.DamagingProjectileAPI;
import com.fs.starfarer.api.combat.EveryFrameCombatPlugin;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ViewportAPI;
import data.scripts.weapons.helperUtils.RebalCollisionCheck;

import org.lazywizard.lazylib.MathUtils;
import data.scripts.weapons.helperUtils.RebalWeaponUtils;
import java.util.ArrayList;
import java.util.List;

public class BeamDamageScalar implements EveryFrameCombatPlugin {
	CombatEngineAPI engine;
	
	public static List<DamagingProjectileAPI> deadProjectiles = new ArrayList<>();
	
	@Override
	public void init(CombatEngineAPI cEngine) {
		engine = cEngine;
		deadProjectiles.clear();
	}	
	
	private float[] damageAmount(BeamAPI beam, float amount){
		//Now that we have the target, get the weapon ID and get the adjuested DPS and range
		WeaponAPI weapon = beam.getWeapon();

		DerivedWeaponStatsAPI stats = weapon.getDerivedStats();
		float maxRange = weapon.getRange();
		maxRange *= maxRange;
		//The range is now used to give additional damage output, using how close we are to the target.
		//Anything less than 1/3 range is at maximum damage, here at 4X damage (but easily adjusted).
		float theRangeMult = Math.min(Math.max((maxRange / MathUtils.getDistanceSquared(beam.getTo(), beam.getFrom()))-1f,0f),3f);
		
		float dam = (stats.getDps() * amount) * theRangeMult;
		float emp = (stats.getEmpPerSecond() * amount) * theRangeMult;	
		float[] returnMe = new float[2];
		returnMe[0] = dam;
		returnMe[1] = emp;
		return returnMe;
	}
	
	@Override
    public void advance(float amount, List events){
        if(engine == null) return;
		if(engine.isPaused()) return;
		if(!engine.isInCampaign() && !engine.isInCampaignSim() && !engine.isMission() && !engine.isSimulation()) return;
		
		
		for(DamagingProjectileAPI projectile : engine.getProjectiles()){
			//Bunch of booleans to queries to eliminate various forms of suck.
			if(!projectile.getDamageType().equals(DamageType.ENERGY) || !projectile.didDamage()) continue;
			if(projectile.getDamageTarget() == null) continue;
			if(projectile.getWeapon() == null) continue;
			if(projectile.getSource() == null) continue;
			if(!(projectile.getDamageTarget() instanceof ShipAPI)) continue;

			//I'm not sure, but I suspect .contains is more expensive than the booleans above; hence that buck stops here.
			if(deadProjectiles.contains(projectile)) continue;
			//Never ever do any of this chunky math again for this projectile...
			deadProjectiles.add(projectile);
			//Don't go in circular loops with EZ_Damage stuff!
			String id = projectile.getWeapon().getId();
			if(	id.equalsIgnoreCase("ez_damage_emp") || 
				id.equalsIgnoreCase("ez_damage_e") || 	
				id.equalsIgnoreCase("ez_damage_k") || 
				id.equalsIgnoreCase("ez_damage_h") || 
				id.equalsIgnoreCase("ez_damage_f")
				) continue;			
			
			float maxRange =  projectile.getWeapon().getRange();
			maxRange *= maxRange;
			float rangeToDet = MathUtils.getDistanceSquared(projectile.getLocation(), projectile.getWeapon().getLocation());
			float ratio = Math.min(Math.max((maxRange / rangeToDet)-1f,0f),4f);
			if(ratio > 0f){
				boolean damageFlux = false;
				ShipAPI sTarg = (ShipAPI) projectile.getDamageTarget();
				if(sTarg.getShield() != null){
					if(sTarg.getShield().isOn() && RebalCollisionCheck.isInShieldArc(sTarg, projectile.getLocation())){
						damageFlux = true;
					}
				}			
				
				float dam = projectile.getDamageAmount();
				float emp = projectile.getEmpAmount();

				//Do some additional damage now, based on the range.  Don't use projectile unless it's absolutely necessary.
				if(!damageFlux){
					RebalWeaponUtils.rebalDamageThisEntity(projectile.getLocation(), projectile.getDamageTarget(), dam, emp, DamageType.ENERGY, engine, projectile.getSource());
				} else {
					sTarg.getFluxTracker().increaseFlux(dam + (emp / 4f), true);
				}
			}
		}
		
		for(BeamAPI beam : engine.getBeams()){
			if(!beam.didDamageThisFrame() || beam.getBrightness() < 0.99f) continue;
			if(beam.getDamageTarget() == null) continue;
			if(beam.getSource() == null) continue;
			if(beam.getWeapon() == null) continue;
			
			if(beam.getDamageTarget() instanceof ShipAPI){
				boolean damageFlux = false;
				ShipAPI sTarg = (ShipAPI) beam.getDamageTarget();
				if(sTarg.getShield() != null){
					if(sTarg.getShield().isOn() && RebalCollisionCheck.isInShieldArc(sTarg, beam.getTo())){
						damageFlux = true;
					}
				}
				List hullmods = sTarg.getVariant().getSortedMods();
				if(hullmods.contains("shields_formshield")){
					damageFlux = true;
				}
				float[] theDamage = damageAmount(beam,amount);
				float damage = theDamage[0];
				float emp = theDamage[1];
				DamageType damType = beam.getWeapon().getDamageType();
				//Do some additional damage now, based on the range
				if(!damageFlux){
					RebalWeaponUtils.rebalDamageThisEntity(beam.getTo(), beam.getDamageTarget(), damage, emp, damType, engine, beam.getSource());
				} else {
					sTarg.getFluxTracker().increaseFlux(damage + (emp / 4f), true);
				}				
			} else {
				float[] theDamage = damageAmount(beam,amount);
				float damage = theDamage[0];
				float emp = theDamage[1];
				DamageType damType = beam.getWeapon().getDamageType();
				RebalWeaponUtils.rebalDamageThisEntity(beam.getTo(), beam.getDamageTarget(), damage, emp, damType, engine, beam.getSource());
			}
		}
    }

	@Override
	public void renderInWorldCoords(ViewportAPI vapi) {
	}

	@Override
	public void renderInUICoords(ViewportAPI vapi) {
	}
}