package data.scripts.plugins;

import com.fs.starfarer.api.BaseModPlugin;
import com.fs.starfarer.api.Global;
import java.awt.Color;
import java.util.List;

import org.lwjgl.util.vector.Vector2f;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.EveryFrameCombatPlugin;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ViewportAPI;
import com.fs.starfarer.api.graphics.SpriteAPI;
import data.scripts.fx_SharedLib;
import org.lazywizard.lazylib.MathUtils;
import static org.lazywizard.lazylib.MathUtils.getRandomNumberInRange;

public class ShipExplosion extends BaseModPlugin implements EveryFrameCombatPlugin  {
	private CombatEngineAPI engine;
	
	private final Color ExplosionColor = new Color (255,200,0,255);
	private final Color ExplosionColorFade = new Color (255,32,0,0);
    private final Color smokeColor = new Color(45, 40, 35, 255); 
	
    private final SpriteAPI fx_smoke_tendril_001 = Global.getSettings().getSprite("fx", "fx_smoke_tendril_001");
    private final SpriteAPI fx_smoke_tendril_002 = Global.getSettings().getSprite("fx", "fx_smoke_tendril_002");
    private final SpriteAPI fx_smoke_tendril_003 = Global.getSettings().getSprite("fx", "fx_smoke_tendril_003");	
	
	private final Vector2f noVec = new Vector2f(0f,0f);
	
	@Override
	public void init(CombatEngineAPI eng) {
            engine = eng;
	}
	        
        @Override
	public void advance(float amount, List events) 
	{
        if(engine == null) return;
		if(engine.isPaused()) return;
		if(!engine.isInCampaign() && !engine.isInCampaignSim() && !engine.isMission() && !engine.isSimulation()) return;
		float decline = 0f;
		boolean canDestroyHulks = false;
		if(amount >= 0.0333f){
			decline = 0.5f;
			canDestroyHulks = true;
		}  
		//Gets all ships and iterates through them
		for(ShipAPI ship : engine.getShips()){
			float randChance = (float) Math.random();
			randChance -= decline;
			//GOING CRITICAL, the piece version
			//Randomized destruction of Pieces are done on a regular basis, to clear battlefields and keep performance pretty good.
			if(ship.isPiece() && randChance < 0.001f){
				//float dam = Math.max(7500f,Math.min(ship.getMaxHitpoints()/2f, 30000f));
				//explosionBitz(dam, ship.getLocation());
				int power = (int)Math.max(3000f,Math.min(ship.getMaxHitpoints()/2f, 30000f)) / 750;
				int repeats = getRandomNumberInRange(7, 15);
				float smokeGrowth = getRandomNumberInRange(1.001f, 1.002f);
				float smokeLifeTime = getRandomNumberInRange(3f,5f);
				float smokeFadeTime = getRandomNumberInRange(3f,5f);				
				for (int i = 0; i < repeats; i++) {
					float angle = getRandomNumberInRange(0f, 360f);
					float size = getRandomNumberInRange(40f, 80f)* power;
					int choice = MathUtils.getRandomNumberInRange(0, 2);
					//GLOW EFFECT
					doHEGlow(ship.getLocation(),  noVec, size, angle, angle, 1, choice, ExplosionColor, ExplosionColorFade);
					//SMOKE EFFECT
					doSmoke(ship.getLocation(),  noVec, size, angle, angle, 1, smokeGrowth, smokeLifeTime, smokeFadeTime, choice,  smokeColor, smokeColor);						
				}				
				engine.removeEntity(ship);
			} 
			//Kill non-pieces quickly if framerate goes below 30.
			if(!ship.isFighter() && !ship.isDrone() && ship.isHulk() && canDestroyHulks && randChance < 0.001f){
				//float dam = Math.max(7500f,Math.min(ship.getMaxHitpoints()/2f, 30000f));
				//explosionBitz(dam, ship.getLocation());
				int power = (int)Math.max(3000f,Math.min(ship.getMaxHitpoints()/2f, 30000f)) / 750;
				int repeats = getRandomNumberInRange(7, 15);
				float smokeGrowth = getRandomNumberInRange(1.001f, 1.002f);
				float smokeLifeTime = getRandomNumberInRange(3f,5f);
				float smokeFadeTime = getRandomNumberInRange(3f,5f);					
				for (int i = 0; i < repeats; i++) {
					float angle = getRandomNumberInRange(0f, 360f);
					float size = getRandomNumberInRange(40f, 80f)* power;
					int choice = MathUtils.getRandomNumberInRange(0, 2);
					//GLOW EFFECT
					doHEGlow(ship.getLocation(),  noVec, size, angle, angle, 1, choice, ExplosionColor, ExplosionColorFade);
					//SMOKE EFFECT
					doSmoke(ship.getLocation(),  noVec, size, angle, angle, 1, smokeGrowth, smokeLifeTime, smokeFadeTime, choice,  smokeColor, smokeColor);						
				}				
				
				engine.removeEntity(ship);
			} 
			
			//Kill fighter Hulks instantly, always
			if((ship.getHullLevel() <= 0.03f || ship.isHulk())&& (ship.isFighter() || ship.isDrone())){
//				float dam = 1000f;
//				if(amount > 0.0333f){
//					dam = 200f;
//				}
				int repeats = getRandomNumberInRange(5, 7);
				float smokeGrowth = getRandomNumberInRange(1.001f, 1.003f);
				float smokeLifeTime = getRandomNumberInRange(1f,3f);
				float smokeFadeTime = getRandomNumberInRange(1f, 1.5f);							
				for (int i = 0; i < repeats; i++) {
					float angle = getRandomNumberInRange(0f, 360f);
					float size = getRandomNumberInRange(80f, 160f);
					int choice = MathUtils.getRandomNumberInRange(0, 2);

					//GLOW EFFECT
					doHEGlow(ship.getLocation(),  noVec, size, angle, angle, 1, choice, ExplosionColor, ExplosionColorFade);
					//SMOKE EFFECT
					doSmoke(ship.getLocation(),  noVec, size, angle, angle, 1, smokeGrowth, smokeLifeTime, smokeFadeTime, choice,  smokeColor, smokeColor);						
				}				

					//explosionBitz(dam, ship.getLocation());

				engine.removeEntity(ship);
			}
		}
	}	
	
    private SpriteAPI returnSmoke(int choice) {
		switch (choice) {
			case 0:
				return fx_smoke_tendril_001;
			case 1:
				return fx_smoke_tendril_002;
			case 2:
				return fx_smoke_tendril_003;
			default:
				return fx_smoke_tendril_001;
		}
    }	
	
	private void doHEGlow(Vector2f projLoc,  Vector2f targetVel, float size, float angle, float facing, float power, int choice, Color projColor, Color projColorFade){
		float vel = getRandomNumberInRange(5, 10f);
		
		float growth = getRandomNumberInRange(1.02f, 1.05f);
		float lifeTime = getRandomNumberInRange(0.1f, 0.3f);
		float fadeTime = getRandomNumberInRange(0.1f, 0.15f);

		Vector2f velocity = fx_SharedLib.circularVelocity(angle, vel);
		velocity = new Vector2f(velocity.getX() + targetVel.getX(), velocity.getY() + targetVel.getY());
		fx_SharedLib.newParticle(projLoc, velocity, size, 1f, angle, true, true, returnSmoke(choice), projColor, projColorFade, lifeTime, fadeTime, growth);	
	}
	
	private void doSmoke(Vector2f projLoc,  Vector2f targetVel, float size, float angle, float facing, float power, float growth, float lifeTime, float fadeTime, int choice, Color projColor, Color projColorFade){
		float vel = getRandomNumberInRange(1, 3f);
		
		Vector2f velocity = fx_SharedLib.circularVelocity(angle, vel);
		velocity = new Vector2f(velocity.getX() + targetVel.getX(), velocity.getY() + targetVel.getY());
		fx_SharedLib.newParticle(projLoc, velocity, size, 1f, angle, false, false, returnSmoke(choice), smokeColor, smokeColor, lifeTime, fadeTime, growth);	
	}	

	/**
	* Returns a {@link Vector2f} velocity vector pointing from {@code position}
	*
	* @param position -- The origin of the vector.
	* @param velocity -- the velocity we wish to have going randomly away from the origin.
	* @return newVel -- A {@link Vector2f} pointing at {@code destination} that is the velocity.
	*/
	private static Vector2f randomCircularVelocity(float velocity)
	{
		Vector2f newVel = MathUtils.getRandomPointOnCircumference(null, velocity);
		return newVel;
	}
        
    private void explosionBitz(float dam, Vector2f point){
        //EXPLOSION BITZ
        //This code auto-magically builds a reasonably-cool generic explosion, based on weapon damage.
        //There isn't much code that's expensive here and the game does auto-cull particles, so this can be made more complex if you want.  Just be careful about how many particles you create per event, since that's a linear cost (i.e., if you stick this on a Vulcan-type weapon, keep the total count low, low or re-write that part).  
        int repeat = Math.max(5,(int) (dam / 60f));
        int repeatTwo = repeat / 3;

        //This bit makes backdrop glows show up fairly briefly behind our main explosion.
        //Just one particle per repeat, but it costs a lot of operations to build it.
        for(int i = 0; i < repeatTwo; i++)
        {
                Vector2f randPoint = MathUtils.getRandomPointInCircle(point,dam/35f);
                float randSize = MathUtils.getRandomNumberInRange(dam/25f,Math.max(dam / 20f,15f));
                Vector2f randVec = randomCircularVelocity(MathUtils.getRandomNumberInRange(30f,Math.max(dam / 25f,31f)));
                float randDur = 0.1f + MathUtils.getRandomNumberInRange(0.1f,0.75f);
                int yelVal = (int) (Math.random() * 128f + 32f);
                int randTrans = (int) MathUtils.getRandomNumberInRange(64f,128f);

                engine.addSmoothParticle(randPoint, randVec, randSize, 1f, randDur, new Color(128,yelVal,0,255));
        }

        //Main explosion, fast & slow fires and smoke at the end.
        //Six particles per repeat, min 30.
        for(int i = 0; i < repeat; i++){
            int yelVal = (int) (Math.random() * 128f + 64f);

            //Random Vectors, in order of speed; generally, we need smoke and central poof to be slower than others

            Vector2f randVecFast = randomCircularVelocity(MathUtils.getRandomNumberInRange(30f,Math.max(dam / 25f,31f)));
            Vector2f randVec = randomCircularVelocity(MathUtils.getRandomNumberInRange(30f,Math.max(dam / 25f,31f)));
            Vector2f randVecTwo = randomCircularVelocity(MathUtils.getRandomNumberInRange(30f,Math.max(dam / 35f,31f)));
            Vector2f randVecThree = randomCircularVelocity(MathUtils.getRandomNumberInRange(30f,Math.max(dam / 40f,30f)));
            Vector2f randVecSlow = randomCircularVelocity(MathUtils.getRandomNumberInRange(1f,Math.max(dam / 100f,2f)));
            Vector2f randVecSlowTwo = randomCircularVelocity(MathUtils.getRandomNumberInRange(1f,Math.max(dam / 100f,2f)));
            Vector2f randVecSlowThree = randomCircularVelocity(MathUtils.getRandomNumberInRange(1f,Math.max(dam / 100f,2f)));
            Vector2f randVecSlowFour = randomCircularVelocity(MathUtils.getRandomNumberInRange(1f,Math.max(dam / 100f,2f)));

            float randSize = MathUtils.getRandomNumberInRange(10f,Math.max(dam / 50f,15f));
            float randSizeTwo = MathUtils.getRandomNumberInRange(5f,10f);
            float randDur = 1f + MathUtils.getRandomNumberInRange(3f,10f);
            float randDurTwo = MathUtils.getRandomNumberInRange(0.5f,1f);
            int randTrans = (int) MathUtils.getRandomNumberInRange(32f,200f);

            //Do a bunch of glowy stuff, including some long-term particles that will fade out slowly.

            //Fast stuff
            engine.addHitParticle(point, randVecFast, randSizeTwo, 1f, randDurTwo * 0.75f, new Color(255,128,0,255));
            engine.addHitParticle(point, randVec, randSize, 1f, randDurTwo * 1.5f, new Color(128,64,0,255));
            engine.addHitParticle(point, randVecTwo, randSize, 1f, randDurTwo * 1.25f, new Color(128,64,0,255));
            engine.addHitParticle(point, randVecThree, randSize, 1f, randDurTwo, new Color(128,64,0,255));

            //Slow movers
            engine.addSmokeParticle(point, randVecSlow, randSizeTwo, 1f, randDur / 3f, new Color(yelVal - 64,yelVal - 64,yelVal - 64,255));
            engine.addSmokeParticle(point, randVecSlowTwo, randSizeTwo, 1f, randDur / 1.5f, new Color(yelVal - 64,yelVal - 64,yelVal - 64,255));
            engine.addSmokeParticle(point, randVecSlowThree, randSizeTwo * 1.3f, 1f, randDur * 2f, new Color(yelVal - 64,yelVal - 64,yelVal - 64,128));
            engine.addSmokeParticle(point, randVecSlowFour, randSizeTwo * 2.5f, 1f, randDur * 2.5f, new Color(yelVal - 64,yelVal - 64,yelVal - 64,128));
        }
    }

	@Override
	public void renderInWorldCoords(ViewportAPI vapi) {
	}

	@Override
	public void renderInUICoords(ViewportAPI vapi) {
	}
}	