package data.hullmods;

import com.fs.starfarer.api.combat.BaseHullMod;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;
import java.util.HashMap;
import java.util.Map;

public class AdvancedTargetingCore extends BaseHullMod {
	
	private static final Map<HullSize,Float> MAG = new HashMap();
	static {
		MAG.put(HullSize.FIGHTER, -20f);
		MAG.put(HullSize.FRIGATE, -20f);
		MAG.put(HullSize.DESTROYER, -15f);
		MAG.put(HullSize.CRUISER, -10f);
		MAG.put(HullSize.CAPITAL_SHIP, -5f);
	}	

	@Override
	public boolean isApplicableToShip(ShipAPI ship) {
		return (
				ship.getMutableStats().getBeamWeaponRangeBonus().getMult() < 1.25f 
				&& ship.getMutableStats().getBallisticWeaponRangeBonus().getMult() < 1.25f
				&& ship.getMutableStats().getEnergyWeaponRangeBonus().getMult() < 1.25f
				&& ship.getMutableStats().getNonBeamPDWeaponRangeBonus().getMult() < 1.25f 
				&& ship.getMutableStats().getBeamPDWeaponRangeBonus().getMult() < 1.25f 
				);
	}
	
	@Override
	public String getUnapplicableReason(ShipAPI ship) {
		return "Incompatible with other Hull Mods that increase ranges.";
	}
	
	public static float RANGE_BONUS = 115f;
	public static float PD_MINUS = 20f;
	
	@Override
	public String getDescriptionParam(int index, HullSize hullSize) {
		if (index == 0) return "" + (int)Math.round(RANGE_BONUS) + "%";
		if (index == 1) return "" + (100 - (int)Math.round(PD_MINUS)) + "%";
		if (index == 2) return "" + Math.round(MAG.get(hullSize)) + " su/sec.";
		return null;
	}
	
	
	@Override
	public void applyEffectsBeforeShipCreation(HullSize hullSize, MutableShipStatsAPI stats, String id) {
		stats.getBallisticWeaponRangeBonus().modifyMult(id, 1f + (RANGE_BONUS * 0.01f));
		stats.getEnergyWeaponRangeBonus().modifyMult(id, 1f + (RANGE_BONUS * 0.01f));
		stats.getNonBeamPDWeaponRangeBonus().modifyMult(id, 1f - (PD_MINUS * 0.01f));
		stats.getBeamPDWeaponRangeBonus().modifyMult(id, 1f - (PD_MINUS * 0.01f));
		stats.getMaxSpeed().modifyFlat(id, MAG.get(hullSize));
	}
}
