package data.hullmods;

import java.util.HashMap;
import java.util.Map;

import com.fs.starfarer.api.combat.BaseHullMod;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;

public class IntegratedTargetingUnit extends BaseHullMod {
	
	private static final Map<HullSize,Float> MAG = new HashMap();
	static {
		MAG.put(HullSize.FIGHTER, 25f);
		MAG.put(HullSize.FRIGATE, 30f);
		MAG.put(HullSize.DESTROYER, 35f);
		MAG.put(HullSize.CRUISER, 40f);
		MAG.put(HullSize.CAPITAL_SHIP, 45f);
	}
	
	@Override
	public String getDescriptionParam(int index, HullSize hullSize) {
		int hullInt = Math.round(MAG.get(hullSize));
		if (index == 0) return "" + hullInt + "%";
		if (index == 1) return "" + hullInt/2 + "%";
		return null;
	}
	
	
	@Override
	public void applyEffectsBeforeShipCreation(HullSize hullSize, MutableShipStatsAPI stats, String id) {
		float thisMag = MAG.get(hullSize);
		stats.getBallisticWeaponRangeBonus().modifyMult(id, 1f + (thisMag * 0.01f));
		stats.getEnergyWeaponRangeBonus().modifyMult(id, 1f + (thisMag * 0.01f));
		stats.getTurnAcceleration().modifyMult(id, 1f - (thisMag / 2f * 0.01f));
		stats.getMaxTurnRate().modifyMult(id, 1f - (thisMag / 2f * 0.01f));
	}


	@Override
	public boolean isApplicableToShip(ShipAPI ship) {
		return (
				ship.getMutableStats().getBeamWeaponRangeBonus().getMult() < 1.25f 
				&& ship.getMutableStats().getBallisticWeaponRangeBonus().getMult() < 1.25f
				&& ship.getMutableStats().getEnergyWeaponRangeBonus().getMult() < 1.25f
				&& ship.getMutableStats().getNonBeamPDWeaponRangeBonus().getMult() < 1.25f 
				&& ship.getMutableStats().getBeamPDWeaponRangeBonus().getMult() < 1.25f 
				);
	}
	
	@Override
	public String getUnapplicableReason(ShipAPI ship) {
		return "Incompatible with other Hull Mods that increase ranges.";
	}
	
}
