package data.hullmods;

import java.util.HashMap;
import java.util.Map;

import com.fs.starfarer.api.combat.BaseHullMod;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;

public class Militarized extends BaseHullMod {

	/* Militarized Auxiliary 
	 * Basically take a civilian hull, bring it up to military standards. (More or less.)
	 * - military sensor profile/detection (handled in skin by removing civ hull)
	 * - more OP (handled in skin)
	 * - slightly better flux handling
	 * - slightly better armour
	 * - slightly better speed/maneuver
	 * - 
	 */
	private static final float CAPACITY_MULT = 1.1f;
	private static final float DISSIPATION_MULT = 1.1f;
	private static final float HANDLING_MULT = 1.1f;
	
	
	private static final Map<HullSize,Float> MAG = new HashMap();
	static {
		MAG.put(HullSize.FIGHTER, 25f);
		MAG.put(HullSize.FRIGATE, 25f);
		MAG.put(HullSize.DESTROYER, 30f);
		MAG.put(HullSize.CRUISER, 35f);
		MAG.put(HullSize.CAPITAL_SHIP, 45f);
	}
	
	@Override
	public void applyEffectsBeforeShipCreation(HullSize hullSize, MutableShipStatsAPI stats, String id) {
		// Slightly better armour
		stats.getArmorBonus().modifyPercent(id, MAG.get(hullSize));
		
		// 10% better flux stats
		stats.getFluxCapacity().modifyMult(id, CAPACITY_MULT);
		stats.getFluxDissipation().modifyMult(id, DISSIPATION_MULT);
		
		// 10% better handling all around!
		stats.getMaxSpeed().modifyMult(id, HANDLING_MULT);
		stats.getAcceleration().modifyMult(id, HANDLING_MULT);
		stats.getDeceleration().modifyMult(id, HANDLING_MULT);
		stats.getMaxTurnRate().modifyMult(id, HANDLING_MULT);
		stats. getTurnAcceleration().modifyMult(id, HANDLING_MULT);
		
	}
	
	@Override
	public String getDescriptionParam(int index, HullSize hullSize) {
		if (index == 0) return "" + Math.round(MAG.get(hullSize)) + "%";
		if (index == 1) return "" + (int) ((HANDLING_MULT - 1f) * 100f) + "%";
		if (index == 2) return "" + (int) ((CAPACITY_MULT - 1f) * 100f) + "%";
		if (index == 3) return "" + (int) ((DISSIPATION_MULT - 1f) * 100f) + "%";
		return null;
	}


}
