package data.hullmods;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

import com.fs.starfarer.api.combat.BaseHullMod;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;
import com.fs.starfarer.api.impl.campaign.ids.HullMods;
import com.fs.starfarer.api.util.Misc;

public class SafetyOverrides extends BaseHullMod {

	private static final Map<HullSize,Float> SPEED = new HashMap();
	static {
		SPEED.put(HullSize.FIGHTER, 50f);
		SPEED.put(HullSize.FRIGATE, 50f);
		SPEED.put(HullSize.DESTROYER, 30f);
		SPEED.put(HullSize.CRUISER, 25f);
		SPEED.put(HullSize.CAPITAL_SHIP, 20f);
	}
	
	private static final float PEAK_MULT = 0.33f;
	private static final float FLUX_DISSIPATION_MULT = 2f;
	
	private static final float RANGE_THRESHOLD = 450f;
	private static final float RANGE_MULT = 0.25f;
	
	@Override
	public void applyEffectsBeforeShipCreation(HullSize hullSize, MutableShipStatsAPI stats, String id) {
		stats.getMaxSpeed().modifyFlat(id, SPEED.get(hullSize));
		stats.getAcceleration().modifyFlat(id, SPEED.get(hullSize) * 2f);
		stats.getDeceleration().modifyFlat(id, SPEED.get(hullSize) * 2f);
		stats.getZeroFluxMinimumFluxLevel().modifyFlat(id, 2f); // set to two, meaning boost is always on 
		
		stats.getFluxDissipation().modifyMult(id, FLUX_DISSIPATION_MULT);
		
		stats.getPeakCRDuration().modifyMult(id, PEAK_MULT);

		stats.getVentRateMult().modifyMult(id, 0f);
		
		stats.getWeaponRangeThreshold().modifyFlat(id, RANGE_THRESHOLD);
		stats.getWeaponRangeMultPastThreshold().modifyMult(id, RANGE_MULT);
	}
	
	@Override
	public String getDescriptionParam(int index, HullSize hullSize) {
		if (index == 0) return "" + Math.round(SPEED.get(hullSize));
		if (index == 1) return Misc.getRoundedValue(FLUX_DISSIPATION_MULT);
		if (index == 2) return Misc.getRoundedValue(PEAK_MULT);
		if (index == 3) return Misc.getRoundedValue(RANGE_THRESHOLD);
	
		return null;
	}

	@Override
	public boolean isApplicableToShip(ShipAPI ship) {
		return !ship.getVariant().hasHullMod(HullMods.CIVGRADE);
	}
	
	@Override
	public String getUnapplicableReason(ShipAPI ship) {
		if (ship.getVariant().hasHullMod(HullMods.CIVGRADE)) {
			return "Can not be installed on civilian ships!";
		}
		return null;
	}
	
	private final Color color = new Color(255,100,255,255);
	@Override
	public void advanceInCombat(ShipAPI ship, float amount) {
			ship.getEngineController().fadeToOtherColor(this, color, null, 1f, 0.4f);
			ship.getEngineController().extendFlame(this, 0.25f, 0.25f, 0.25f);
	}
}
