package data.scripts.plugins;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.BeamAPI;
import com.fs.starfarer.api.combat.CombatAsteroidAPI;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.CombatEntityAPI;
import com.fs.starfarer.api.combat.DamagingProjectileAPI;
import com.fs.starfarer.api.combat.EveryFrameCombatPlugin;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ViewportAPI;
import com.fs.starfarer.api.graphics.SpriteAPI;

import data.scripts.fx_SharedLib;

import java.awt.Color;
import java.util.HashSet;

import java.util.List;

import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lwjgl.util.vector.Vector2f;
import static org.lazywizard.lazylib.MathUtils.getRandomNumberInRange;

public class FX_Plugin_B implements EveryFrameCombatPlugin {

    CombatEngineAPI engine;

    public final Color smokeColor = new Color(45, 40, 35, 255);
    public final Color sparkColor = new Color(255, 255, 0, 255);
    public final Color fadeSpark = new Color(128, 32, 0, 255);

    //FX Sprites
    public final SpriteAPI fx_muzzleflash_001 = Global.getSettings().getSprite("fx", "fx_muzzleflash_001");
    public final SpriteAPI fx_muzzleflash_002 = Global.getSettings().getSprite("fx", "fx_muzzleflash_002");
    public final SpriteAPI fx_muzzleflash_003 = Global.getSettings().getSprite("fx", "fx_muzzleflash_003");

    public final SpriteAPI fx_shieldflash_001 = Global.getSettings().getSprite("fx", "fx_shieldflash_001");
    public final SpriteAPI fx_shieldflash_002 = Global.getSettings().getSprite("fx", "fx_shieldflash_002");
    public final SpriteAPI fx_shieldflash_003 = Global.getSettings().getSprite("fx", "fx_shieldflash_003");

	public final SpriteAPI fx_laserflash_001 = Global.getSettings().getSprite("fx", "fx_laserflash_001");
    public final SpriteAPI fx_laserflash_002 = Global.getSettings().getSprite("fx", "fx_laserflash_002");
    public final SpriteAPI fx_laserflash_003 = Global.getSettings().getSprite("fx", "fx_laserflash_003");

    public final SpriteAPI fx_smoke_tendril_001 = Global.getSettings().getSprite("fx", "fx_smoke_tendril_001");
    public final SpriteAPI fx_smoke_tendril_002 = Global.getSettings().getSprite("fx", "fx_smoke_tendril_002");
    public final SpriteAPI fx_smoke_tendril_003 = Global.getSettings().getSprite("fx", "fx_smoke_tendril_003");
	
    public final SpriteAPI fx_smoke_puff_001 = Global.getSettings().getSprite("fx", "fx_smoke_puff_001");
    public final SpriteAPI fx_smoke_puff_002 = Global.getSettings().getSprite("fx", "fx_smoke_puff_002");
    public final SpriteAPI fx_smoke_puff_003 = Global.getSettings().getSprite("fx", "fx_smoke_puff_003");	

	public final SpriteAPI fx_shrapnel_001 = Global.getSettings().getSprite("fx", "fx_shrapnel_001");
	
    public final SpriteAPI fx_generic_pulse = Global.getSettings().getSprite("fx", "fx_generic_pulse");

    @Override
    public void init(CombatEngineAPI cEngine) {
		engine = cEngine;
    }

    @Override
    public void advance(float amount, List events) {
		// Obvious exploits are obvious
		if (engine == null) return;
		if (!engine.isInCampaign() && !engine.isInCampaignSim() && !engine.isMission() && !engine.isSimulation()) {
			return;
		}
		if (engine.isPaused()) return;

		//MISSILE EXPLOSION SYSTEM
		// get the missiles in this frame
		HashSet<MissileAPI> activeMissiles = new HashSet<>();
		activeMissiles.addAll(engine.getMissiles());

		// the difference between the missiles in the last frame to this frame are those that *could* have done damage
		for (MissileAPI missile : fx_SharedLib.seenMissiles) {
			if (fx_SharedLib.deadShotList.contains(missile)) continue;
			if(missile.getWeapon() == null) continue;
			if(missile.getWeapon().getShip() == null) continue;

			if (missile.didDamage()) {
				int damageType;
				switch (missile.getDamageType()) {
					case ENERGY:
						damageType = 0;
						break;
					case HIGH_EXPLOSIVE:
						damageType = 1;
						break;
					case KINETIC:
						damageType = 2;
						break;
					case FRAGMENTATION:
						damageType = 3;
						break;
					default:
						damageType = 0;
						break;
				}
				int power = Math.min(200, Math.max(1, Math.round(missile.getWeapon().getDerivedStats().getDamagePerShot() / 5)));
				doProjectileExplosion(missile, null, power, power > 3, damageType);
				fx_SharedLib.deadShotList.add(missile);
			}
		}

		// set our missiles for the next frame
		fx_SharedLib.seenMissiles = activeMissiles;

		//BEAM FX SYSTEM
		for (BeamAPI beam : engine.getBeams()) {
			if (beam.getWeapon() == null) continue;
			if (beam.getWeapon().getShip() == null) continue;
			if (beam.getDamageTarget() == null) continue;

			//Generates a bit of "spark" when the Beam strikes something.
			if (beam.didDamageThisFrame() && beam.getBrightness() > 0.99f) {
				float angle = VectorUtils.getAngle(beam.getFrom(), beam.getTo());
				int power = Math.min(5, Math.max(1, Math.round(beam.getWeapon().getDerivedStats().getDps() / 75)));
				boolean didEffect = false;
				CombatEntityAPI target = beam.getDamageTarget();
				if (target instanceof ShipAPI) {
					ShipAPI tShip = (ShipAPI) target;
					if (tShip.getShield() != null) {
						if (tShip.getShield().isOn() && fx_SharedLib.isWithinShieldArc(tShip, beam.getTo())) {
							didEffect = true;

							if (beam.getWeapon().isBurstBeam() || getRandomNumberInRange(0f, 10f) > 9f) {
							power = Math.min(400, Math.max(1, Math.round(beam.getWeapon().getDerivedStats().getDamagePerShot())));
							float newAngle = VectorUtils.getAngle(tShip.getShield().getLocation(), beam.getTo()) + getRandomNumberInRange(-45f, 45f);
							float vel = getRandomNumberInRange(0, 3f);
							float size = getRandomNumberInRange(50f + (float) power, 75f + (float) power);
							float growth = getRandomNumberInRange(0.995f, 1.0f);
							float lifeTime = getRandomNumberInRange(0.1f, 0.15f);
							float fadeTime = getRandomNumberInRange(0.1f, 0.15f);
							Color projColor;
							Color projColorFade;
							if (!fx_SharedLib.projCoreColorMap.containsKey(beam.getWeapon().getId()) || !fx_SharedLib.projFringeColorMap.containsKey(beam.getWeapon().getId())) {
								Color[] bColors = fx_SharedLib.getBeamColors(beam);
								projColor = bColors[0];
								projColorFade = bColors[1];
							} else {
								projColor = (Color) fx_SharedLib.projCoreColorMap.get(beam.getWeapon().getId());
								projColorFade = (Color) fx_SharedLib.projFringeColorMap.get(beam.getWeapon().getId());
							}
							Vector2f velocity = fx_SharedLib.circularVelocity(newAngle, vel);
							Vector2f targetVel = beam.getWeapon().getShip().getVelocity();
							velocity = velocity.translate(targetVel.getX(), targetVel.getY());
							int choice = MathUtils.getRandomNumberInRange(0, 2);
							fx_SharedLib.newParticle(beam.getTo(), velocity, size * 2f, 1.0f, newAngle, true, true, returnShieldFlash(choice), projColor, projColorFade, lifeTime, fadeTime, growth);
							}
						}
					}
				}

				if (target instanceof MissileAPI || target instanceof CombatAsteroidAPI || !didEffect) {
					for (int i = 0; i < power; i++) {
						if (getRandomNumberInRange(0f, 10f) > 5f) {
							float newAngle = angle - 180f + MathUtils.getRandomNumberInRange(-20f, 20f);
							float vel = getRandomNumberInRange(200f + (power * 100f), 400f + (power * 100f));
							float size = getRandomNumberInRange(10f + (float) power * 3f, 20f + (float) power * 3f);
							float growth = getRandomNumberInRange(0.995f, 1.0f);
							float ratio = 1.0f;
							float lifeTime = getRandomNumberInRange(0.1f, 0.3f);
							float fadeTime = getRandomNumberInRange(0.1f, 0.3f);
							Vector2f velocity = fx_SharedLib.circularVelocity(newAngle, vel);
							Vector2f targetVel = beam.getDamageTarget().getVelocity();
							velocity = velocity.translate(targetVel.getX(), targetVel.getY());
							fx_SharedLib.newParticle(beam.getTo(), velocity, size, ratio, newAngle, true, true, fx_generic_pulse, sparkColor, fadeSpark, lifeTime, fadeTime, growth);
						}
					}
				}
			}

			//Generates a "muzzle flash" for a Beam that's active
			if (beam.getBrightness() > 0.5f && getRandomNumberInRange(0f, 10f) > 5f) {
				if (beam.getWeapon() == null) {
					continue;
				}
				if (beam.getWeapon().getShip() == null) {
					continue;
				}
				int power = Math.min(200, Math.max(1, Math.round(beam.getWeapon().getDerivedStats().getDamagePerShot() / 15)));
				float newAngle = VectorUtils.getAngle(beam.getFrom(), beam.getTo());
				float vel = getRandomNumberInRange(10, 15f);
				float size = getRandomNumberInRange(100f + (float) power, 125f + (float) power);
				float growth = getRandomNumberInRange(0.8f, 0.9f);
				float mult;
				float lifeTime = getRandomNumberInRange(0.1f, 0.15f);
				float fadeTime = getRandomNumberInRange(0.1f, 0.15f);
				Color projColor;
				Color projColorFade;
				if (!fx_SharedLib.projCoreColorMap.containsKey(beam.getWeapon().getId()) || !fx_SharedLib.projFringeColorMap.containsKey(beam.getWeapon().getId())) {
					Color[] bColors = fx_SharedLib.getBeamColors(beam);
					projColor = bColors[0];
					projColorFade = bColors[1];
					mult = (float)fx_SharedLib.projSizeMap.get(beam.getWeapon().getId());
				} else {
					projColor = (Color) fx_SharedLib.projCoreColorMap.get(beam.getWeapon().getId());
					projColorFade = (Color) fx_SharedLib.projFringeColorMap.get(beam.getWeapon().getId());
					mult = (float)fx_SharedLib.projSizeMap.get(beam.getWeapon().getId());
				}
				Vector2f velocity = fx_SharedLib.circularVelocity(newAngle, vel);
				Vector2f targetVel = beam.getWeapon().getShip().getVelocity();
				velocity = velocity.translate(targetVel.getX(), targetVel.getY());
				int choice = MathUtils.getRandomNumberInRange(0, 2);
				fx_SharedLib.newParticle(beam.getFrom(), velocity, size * mult, 1.0f, newAngle, true, true, returnLaserFlash(choice), projColor, projColorFade, lifeTime, fadeTime, growth);
			}
		}

		//Handles explosions and muzzle-flashes for projectiles.
		for (DamagingProjectileAPI projectile : engine.getProjectiles()) {
			if (fx_SharedLib.deadShotList.contains(projectile)) continue;
			//Checks if this Projectile is a MissileAPI.
			//If not a brand-new Missile, then it doesn't get handled via projectileHandler()
			if (projectile instanceof MissileAPI) {
				if (projectile.getElapsed() < 0.01f) {
					projectileHandler(projectile);
				}
			} else {
				projectileHandler(projectile);
			}
		}
    }

	//PROJECTILE HANDLER
    public void projectileHandler(DamagingProjectileAPI projectile) {
		if (projectile.getWeapon() == null) return;
		if (projectile.getWeapon().getShip() == null) return;

		CombatEntityAPI dTarg = projectile.getDamageTarget();

		//PROJECTILE EXPLOSION HANDLER
		if (dTarg != null) {
			int damageType;
			switch (projectile.getDamageType()) {
				case ENERGY:
					damageType = 0;
					break;
				case HIGH_EXPLOSIVE:
					damageType = 1;
					break;
				case KINETIC:
					damageType = 2;
					break;
				case FRAGMENTATION:
					damageType = 3;
					break;
				default:
					damageType = 0;
					break;
			}
			int power = Math.min(200, Math.max(1, Math.round(projectile.getWeapon().getDerivedStats().getDamagePerShot() / 15)));
			doProjectileExplosion(projectile, dTarg, power, power > 3, damageType);
			fx_SharedLib.deadShotList.add(projectile);
		}

		//MUZZLE-FLASH HANDLER
		if (projectile.getElapsed() < 0.01f) {
			int power = Math.min(200, Math.max(1, Math.round(projectile.getWeapon().getDerivedStats().getDamagePerShot() / 15)));
			float newAngle = projectile.getFacing();
			float vel = getRandomNumberInRange(5, 10f);
			float size = getRandomNumberInRange(50f + (float) power, 100f + (float) power);
			float growth = getRandomNumberInRange(0.8f, 0.9f);
			float mult;
			float lifeTime = getRandomNumberInRange(0.1f, 0.3f);
			float fadeTime = getRandomNumberInRange(0.1f, 0.15f);

			Color projColor;
			Color projColorFade;
			if (!fx_SharedLib.projCoreColorMap.containsKey(projectile.getProjectileSpecId()) || !fx_SharedLib.projFringeColorMap.containsKey(projectile.getProjectileSpecId())) {
				Color[] bColors = fx_SharedLib.getProjectileColors(projectile);
				projColor = bColors[0];
				projColorFade = bColors[1];
				mult = (float)fx_SharedLib.projSizeMap.get(projectile.getWeapon().getId());
			} else {
				projColor = (Color) fx_SharedLib.projCoreColorMap.get(projectile.getProjectileSpecId());
				projColorFade = (Color) fx_SharedLib.projFringeColorMap.get(projectile.getProjectileSpecId());
				if(fx_SharedLib.projSizeMap.containsKey(projectile.getWeapon().getId())){
					mult = (float)fx_SharedLib.projSizeMap.get(projectile.getWeapon().getId());
				} else {
					mult = 1f;
				}
			}

			Vector2f velocity = fx_SharedLib.circularVelocity(newAngle, vel);
			Vector2f targetVel = projectile.getWeapon().getShip().getVelocity();
			velocity = velocity.translate(targetVel.getX(), targetVel.getY());
			int choice = MathUtils.getRandomNumberInRange(0, 2);
			fx_SharedLib.newParticle(projectile.getLocation(), velocity, size * mult, 1.0f, newAngle, true, true, returnMuzzleFlash(choice), projColor, projColorFade, lifeTime, fadeTime, growth);
		}
    }
	
	public void doGlow(Vector2f projLoc,  Vector2f targetVel, float size, float angle, float facing, float power, int choice, Color projColor, Color projColorFade){
		float vel = getRandomNumberInRange(5, 10f);
		
		float growth = getRandomNumberInRange(1.02f, 1.05f);
		float lifeTime = getRandomNumberInRange(0.1f, 0.3f);
		float fadeTime = getRandomNumberInRange(0.1f, 0.15f);

		Vector2f velocity = fx_SharedLib.circularVelocity(angle, vel);
		velocity = new Vector2f(velocity.getX() + targetVel.getX(), velocity.getY() + targetVel.getY());
		fx_SharedLib.newParticle(projLoc, velocity, size, 1f, angle, true, true, returnMuzzleFlash(choice), projColor, projColorFade, lifeTime, fadeTime, growth);	
	}
	
	public void doHEGlow(Vector2f projLoc,  Vector2f targetVel, float size, float angle, float facing, float power, int choice, Color projColor, Color projColorFade){
		float vel = getRandomNumberInRange(5, 10f);
		
		float growth = getRandomNumberInRange(1.02f, 1.05f);
		float lifeTime = getRandomNumberInRange(0.1f, 0.3f);
		float fadeTime = getRandomNumberInRange(0.1f, 0.15f);

		Vector2f velocity = fx_SharedLib.circularVelocity(angle, vel);
		velocity = new Vector2f(velocity.getX() + targetVel.getX(), velocity.getY() + targetVel.getY());
		fx_SharedLib.newParticle(projLoc, velocity, size, 1f, angle, true, true, returnSmoke(choice), projColor, projColorFade, lifeTime, fadeTime, growth);	
	}	
	
	public void kineticGlow(Vector2f projLoc,  Vector2f targetVel, float size, float angle, float facing, float power, int choice, Color projColor, Color projColorFade){
		float vel = getRandomNumberInRange(10f, 20f);
		
		float growth = 1.05f;
		float lifeTime = getRandomNumberInRange(0.5f, 0.6f);
		float fadeTime = getRandomNumberInRange(0.1f, 0.15f);

		Vector2f velocity = fx_SharedLib.circularVelocity(angle, vel);
		velocity = new Vector2f(velocity.getX() + targetVel.getX(), velocity.getY() + targetVel.getY());
		fx_SharedLib.newParticle(projLoc, velocity, size, 1f, angle, true, true, fx_shrapnel_001, projColor, projColorFade, lifeTime, fadeTime, growth);	
	}	
	
	public void shrapnelGlow(Vector2f projLoc,  Vector2f targetVel, float size, float angle, float facing, float power, int choice, Color projColor, Color projColorFade){
		float vel = getRandomNumberInRange(45f, 75f);
		
		float growth = 1.03f;
		float lifeTime = getRandomNumberInRange(0.1f, 0.3f);
		float fadeTime = getRandomNumberInRange(0.1f, 0.15f);

		Vector2f velocity = fx_SharedLib.circularVelocity(angle, vel);
		fx_SharedLib.newParticle(projLoc, velocity, size, 1f, angle, true, true, fx_shrapnel_001, projColor, projColorFade, lifeTime, fadeTime, growth);	
	}		
	
	public void doSmoke(Vector2f projLoc,  Vector2f targetVel, float size, float angle, float facing, float power, float growth, float lifeTime, float fadeTime, int choice, Color projColor, Color projColorFade){
		float vel = getRandomNumberInRange(5, 10f);
		
		Vector2f velocity = fx_SharedLib.circularVelocity(angle, vel);
		velocity = new Vector2f(velocity.getX() + targetVel.getX(), velocity.getY() + targetVel.getY());
		fx_SharedLib.newParticle(projLoc, velocity, size, 1f, angle, false, false, returnSmoke(choice), smokeColor, smokeColor, lifeTime, fadeTime, growth);	
	}

	public void doSmokePuff(Vector2f projLoc,  Vector2f targetVel, float size, float angle, float facing, float power, float growth, float lifeTime, float fadeTime, int choice, Color projColor, Color projColorFade){
		float vel = getRandomNumberInRange(5, 10f);
		
		Vector2f velocity = fx_SharedLib.circularVelocity(angle, vel);
		fx_SharedLib.newParticle(projLoc, velocity, size, 1f, angle, false, false, returnSmokePuff(choice), smokeColor, smokeColor, lifeTime, fadeTime, growth);	
	}	

    public void doProjectileExplosion(DamagingProjectileAPI proj, CombatEntityAPI targ, int power, boolean doSmoke, int damageType) {
		int repeats = Math.min(7, Math.max(1, power / 5) + MathUtils.getRandomNumberInRange(0, 2));
		float facing = proj.getFacing();

		Vector2f targetVel;
		if (targ == null) {
			targetVel = new Vector2f(0.0f, 0.0f);
		} else {
			targetVel = targ.getVelocity();
		}

		Color projColor;
		Color projColorFade;
		if (!fx_SharedLib.projCoreColorMap.containsKey(proj.getProjectileSpecId()) || !fx_SharedLib.projFringeColorMap.containsKey(proj.getProjectileSpecId())) {
			Color[] bColors = fx_SharedLib.getProjectileColors(proj);
			projColor = bColors[0];
			projColorFade = bColors[1];
		} else {
			projColor = (Color) fx_SharedLib.projCoreColorMap.get(proj.getProjectileSpecId());
			projColorFade = (Color) fx_SharedLib.projFringeColorMap.get(proj.getProjectileSpecId());
		}


			switch (damageType) {
				case 0:
					//ENERGY Does a glow effect, no smoke
					//GLOW EFFECT
					for (int i = 0; i < repeats; i++) {
						float angle = facing - 180f + getRandomNumberInRange(-35f, 35f);
						float size = getRandomNumberInRange(10f + (float) power, 20f + (float) power);
						int choice = MathUtils.getRandomNumberInRange(0, 2);				
						doGlow(proj.getLocation(),  targetVel, size, angle, facing, power, choice, projColor, projColorFade);
					}
					break;
				case 1:
					//HIGH_EXPLOSIVE does a smoke-tendril effect, layering the glow on the smoke, using the same angle and size
					float smokeGrowth = getRandomNumberInRange(1.001f, 1.003f);
					float smokeLifeTime = Math.max(1.0f,Math.min(3f, getRandomNumberInRange(0.1f, 0.3f) * power / 10f));
					float smokeFadeTime = Math.max(0.1f,Math.min(1f, getRandomNumberInRange(0.1f, 0.15f) * power / 10f));
					for (int i = 0; i < repeats; i++) {
						float angle = facing - 180f + getRandomNumberInRange(-35f, 35f);
						float size = getRandomNumberInRange(10f + (float) power, 20f + (float) power);
						int choice = MathUtils.getRandomNumberInRange(0, 2);							
						//GLOW EFFECT
						doHEGlow(proj.getLocation(),  targetVel, size, angle, facing, power, choice, projColor, projColorFade);
						//SMOKE EFFECT
						doSmoke(proj.getLocation(),  targetVel, size, angle, facing, power, smokeGrowth, smokeLifeTime, smokeFadeTime, choice,  smokeColor, smokeColor);						
					}
					break;
				case 2:
					//KINETIC does a spike effect
					for (int i = 0; i < repeats; i++) {
						float angle = facing - 180f + getRandomNumberInRange(-35f, 35f);
						float kangle = facing - 180f + getRandomNumberInRange(-15f, 15f);
						float size = getRandomNumberInRange(10f + (float) power, 20f + (float) power);
						float ksize = Math.min(20f,Math.max(10f, size));
						int choice = MathUtils.getRandomNumberInRange(0, 2);
						//int choice = MathUtils.getRandomNumberInRange(0, 2);		
						//GLOW EFFECT
						doGlow(proj.getLocation(),  targetVel, size, angle, facing, power, choice, projColor, projColorFade);
						kineticGlow(proj.getLocation(),  targetVel, ksize, kangle, facing, power, 0, projColor, projColorFade);
					}
					break;
				case 3:
					//FRAGMENTATION does a burst effect
					float frsmokeGrowth = getRandomNumberInRange(1.003f, 1.005f);
					float frsmokeLifeTime = Math.max(1.0f,Math.min(3f, getRandomNumberInRange(0.1f, 0.3f) * power / 10f));
					float frsmokeFadeTime = Math.max(0.1f,Math.min(1f, getRandomNumberInRange(0.1f, 0.15f) * power / 10f));					
					for (int i = 0; i < repeats; i++) {
						float angle = getRandomNumberInRange(0, 360f);
						float size = getRandomNumberInRange(10f + (float) power, 20f + (float) power);
						float ksize = Math.min(30f,Math.max(10f, size/2f));
						int choice = MathUtils.getRandomNumberInRange(0, 2);	
						//GLOW EFFECT
						kineticGlow(proj.getLocation(),  targetVel, ksize, angle, facing, power, 0, projColor, projColorFade);
						//SMOKE EFFECT
						doSmokePuff(proj.getLocation(),  targetVel, size*2f, angle, facing, power, frsmokeGrowth, frsmokeLifeTime, frsmokeFadeTime, choice,  smokeColor, smokeColor);							
					}
					break;
				default:
					//Does a generic glow burst effect
					for (int i = 0; i < repeats; i++) {
						float angle = facing + 180 + getRandomNumberInRange(-35f, 35f);
						float size = getRandomNumberInRange(10f + (float) power, 20f + (float) power);
						int choice = MathUtils.getRandomNumberInRange(0, 2);	
						//GLOW EFFECT
						doGlow(proj.getLocation(),  targetVel, size, angle, facing, power, choice, projColor, projColorFade);
					}
					break;
				}

    }

    public SpriteAPI returnSmoke(int choice) {
		switch (choice) {
			case 0:
				return fx_smoke_tendril_001;
			case 1:
				return fx_smoke_tendril_002;
			case 2:
				return fx_smoke_tendril_003;
			default:
				return fx_smoke_tendril_001;
		}
    }
	
    public SpriteAPI returnSmokePuff(int choice) {
		switch (choice) {
			case 0:
				return fx_smoke_puff_001;
			case 1:
				return fx_smoke_puff_002;
			case 2:
				return fx_smoke_puff_003;
			default:
				return fx_smoke_puff_001;
		}
    }	

    public SpriteAPI returnShieldFlash(int choice) {
		switch (choice) {
			case 0:
				return fx_shieldflash_001;
			case 1:
				return fx_shieldflash_002;
			case 2:
				return fx_shieldflash_003;
			default:
				return fx_shieldflash_001;
		}
    }

    public SpriteAPI returnLaserFlash(int choice) {
		switch (choice) {
			case 0:
				return fx_laserflash_001;
			case 1:
				return fx_laserflash_002;
			case 2:
				return fx_laserflash_003;
			default:
				return fx_laserflash_001;
		}
    }

    public SpriteAPI returnMuzzleFlash(int choice) {
		switch (choice) {
			case 0:
				return fx_muzzleflash_001;
			case 1:
				return fx_muzzleflash_002;
			case 2:
				return fx_muzzleflash_003;
			default:
				return fx_muzzleflash_001;
		}
    }

	@Override
    public void renderInWorldCoords(ViewportAPI vapi) {
    }

    @Override
    public void renderInUICoords(ViewportAPI vapi) {
    }
}
