package data.scripts.plugins;

import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.EveryFrameCombatPlugin;
import com.fs.starfarer.api.combat.ViewportAPI;

import data.scripts.fx_Particle;
import data.scripts.fx_SharedLib;
import static data.scripts.fx_SharedLib.clearMe;
import static data.scripts.fx_SharedLib.handleParticle;
import java.util.ArrayList;

import java.util.List;
import java.util.Map;
import org.lazywizard.lazylib.MathUtils;
import static org.lwjgl.opengl.GL11.*;
import org.lwjgl.util.vector.Vector2f;

public class FX_Plugin implements EveryFrameCombatPlugin {

    CombatEngineAPI engine;

    @Override
    public void init(CombatEngineAPI cEngine) {
		engine = cEngine;
		clearMe();
    }

    @Override
    public void advance(float amount, List events) {
		// Obvious exploits are obvious
		if (engine == null) {
			clearMe();
			return;
		}
		if (engine.isPaused()) {
			fx_SharedLib.frameTime = 0f;
			return;
		}
		if (!engine.isInCampaign() && !engine.isInCampaignSim() && !engine.isMission() && !engine.isSimulation()) {
			return;
		}
		
		//Self-regulating code, to cut down on particle generation dynamically if we're having framerate issues.
		if(amount > 0.025f && amount < 0.033f){
			fx_SharedLib.MAX_PARTICLES = Math.min(1000, fx_SharedLib.MAX_PARTICLES);
			fx_SharedLib.MAX_ADDITIVE_PARTICLES = Math.min(1000, fx_SharedLib.MAX_PARTICLES);
		} else if(amount >= 0.033f && amount < 0.067f) {
			fx_SharedLib.MAX_PARTICLES = Math.min(500, fx_SharedLib.MAX_PARTICLES);
			fx_SharedLib.MAX_ADDITIVE_PARTICLES = Math.min(500, fx_SharedLib.MAX_PARTICLES);			
		} else if(amount >= 0.067f){
			fx_SharedLib.MAX_PARTICLES = Math.min(250, fx_SharedLib.MAX_PARTICLES);
			fx_SharedLib.MAX_ADDITIVE_PARTICLES = Math.min(250, fx_SharedLib.MAX_PARTICLES);			
		} else {
			fx_SharedLib.MAX_PARTICLES = fx_SharedLib.ORIGINAL_MAX_PARTICLES;
			fx_SharedLib.MAX_ADDITIVE_PARTICLES = fx_SharedLib.ORIGINAL_MAX_PARTICLES;		
		}

		fx_SharedLib.frameTime = amount;
		if (engine.getTimeMult().getModifiedValue() != 1f) {
			fx_SharedLib.frameTime /= engine.getTimeMult().getModifiedValue();
		}

		//Kills dead particles; fast boolean checker
		for (Object value : fx_SharedLib.particleList.values()) {
			ArrayList thisList = (ArrayList) value;
			for (int i = 0; i < thisList.size(); i++) {
				fx_Particle particle = (fx_Particle) thisList.get(i);
				if(particle.isDead() == true){
					thisList.remove(i);
					i--;
				}
			}			
		}
		for (Object value : fx_SharedLib.additiveParticleList.values()) {
			ArrayList thisList = (ArrayList) value;
			for (int i = 0; i < thisList.size(); i++) {
				fx_Particle particle = (fx_Particle) thisList.get(i);
				if(particle.isDead() == true){
					thisList.remove(i);
					i--;
				}
			}			
		}
    }

    @Override
    public void renderInWorldCoords(ViewportAPI vapi) {
		if (engine == null) return;
		if (!engine.isInCampaign() && !engine.isInCampaignSim() && !engine.isMission() && !engine.isSimulation()) {
			return;
		}
		
		//Sets the centerPoint variable that's used to keep off-screen particle generation low.
		fx_SharedLib.centerPoint = vapi.getCenter();

		//OpenGL Start
		glEnable(GL_BLEND);
		glEnable(GL_TEXTURE_2D);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		
		for (Map.Entry<Integer, ArrayList<fx_Particle>> entry : fx_SharedLib.particleList.entrySet()) {
			int key = entry.getKey();
			glBindTexture(GL_TEXTURE_2D, key);
			glBegin(GL_QUADS);
			for(fx_Particle particle : entry.getValue()){
				if (particle.isDead()) continue;
				handleParticle(particle);				
				float size = particle.size.getX() / 2f;

				glColor4ub((byte)particle.curColor.getRed(),(byte)particle.curColor.getGreen(),(byte)particle.curColor.getBlue(),(byte)particle.curColor.getAlpha());

				glTexCoord2f(0, 1);
				Vector2f vec = MathUtils.getPointOnCircumference(particle.position, size, particle.angle + 315f);
				glVertex2f(vec.getX(),vec.getY());

				glTexCoord2f(0, 0);
				vec = MathUtils.getPointOnCircumference(particle.position, size, particle.angle + 225f);
				glVertex2f(vec.getX(),vec.getY());

				glTexCoord2f(1, 0);
				vec = MathUtils.getPointOnCircumference(particle.position, size, particle.angle + 135f);
				glVertex2f(vec.getX(),vec.getY());

				glTexCoord2f(1, 1);
				vec = MathUtils.getPointOnCircumference(particle.position, size, particle.angle + 45f);
				glVertex2f(vec.getX(),vec.getY());				
			}
			glEnd();
		}
		
		glEnable(GL_BLEND);
		glEnable(GL_TEXTURE_2D);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		for (Map.Entry<Integer, ArrayList<fx_Particle>> entry : fx_SharedLib.additiveParticleList.entrySet()) {
			int key = entry.getKey();
			glBindTexture(GL_TEXTURE_2D, key);
			glBegin(GL_QUADS);
			for(fx_Particle particle : entry.getValue()){
				if (particle.isDead()) continue;
				handleParticle(particle);				
				float size = particle.size.getX() / 2f;

				glColor4ub((byte)particle.curColor.getRed(),(byte)particle.curColor.getGreen(),(byte)particle.curColor.getBlue(),(byte)particle.curColor.getAlpha());

				glTexCoord2f(0, 1);
				Vector2f vec = MathUtils.getPointOnCircumference(particle.position, size, particle.angle + 315f);
				glVertex2f(vec.getX(),vec.getY());

				glTexCoord2f(0, 0);
				vec = MathUtils.getPointOnCircumference(particle.position, size, particle.angle + 225f);
				glVertex2f(vec.getX(),vec.getY());

				glTexCoord2f(1, 0);
				vec = MathUtils.getPointOnCircumference(particle.position, size, particle.angle + 135f);
				glVertex2f(vec.getX(),vec.getY());

				glTexCoord2f(1, 1);
				vec = MathUtils.getPointOnCircumference(particle.position, size, particle.angle + 45f);
				glVertex2f(vec.getX(),vec.getY());				
			}
			glEnd();
		}
    }

    @Override
    public void renderInUICoords(ViewportAPI vapi) {
    }
}
