 package data.scripts.plugins;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.ModSpecAPI;
import com.fs.starfarer.api.combat.CollisionClass;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.DamagingProjectileAPI;
import com.fs.starfarer.api.combat.EveryFrameCombatPlugin;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ViewportAPI;
import com.fs.starfarer.api.combat.WeaponAPI;
import com.fs.starfarer.api.graphics.SpriteAPI;

import data.scripts.fx_SharedLib;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;

import java.util.List;
import java.util.Map;
import org.json.JSONObject;

import org.lazywizard.lazylib.MathUtils;
import org.lwjgl.util.vector.Vector2f;

public class AAA_Thruster_FX_Plugin implements EveryFrameCombatPlugin {

    CombatEngineAPI engine;

    //FX Sprites
    public final SpriteAPI fx_engineflash_001 = Global.getSettings().getSprite("fx", "fx_engineflash_001");
    public final SpriteAPI fx_engineflash_002 = Global.getSettings().getSprite("fx", "fx_engineflash_002");
    public final SpriteAPI fx_engineflash_003 = Global.getSettings().getSprite("fx", "fx_engineflash_003");

    public final SpriteAPI fx_jetthruster_001 = Global.getSettings().getSprite("fx", "fx_jetthruster_001");
    public final SpriteAPI fx_jetthruster_002 = Global.getSettings().getSprite("fx", "fx_jetthruster_002");
    public final SpriteAPI fx_jetthruster_003 = Global.getSettings().getSprite("fx", "fx_jetthruster_003");

    public final SpriteAPI fx_techengine_001 = Global.getSettings().getSprite("fx", "fx_techengine_001");
    public final SpriteAPI fx_techengine_002 = Global.getSettings().getSprite("fx", "fx_techengine_002");
    public final SpriteAPI fx_techengine_003 = Global.getSettings().getSprite("fx", "fx_techengine_003");

    public final SpriteAPI fx_electro_001 = Global.getSettings().getSprite("fx", "fx_electro_001");
    public final SpriteAPI fx_electro_002 = Global.getSettings().getSprite("fx", "fx_electro_002");
    public final SpriteAPI fx_electro_003 = Global.getSettings().getSprite("fx", "fx_electro_003");

	public final SpriteAPI fx_enginestable_001 = Global.getSettings().getSprite("fx", "fx_enginestable_001");
	public final SpriteAPI fx_enginestable_002 = Global.getSettings().getSprite("fx", "fx_enginestable_002");
	public final SpriteAPI fx_enginestable_003 = Global.getSettings().getSprite("fx", "fx_enginestable_003");

	public final Map missileCenterMap = new HashMap();
	public final Map missileSizeMap = new HashMap();

	public final Map missileCoreColorMap = new HashMap();
	public final Map missileFringeColorMap = new HashMap();

	public final Map engineCoreColorMap = new HashMap();
	public final Map engineFringeColorMap = new HashMap();
	public final Map engineStyleMap = new HashMap();

	public final List<String> shipsNoEngines = new ArrayList();
	public final List<String> missilesNoEngines = new ArrayList();
	public final List<String> flares = new ArrayList();

    @Override
    public void init(CombatEngineAPI cEngine) {
		engine = cEngine;
    }

    @Override
    public void advance(float amount, List events) {
		// Obvious exploits are obvious
		if (engine == null) return;
		if (!engine.isInCampaign() && !engine.isInCampaignSim() && !engine.isMission() && !engine.isSimulation()) {
			return;
		}
		if(engine.isPaused()) return;
		if (fx_SharedLib.projCoreColorMap == null) return;

		//THRUSTER FX SYSTEM
		for (ShipAPI ship : engine.getShips()) {
			//Don't run this on Hulks, obviously
			if (ship.isHulk()) continue;
			if(ship.getEngineController().isFlamedOut()) continue;
			if(shipsNoEngines.contains(ship.getHullSpec().getBaseHullId())) continue;
			float shipSpeed = Math.abs(ship.getVelocity().length());
			float speedScalar = shipSpeed / ship.getMutableStats().getMaxSpeed().getBaseValue();
			boolean limiter = false;
			if(ship.getTravelDrive().isActive()){
				speedScalar = Math.max(1f, Math.min(speedScalar, 3.5f));
				limiter = true;
			}
			if(!limiter && ship.getSystem() != null){
					if(ship.getSystem().isActive()){
						speedScalar = Math.max(1f, Math.min(speedScalar, 2.5f));
						limiter = true;
					}
			}	
			if(!limiter && ship.getFluxTracker().isEngineBoostActive()){
				speedScalar = Math.max(0.5f, Math.min(speedScalar, 1.75f));
				limiter = true;				
			}
			if(!limiter){
				speedScalar = Math.max(0.25f, Math.min(speedScalar, 1.5f));
			}

			//This should probably get cleaned up into a HashMap or ArrayList for speed, later
			List<WeaponAPI> weapons = ship.getAllWeapons();
			//Checks all Weapons, looks for a Thruster, makes stuff happen if we're accelerating.
			boolean noEngines = true;
			int maxEngines = 0;
			int goodEngines = 0;
			for (WeaponAPI weapon : weapons) {
				if (weapon.getId().equals("thruster_fighter_sm")) {
					ship.removeWeaponFromGroups(weapon);
					noEngines = false;
					maxEngines += 1;
					if(weapon.getCurrHealth() > 0f){
						goodEngines += 1;
						String engineStyle;
						Color projColor;
						Color projColorFade;
						//Do we have the engine colors for this Ship?  No?  Run expensive code.
						if (!engineCoreColorMap.containsKey(ship.getHullSpec().getBaseHullId()) || !engineFringeColorMap.containsKey(ship.getHullSpec().getBaseHullId())) {
							Color[] bColors = getEngineColors(ship);
							projColor = bColors[0];
							projColorFade = bColors[1];
							engineStyle = (String) getEngineStyle(ship);
						} else {
							projColor = (Color) engineCoreColorMap.get(ship.getHullSpec().getBaseHullId());
							projColorFade = (Color) engineFringeColorMap.get(ship.getHullSpec().getBaseHullId());
							engineStyle = (String) engineStyleMap.get(ship.getHullSpec().getBaseHullId());
						}

						//Base size of engines after we apply arc as a multiplier
						boolean accelerating = false;
						float size = MathUtils.getRandomNumberInRange(25f, 45f);

						//Are we accelerating?  If yes, make the flame bigger.
						if (!ship.isFighter()) {
							if (ship.getEngineController().isAccelerating()) {
								accelerating = true;
								size *= speedScalar;
							}
						} else {
							ShipAPI leader = ship.getWingLeader();
							if(leader != null){
								if(leader.getEngineController() != null){
									if (leader.getEngineController().isAccelerating()) {
										accelerating = true;
										size *= speedScalar;
									}
								}
							}
						}
						if(ship.getCollisionClass().equals(CollisionClass.NONE)){
							int red = projColor.getRed() / 2;
							int blue = projColor.getBlue() / 2;
							int green = projColor.getGreen() / 2;
							projColor = new Color(red,green,blue,255);
							red = projColorFade.getRed() / 2;
							blue = projColorFade.getBlue() / 2;
							green = projColorFade.getGreen() / 2;							
							projColorFade = new Color(red,green,blue,255);
						}
						//This one line controls final size for the engines, treating the arc as a multiplier.
						size *= weapon.getArc();

						if(accelerating){
							int choice = MathUtils.getRandomNumberInRange(0, 2);
							Vector2f vel;
							switch (engineStyle){
								case "electro":
									vel = new Vector2f(ship.getVelocity().getX() * 0.25f, ship.getVelocity().getY() * 0.25f);
									fx_SharedLib.newParticle(weapon.getLocation(), vel, size * 3f, 1f, ship.getFacing() + weapon.getArcFacing(), true, true, returnElectroFlash(choice), projColor, projColorFade, 0.05f, 0.05f, 1f);
									break;
								case "techEngine":
									vel = new Vector2f(ship.getVelocity().getX() * 0.25f, ship.getVelocity().getY() * 0.25f);
									fx_SharedLib.newParticle(weapon.getLocation(), vel, size * 4f, 1f, ship.getFacing() + weapon.getArcFacing(), true, true, returnTechEngineFlash(choice), projColor, projColorFade, 0.05f, 0.05f, 1f);
									break;
								case "jetThruster":
									vel = new Vector2f(ship.getVelocity().getX() * 0.25f, ship.getVelocity().getY() * 0.25f);
									fx_SharedLib.newParticle(weapon.getLocation(), vel, size * 4f, 1f, ship.getFacing() + weapon.getArcFacing(), true, true, returnJetThrusterFlash(choice), projColor, projColorFade, 0.1f, 0.05f, 0.999f);
									break;
								default:
									vel = new Vector2f(ship.getVelocity().getX() * 0.25f, ship.getVelocity().getY() * 0.25f);
									fx_SharedLib.newParticle(weapon.getLocation(), vel, size * 2f, 1f, ship.getFacing() + weapon.getArcFacing(), true, true, returnEngineFlash(choice), projColor, projColorFade, 0.1f, 0.05f, 0.999f);
									break;
							}

						} else {
							int choice = MathUtils.getRandomNumberInRange(0, 2);
							Vector2f vel = new Vector2f(ship.getVelocity().getX() * 0.25f, ship.getVelocity().getY() * 0.25f);
							fx_SharedLib.newParticle(weapon.getLocation(), vel, size * 2f, 1f, ship.getFacing() + weapon.getArcFacing(), true, true, returnEngineStable(choice), projColor, projColorFade, 0.1f, 0.05f, 0.999f);
						}
					}
				}
			}
			if(noEngines){
				shipsNoEngines.add(ship.getHullSpec().getBaseHullId());
			} else {
				if(goodEngines < maxEngines){
					float enginePercent = (float) goodEngines / maxEngines;
					ship.getMutableStats().getTurnAcceleration().modifyMult("engines_out", enginePercent, "Engines Damaged!");
					ship.getMutableStats().getMaxSpeed().modifyMult("engines_out", enginePercent, "Engines Damaged!");
					ship.getMutableStats().getMaxTurnRate().modifyMult("engines_out", enginePercent, "Engines Damaged!");
					ship.getMutableStats().getAcceleration().modifyMult("engines_out", enginePercent, "Engines Damaged!");
				} else {
					ship.getMutableStats().getTurnAcceleration().unmodify("engines_out");
					ship.getMutableStats().getMaxSpeed().unmodify("engines_out");
					ship.getMutableStats().getMaxTurnRate().unmodify("engines_out");
					ship.getMutableStats().getAcceleration().unmodify("engines_out");
				}
			}
		}

		//MISSILE THRUSTER SYSTEM
		for (MissileAPI missile : engine.getMissiles()) {
			if (fx_SharedLib.deadShotList.contains(missile)) continue;
			if(missilesNoEngines.contains(missile.getProjectileSpecId())) continue;
			if(missile.getWeapon() == null) continue;
			if(missile.getWeapon().getId() == null) continue;
			//Specific behavior for Flares, because they're screwy.
			if(flares.contains(missile.getWeapon().getId())){
				Color missColor;
				Color missColorFade;
				missColor = (Color) missileCoreColorMap.get(missile.getWeapon().getId());
				missColorFade = (Color) missileFringeColorMap.get(missile.getWeapon().getId());

				float size = 20f * (float) missileSizeMap.get(missile.getWeapon().getId());
				Vector2f velocity = new Vector2f(missile.getVelocity().getX()*0.25f,missile.getVelocity().getY()*0.25f);
				int choice = MathUtils.getRandomNumberInRange(0, 2);
				fx_SharedLib.newParticle(missile.getLocation(), velocity, size, 1f, MathUtils.getRandomNumberInRange(0f, 360f), true, true, returnEngineFlash(choice), missColor, missColorFade, 0.1f, 0.05f, 1.001f);

				//Stop here, we're done.
				continue;
			}

			if(missile.getEngineController().isAccelerating()){
				float length = 0f;
				float sizeMult = 1f;
				if(missileCenterMap.containsKey(missile.getWeapon().getId())){
					length = (float) missileCenterMap.get(missile.getWeapon().getId());
					sizeMult = (float) missileSizeMap.get(missile.getWeapon().getId());
				} else {
					try {
						JSONObject json;
						try{
							json = Global.getSettings().loadJSON("data/weapons/proj/" + missile.getProjectileSpecId() + ".proj");
						}catch (Throwable ex){
							try{
								json = Global.getSettings().loadJSON("data/shipsystems/proj/" + missile.getProjectileSpecId() + ".proj");
							} catch (Throwable y){
								//We haven't found the missile's PROJ anywhere, make it safe and exit.
								missilesNoEngines.add(missile.getWeapon().getId());
								continue;
							}
						}
						if(json.has("center")){
							length = (float) json.getJSONArray("center").getDouble(1);
						}
						if(json.has("fxEngineMult")){
							sizeMult = (float)json.getDouble("fxEngineMult");
						}
						missileCenterMap.put(missile.getWeapon().getId(),length);
						missileSizeMap.put(missile.getWeapon().getId(),sizeMult);
					} catch (Throwable ex) {
						//Any other borks?  Exit and don't try this again.
						missilesNoEngines.add(missile.getWeapon().getId());
						continue;
					}
				}
				//Catch bad cases here and throw them out.
				if(length < 0f) {
					fx_SharedLib.deadShotList.add(missile);
					continue;
				}
				Color missColor;
				Color missColorFade;
				if (!missileCoreColorMap.containsKey(missile.getWeapon().getId())) {
					Color[] bColors = getMissileEngineColors(missile);
					missColor = bColors[0];
					missColorFade = bColors[1];
				} else {
					missColor = (Color) missileCoreColorMap.get(missile.getWeapon().getId());
					missColorFade = (Color) missileFringeColorMap.get(missile.getWeapon().getId());
				}
				//Used a second "catch" here to make sure that the missile won't attempt to run if it doesn't have an engine defined.
				if(missilesNoEngines.contains(missile.getProjectileSpecId())) continue;

				Vector2f velocity = new Vector2f(missile.getVelocity().getX()*0.25f,missile.getVelocity().getY()*0.25f);
				Vector2f position = MathUtils.getPointOnCircumference(missile.getLocation(), length, missile.getFacing()-180f);
				float size = 10f * sizeMult;
				int choice = MathUtils.getRandomNumberInRange(0, 2);
				fx_SharedLib.newParticle(position, velocity, size, 1f, missile.getFacing()+180f, true, true, returnEngineFlash(choice), missColor, missColorFade, 0.1f, 0.05f, 1.001f);
			}
		}
    }

    public SpriteAPI returnEngineFlash(int choice) {
		switch (choice) {
			case 0:
				return fx_engineflash_001;
			case 1:
				return fx_engineflash_002;
			case 2:
				return fx_engineflash_003;
			default:
				return fx_engineflash_001;
		}
    }

    public SpriteAPI returnJetThrusterFlash(int choice) {
		switch (choice) {
			case 0:
				return fx_jetthruster_001;
			case 1:
				return fx_jetthruster_002;
			case 2:
				return fx_jetthruster_003;
			default:
				return fx_jetthruster_001;
		}
    }

    public SpriteAPI returnTechEngineFlash(int choice) {
		switch (choice) {
			case 0:
				return fx_techengine_001;
			case 1:
				return fx_techengine_002;
			case 2:
				return fx_techengine_003;
			default:
				return fx_techengine_001;
		}
    }

    public SpriteAPI returnElectroFlash(int choice) {
		switch (choice) {
			case 0:
				return fx_electro_001;
			case 1:
				return fx_electro_002;
			case 2:
				return fx_electro_003;
			default:
				return fx_electro_001;
		}
    }

    public SpriteAPI returnEngineStable(int choice) {
		switch (choice) {
			case 0:
				return fx_enginestable_001;
			case 1:
				return fx_enginestable_002;
			case 2:
				return fx_enginestable_003;
			default:
				return fx_enginestable_001;
		}
    }

	private String getEngineStyle(ShipAPI ship){
		String style = "classic";
		String hullStyle = "LOW_TECH";

		JSONObject json;
		JSONObject secondJson;
		JSONObject subJson;

		try{
			json = Global.getSettings().loadJSON("data/hulls/" + ship.getHullSpec().getBaseHullId() + ".ship");
			hullStyle = json.getString("style");
			if(json.has("fxCustomEngineStyle")){
				style = json.getString("fxCustomEngineStyle");
				engineStyleMap.put(ship.getHullSpec().getBaseHullId(), style);
				return style;
			}
		} catch(Throwable ex) {}

		//We've reached the second scenario- the hull has a style reference, but the data wasn't found in the hull.
		//This approach avoids the JSON-masking effects we see when two JSONs with the same name exist.
		boolean foundMod = false;
		for(ModSpecAPI mod : Global.getSettings().getModManager().getAvailableModsCopy()){
			if(foundMod) continue;
			String modID = mod.getId();
			try{
				secondJson = Global.getSettings().loadJSON("data/config/fx_mod_styles.json", modID);
				if(secondJson.has(hullStyle)){
					try{
						subJson = secondJson.getJSONObject(hullStyle);
						if(subJson.has("fxCustomEngineStyle")){
							foundMod = true;
							style = subJson.getString("fxCustomEngineStyle");
							engineStyleMap.put(ship.getHullSpec().getBaseHullId(), style);
							return style;
						}
					} catch(Throwable ex) {}
				}
			} catch(Throwable ex) {}

		}
		engineStyleMap.put(ship.getHullSpec().getBaseHullId(), style);
		return style;
	}

	private Color[] getEngineColors(ShipAPI ship) {
		Color projColor = new Color(255, 128, 0, 255);
		Color projColorFade = new Color(128, 50, 0, 255);

		JSONObject json;
		JSONObject secondJson;
		JSONObject subJson;

		//Direct Path- ship has JSON data in the .ship definition. Also gets the hullStyle, if this fails
		String hullStyle = "LOW_TECH";
		try{
			json = Global.getSettings().loadJSON("data/hulls/" + ship.getHullSpec().getBaseHullId() + ".ship");
			hullStyle = json.getString("style");
			if(json.has("fxCustomEngineCore") && json.has("fxCustomEngineFringe")){
				int redCore;
				int greenCore;
				int blueCore;
				int alphaCore;

				int redFringe;
				int greenFringe;
				int blueFringe;
				int alphaFringe;

				redCore = (int) json.getJSONArray("fxCustomEngineCore").getInt(0);
				greenCore = (int) json.getJSONArray("fxCustomEngineCore").getInt(1);
				blueCore = (int) json.getJSONArray("fxCustomEngineCore").getInt(2);
				alphaCore = (int) 255;

				redFringe = (int) json.getJSONArray("fxCustomEngineFringe").getInt(0);
				greenFringe = (int) json.getJSONArray("fxCustomEngineFringe").getInt(1);
				blueFringe = (int) json.getJSONArray("fxCustomEngineFringe").getInt(2);
				alphaFringe = (int) 255;

				projColor = new Color(redCore, greenCore, blueCore, alphaCore);
				projColorFade = new Color(redFringe, greenFringe, blueFringe, alphaFringe);

				Color[] returnedColor = new Color[2];
				returnedColor[0] = projColor;
				returnedColor[1] = projColorFade;
				engineCoreColorMap.put(ship.getHullSpec().getBaseHullId(), projColor);
				engineFringeColorMap.put(ship.getHullSpec().getBaseHullId(), projColorFade);
				return returnedColor;
			}
		} catch(Throwable ex) {}

		//We've reached the second scenario- the hull has a style reference, but the data wasn't found in the hull.
		//This approach avoids the JSON-masking effects we see when two JSONs with the same name exist.
		boolean foundMod = false;
		for(ModSpecAPI mod : Global.getSettings().getModManager().getAvailableModsCopy()){
			if(foundMod) continue;
			String modID = mod.getId();
			try{
				secondJson = Global.getSettings().loadJSON("data/config/fx_mod_styles.json", modID);
				if(secondJson.has(hullStyle)){
					try{
						subJson = secondJson.getJSONObject(hullStyle);
						if(subJson.has("fxCustomEngineCore") && subJson.has("fxCustomEngineFringe")){
							foundMod = true;
							int redCore;
							int greenCore;
							int blueCore;
							int alphaCore;

							int redFringe;
							int greenFringe;
							int blueFringe;
							int alphaFringe;

							redCore = (int) subJson.getJSONArray("fxCustomEngineCore").getInt(0);
							greenCore = (int) subJson.getJSONArray("fxCustomEngineCore").getInt(1);
							blueCore = (int) subJson.getJSONArray("fxCustomEngineCore").getInt(2);
							alphaCore = (int) 255;

							redFringe = (int) subJson.getJSONArray("fxCustomEngineFringe").getInt(0);
							greenFringe = (int) subJson.getJSONArray("fxCustomEngineFringe").getInt(1);
							blueFringe = (int) subJson.getJSONArray("fxCustomEngineFringe").getInt(2);
							alphaFringe = (int) 255;

							projColor = new Color(redCore,greenCore,blueCore,alphaCore);
							projColorFade = new Color(redFringe,greenFringe,blueFringe,alphaFringe);

							Color[] returnedColor = new Color[2];
							returnedColor[0] = projColor;
							returnedColor[1] = projColorFade;
							engineCoreColorMap.put(ship.getHullSpec().getBaseHullId(), projColor);
							engineFringeColorMap.put(ship.getHullSpec().getBaseHullId(), projColorFade);
							return returnedColor;
						}
					} catch(Throwable ex) {}
				}
			} catch(Throwable ex) {}
		}

		Color[] returnedColor = new Color[2];
		returnedColor[0] = projColor;
		returnedColor[1] = projColorFade;
		engineCoreColorMap.put(ship.getHullSpec().getBaseHullId(), projColor);
		engineFringeColorMap.put(ship.getHullSpec().getBaseHullId(), projColorFade);
		return returnedColor;
    }

    private Color[] getMissileEngineColors(DamagingProjectileAPI projectile) {
		Color projColor = new Color(255, 255, 128, 255);
		Color projColorFade = new Color(255, 64, 0, 255);

		try {
			JSONObject json;
			//Doubled Try-Catch here, to get PROJ data in the two places it can reside.
			try{
				json = Global.getSettings().loadJSON("data/weapons/proj/" + projectile.getProjectileSpecId() + ".proj");
			} catch(Throwable ex) {
				try{
					json = Global.getSettings().loadJSON("data/shipsystems/proj/" + projectile.getProjectileSpecId() + ".proj");
				} catch(Throwable y) {
					//Not defined; don't do anything.
					missilesNoEngines.add(projectile.getProjectileSpecId());

					//We failed to get a missile PROJ in either location here.
					Color[] returnedColor = new Color[2];
					returnedColor[0] = projColor;
					returnedColor[1] = projColorFade;
					missileCoreColorMap.put(projectile.getWeapon().getId(), projColor);
					missileFringeColorMap.put(projectile.getWeapon().getId(), projColorFade);
					return returnedColor;
				}
			}
			//Gets whether this Projectile is a Flare; deals with it accordingly.
			if(json.has("missileType")){
				if(json.getString("missileType").equalsIgnoreCase("FLARE")
					|| json.getString("missileType").equalsIgnoreCase("FLARE_SEEKER")
					|| json.getString("missileType").equalsIgnoreCase("FLARE_JAMMER"))
				{
					flares.add(projectile.getWeapon().getId());
				}
			}

			//Gets custom colors, if defined.
			if(json.has("fxEngineGlowCore") && json.has("fxEngineGlowFringe")){
				int redCore = (int) json.getJSONArray("fxEngineGlowCore").getInt(0);
				int greenCore = (int) json.getJSONArray("fxEngineGlowCore").getInt(1);
				int blueCore = (int) json.getJSONArray("fxEngineGlowCore").getInt(2);
				int alphaCore = (int) 255;

				int redFringe = (int) json.getJSONArray("fxEngineGlowFringe").getInt(0);
				int greenFringe = (int) json.getJSONArray("fxEngineGlowFringe").getInt(1);
				int blueFringe = (int) json.getJSONArray("fxEngineGlowFringe").getInt(2);
				int alphaFringe = (int) 255;

				missileCoreColorMap.put(projectile.getWeapon().getId(), new Color(redCore, greenCore, blueCore, alphaCore));
				missileFringeColorMap.put(projectile.getWeapon().getId(), new Color(redFringe, greenFringe, blueFringe, alphaFringe));
				projColor = new Color(redCore, greenCore, blueCore, alphaCore);
				projColorFade = new Color(redFringe, greenFringe, blueFringe, alphaFringe);
				if (projColor.getRed() + projColor.getGreen() + projColor.getBlue() >= 764) {
					projColor = new Color(projColorFade.getRed(), projColorFade.getGreen(), projColorFade.getBlue(), 255);
				}
				Color[] returnedColor = new Color[2];
				returnedColor[0] = projColor;
				returnedColor[1] = projColorFade;
				//defined = true;
				return returnedColor;
			}
		} catch (Throwable ex) {
			//Not defined; don't do anything.
			missilesNoEngines.add(projectile.getProjectileSpecId());
			Color[] returnedColor = new Color[2];
			returnedColor[0] = projColor;
			returnedColor[1] = projColorFade;
			missileCoreColorMap.put(projectile.getWeapon().getId(), projColor);
			missileFringeColorMap.put(projectile.getWeapon().getId(), projColorFade);
			return returnedColor;
		//Global.getLogger(EZFactionModPlugin.class).log(Level.ERROR,"ERROR: couldn't find the EZ Faction directory, etc.", ex);
		}

		//If we get all the way here somehow, the Missile doesn't have a spec and we need to ignore it from now on.
		//Not defined; don't do anything.
		missilesNoEngines.add(projectile.getProjectileSpecId());
		Color[] returnedColor = new Color[2];
		returnedColor[0] = projColor;
		returnedColor[1] = projColorFade;
		return returnedColor;
    }

    @Override
    public void renderInWorldCoords(ViewportAPI vapi) {
    }

    @Override
    public void renderInUICoords(ViewportAPI vapi) {
    }
}
