//Drains a smallish amount of Hard Flux per second, depending on ship size and total Flux Capacity.
package data.scripts.plugins;

import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.EveryFrameCombatPlugin;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ViewportAPI;
import java.util.List;


public class FluxCapHardFluxDrain implements EveryFrameCombatPlugin
{
	CombatEngineAPI engine;

    @Override
	public void init(CombatEngineAPI cEngine) {
		this.engine = cEngine;
	}
        
    @Override
	public void advance(float amount, List events)
	{
		// Obvious exploits are obvious
		if (engine == null) return;
		if(engine.isPaused()) return;
		if(!engine.isInCampaign() && !engine.isInCampaignSim() && !engine.isMission() && !engine.isSimulation()) return;
		
		float drainFraction = 150f;

		for (ShipAPI ship : engine.getShips()){
			//Skip all the ships that this shouldn't apply to, like ships that are Phased, Venting, etc., etc.
			if(!ship.isAlive() || ship.isPhased() || ship.getFluxTracker().isOverloadedOrVenting() || ship.getHardFluxLevel() <= 0f) continue;
			//If the ship qualifies for a drain, drain now.
			//Start by computing the Drain Threshold, which is a ratio of Capacity vs. a number that results in total drain / second.
			//Small stuff gets a better (lower) number here, because they get significantly less Capacity per OP, etc.
			if(ship.isDestroyer()){
				drainFraction = 250f;
			} else if(ship.isCruiser()) {
				drainFraction = 350f;	
			} else if(ship.isCapital()){
				drainFraction = 500f;
			}
			//Now apply the drainThreshold vs Capacity and time elapsed to get the amount of Hard Flux drain this frame
			drainFraction = (ship.getMutableStats().getFluxCapacity().getModifiedValue() / drainFraction) * engine.getElapsedInLastFrame();
			//Apply time mult here, if that's a thing 
			if(engine.getTimeMult().getModifiedValue() != 1f){
				drainFraction *= engine.getTimeMult().getModifiedValue();
			}
			//How much Hard Flux should we have now?
			if ((ship.getFluxTracker().getHardFlux() - drainFraction) < 0f) {
				ship.getFluxTracker().setHardFlux(0f);
			} else {
				drainFraction = ship.getFluxTracker().getHardFlux() - drainFraction;
				//Set it.
				ship.getFluxTracker().setHardFlux(drainFraction);
			}
		}
	}

	@Override
	public void renderInWorldCoords(ViewportAPI vapi) {
	}

	@Override
	public void renderInUICoords(ViewportAPI vapi) {
	}
}